package cn.iocoder.yudao.module.wealth.dal.mysql.receivable;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.CurrencyAmount;
import cn.iocoder.yudao.module.wealth.vo.receivable.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Set;

/**
 * 应收款 Mapper
 *
 * @author 郑屹
 */
@Mapper
public interface ReceivableMapper extends AbstractMapper<ReceivableDO> {
    @Override
    default PageResult<ReceivableDO> selectPage(PageVO page, Object object) {
        if (object instanceof ReceivableQueryVO) {
            ReceivableQueryVO vo = (ReceivableQueryVO) object;
            return selectPage(page, new LambdaQuery<ReceivableDO>()
                    .eqIfPresent(ReceivableDO::getReceiptId, vo.getReceiptId())
                    .eqIfPresent(ReceivableDO::getReceiptNo, vo.getReceiptNo())
                    .eqIfPresent(ReceivableDO::getOrderId, vo.getOrderId())
                    .eqIfPresent(ReceivableDO::getOrderNo, vo.getOrderNo())
                    .eqIfPresent(ReceivableDO::getTitleEn, vo.getTitleEn())
                    .eqIfPresent(ReceivableDO::getTitleZh, vo.getTitleZh())
                    .eqIfPresent(ReceivableDO::getNum, vo.getNum())
                    .eqIfPresent(ReceivableDO::getWeight, vo.getWeight())
                    .eqIfPresent(ReceivableDO::getVolume, vo.getVolume())
                    .eqIfPresent(ReceivableDO::getFeeType, vo.getFeeType())
                    .eqIfPresent(ReceivableDO::getExchangeRate, vo.getExchangeRate())
                    .eqIfPresent(ReceivableDO::getCurrencyId, vo.getCurrencyId())
                    .eqIfPresent(ReceivableDO::getUnitPrice, vo.getUnitPrice())
                    .eqIfPresent(ReceivableDO::getTotalAmount, vo.getTotalAmount())
                    .eqIfPresent(ReceivableDO::getDiscountTotal, vo.getDiscountTotal())
                    .eqIfPresent(ReceivableDO::getDiscountRemark, vo.getDiscountRemark())
                    .eqIfPresent(ReceivableDO::getIsPayAdvance, vo.getIsPayAdvance())
                    .eqIfPresent(ReceivableDO::getState, vo.getState())
                    .eqIfPresent(ReceivableDO::getSource, vo.getSource())
                    .eqIfPresent(ReceivableDO::getAuthor, vo.getAuthor())
                    .betweenIfPresent(ReceivableDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(ReceivableDO::getId));
        }

        return null;
    }

    @Override
    default List<ReceivableDO> selectList(Object object) {
        if (object instanceof ReceivableQueryVO) {
            ReceivableQueryVO vo = (ReceivableQueryVO) object;
            return selectList(new LambdaQuery<ReceivableDO>()
                    .eqIfPresent(ReceivableDO::getReceiptId, vo.getReceiptId())
                    .eqIfPresent(ReceivableDO::getReceiptNo, vo.getReceiptNo())
                    .eqIfPresent(ReceivableDO::getOrderId, vo.getOrderId())
                    .eqIfPresent(ReceivableDO::getOrderNo, vo.getOrderNo())
                    .eqIfPresent(ReceivableDO::getOrderItemId, vo.getOrderItemId())
                    .eqIfPresent(ReceivableDO::getTitleZh, vo.getTitleZh())
                    .eqIfPresent(ReceivableDO::getTitleEn, vo.getTitleEn())
                    .eqIfPresent(ReceivableDO::getNum, vo.getNum())
                    .eqIfPresent(ReceivableDO::getWeight, vo.getWeight())
                    .eqIfPresent(ReceivableDO::getVolume, vo.getVolume())
                    .eqIfPresent(ReceivableDO::getFeeType, vo.getFeeType())
                    .eqIfPresent(ReceivableDO::getFeeSource, vo.getFeeSource())
                    .eqIfPresent(ReceivableDO::getExchangeRate, vo.getExchangeRate())
                    .eqIfPresent(ReceivableDO::getCurrencyId, vo.getCurrencyId())
                    .eqIfPresent(ReceivableDO::getUnitPrice, vo.getUnitPrice())
                    .eqIfPresent(ReceivableDO::getTotalAmount, vo.getTotalAmount())
                    .eqIfPresent(ReceivableDO::getDiscountTotal, vo.getDiscountTotal())
                    .eqIfPresent(ReceivableDO::getDiscountRemark, vo.getDiscountRemark())
                    .eqIfPresent(ReceivableDO::getIsPayAdvance, vo.getIsPayAdvance())
                    .eqIfPresent(ReceivableDO::getState, vo.getState())
                    .eqIfPresent(ReceivableDO::getSource, vo.getSource())
                    .eqIfPresent(ReceivableDO::getAuthor, vo.getAuthor())
                    .betweenIfPresent(ReceivableDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(ReceivableDO::getId));
        }

        return null;
    }

    IPage<ReceivableBackVO> receivablePage(@Param("mpPage") IPage<ReceivableBackVO> mpPage, @Param("query") ReceivableQueryVO query);


    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.`status`, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.pick_state, ",
            "o.to_warehouse_state, ",
            "o.tidan_no, ",
            "o.container_number as container_number, ",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_warehouse_line de on o.line_id = de.id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "where r.deleted = 0 and r.receipt_id is null ",
            "AND o.`order_id` = #{orderId}",
            "</script>"
    })
    List<ReceivableBackVO> getReceivableByOrderId(@Param("orderId") Long orderId);


    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.`status`, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.pick_state, ",
            "o.to_warehouse_state, ",
            "o.tidan_no, ",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_order_departure de on de.order_id = o.order_id ",
            "left join ecw_order_objective ob on ob.order_id = o.order_id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "where r.deleted = 0",
            "AND r.state = 0 ",
            "AND r.id in ",
            "<foreach item='id' index='index' collection='ids' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach>)",
            "</script>"
    })
    List<ReceivableBackVO> receivableList(List<Long> ids);

    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.tidan_no, ",
            "o.`status`, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.pick_state, ",
            "o.transport_id, ",
            "o.channel_id, ",
            "de.departure_id, ",
            "ob.objective_id, ",
            "o.to_warehouse_state, ",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_order_departure de on de.order_id = o.order_id ",
            "left join ecw_order_objective ob on ob.order_id = o.order_id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "where r.deleted = 0",
            "AND r.id =  #{receivableId} ",
            "</script>"
    })
    ReceivableBackVO getReceivableBackVo(@Param("receivableId") Long receivableId);


    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.tidan_no, ",
            "o.`status`, ",
            "o.container_number as container_number, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.pick_state, ",
            "o.transport_id, ",
            "o.channel_id, ",
            "de.departure_id, ",
            "ob.objective_id, ",
            "ob.objective ->> '$.titleZh' as objective_name_zh, ",
            "ob.objective ->> '$.titleEn' as objective_name_en, ",
            "o.to_warehouse_state, ",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_order_departure de on de.order_id = o.order_id ",
            "left join ecw_order_objective ob on ob.order_id = o.order_id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "where r.deleted = 0",
            "AND r.receipt_id =  #{receiptId} ",
            "</script>"
    })
    List<ReceivableBackVO> getReceivableBackVoListByReceiptId(@Param("receiptId") Long receiptId);


    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.tidan_no, ",
            "o.`status`, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.pick_state, ",
            "o.transport_id, ",
            "o.channel_id, ",
            "de.departure_id, ",
            "ob.objective_id, ",
            "o.to_warehouse_state, ",
            "er.write_off_proportion as write_off_proportion,",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",

            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_order_departure de on de.order_id = o.order_id ",
            "left join ecw_order_objective ob on ob.order_id = o.order_id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "left join ecw_receipt er on r.receipt_id = er.id ",
            "where r.deleted = 0",
            "AND o.order_id =  #{orderId} ",
            "</script>"
    })
    List<ReceivableBackVO> getReceivableListByOrderId(@Param("orderId") Long orderId);

    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks ",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "where r.deleted = 0 AND r.is_pay_advance = 1",
            "AND r.order_id =  #{orderId} ",
            "</script>"
    })
    List<ReceivableBackVO> getFirstReceivableListByOrderId(@Param("orderId") Long orderId);

    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select  ",
            "a.collection_type as collection_type, ",
            "a.fee_type as fee_type, ",
            "a.fee_source as fee_source, ",
            "a.currency_id as currency_id,",
            "c.title_zh as currency,",
            "p.label as collection_type_label,",
            "f.label as fee_type_label,",
            "COALESCE(b.write_off_proportion,0) AS write_off_proportion,",
            "COALESCE (a.discount_total ,0) as discount_total,",
            "a.receipt_id as receipt_id,",
            "IF(write_off_proportion is null,0, a.total_amount * ( IFNULL( write_off_proportion, 0 ) / 100 ))  AS payed_amount, ",
            "a.total_amount as total_amount",
            "from ecw_receivable a  ",
            "left join system_dict_data p on p.dict_type = 'payment_type' and a.collection_type = p.value and p.deleted=0",
            "left join system_dict_data f on f.dict_type = 'receivable_fee_type' and a.fee_type = f.value and f.deleted=0 ",
            "left join ecw_currency c on c.id = a.currency_id",
            "left join ecw_receipt b on a.receipt_id = b.id and b.deleted = 0",
            "where a.deleted = 0 ",
            "and a.order_id =  #{orderId} ",
            "</script>"
    })
    List<ReceivableOrderPayedFeeTypeVO> getPayedMoneyByFeeType(@Param("orderId") Long orderId);


    @ResultType(ReceivableDO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "IFNULL(u.nickname, '/') as author_name",
            "from ecw_receivable r ",
            "left join system_user u on u.id = r.author",
            "where r.deleted = 0",
            "AND r.order_id =  #{orderId} ",
            "</script>"
    })
    List<ReceivableDO> receivableListByOrderId(@Param("orderId") Long orderId);

    @ResultType(ReceivableDO.class)
    @Select({
            "<script>",
            "select rb.*, ",
            "IFNULL(rt.write_off_proportion, 0) as write_off_proportion",
            "from ecw_receivable rb ",
            "left join ecw_receipt rt on rb.receipt_id = rt.id",
            "where rb.deleted = 0",
            "AND rb.order_id =  #{orderId} ",
            "</script>"
    })
    List<ReceivableDO> allByOrderId(@Param("orderId") Long orderId);

    @Update({
            "update ecw_receivable set `deleted` = 0 ,update_time = now() where `id` = #{receivableId}"
    })
    void revokeDeleteReceivableById(@Param("receivableId") Long receivableId);

    IPage<CostDetailPageVO> getCostDetailPage(IPage<CostDetailPageVO> page, @Param("query") CostDetailPageQueryVO query);

    List<WealthMoneyAmountVO> costDetailSearchAmount(@Param("query") CostDetailPageQueryVO query);

    List<WealthMoneyAmountVO> costDetailAmount();

    List<WealthMoneyAmountVO> receivablePageAmount(@Param("query") ReceivableQueryVO query);

    List<BatchGenReceiptPageVO> batchGenReceiptPage(@Param("start") int start,@Param("size") int size, @Param("query") BatchGenReceiptPageQueryVO query);

    int countBatchGenReceiptPage(@Param("query") BatchGenReceiptPageQueryVO query);

    List<CurrencyAmount> calculateReceivableAmountByOrderId(Long orderId);

    IPage<BatchGenReceiptReceivablePageVO> batchGenReceiptReceivablePage(IPage<BatchGenReceiptReceivablePageVO> mpPage, @Param("query") BatchGenReceiptReceivablePageQueryVO query);

    List<CurrencyAmount> batchGenReceiptReceivablAmount(@Param("query") BatchGenReceiptReceivablePageQueryVO query);

    List<ReceivableBoxReportPageVO> getReceivableBoxPage(@Param("query") ReceivableBoxReportQueryVO query);

    List<ReceivebleBoxRelationOrder> getReceivableBoxRelationOrders(@Param("shipmentIdList") List<Long> shipmentIdList);

    List<ReceivableDO> selectBoxReceivablesByOrderIds(@Param("orderIds") Set<Long> orderIds);

    List<Long> getAllBoxId();

    List<Long> getDestWarehouseIdListByDestCity(@Param("destCity") List<Long> destCity);
}
