package cn.iocoder.yudao.module.wealth.job;

import cn.iocoder.yudao.framework.quartz.core.handler.JobHandler;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receipt.ReceiptMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.ReceiptItemMapper;
import cn.iocoder.yudao.module.wealth.enums.InvoiceDataStatusEnum;
import cn.iocoder.yudao.module.wealth.enums.InvoicingStatusEnum;
import cn.iocoder.yudao.module.wealth.enums.ReceiptItemStatusInReceiptEnum;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@Component
@Slf4j
public class UpdateReceiptNewFieldTask implements JobHandler {
    @Resource
    private ReceiptItemMapper receiptItemMapper;
    @Resource
    private ReceiptMapper receiptMapper;
    /**
     * 更新收款单新字段：应收明细状态
     * @param param 参数
     */
    @Override
    public String execute(String param) throws Exception {
        log.info("更新收款单新字段：应收明细状态");
        long l = System.currentTimeMillis();
        LambdaQueryWrapper<ReceiptDO> receiptDOLambdaQueryWrapper = new LambdaQueryWrapper<>();
        receiptDOLambdaQueryWrapper.select(ReceiptDO::getId,ReceiptDO::getOpenInvoice)
                .eq(ReceiptDO::getDeleted, false);
        List<ReceiptDO> receiptDOS = receiptMapper.selectList(receiptDOLambdaQueryWrapper);
        List<Long> idList = receiptDOS.stream().map(ReceiptDO::getId).collect(Collectors.toList());
        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.in(ReceiptItemDO::getReceiptId, idList)
                .select(ReceiptItemDO::getReceiptId, ReceiptItemDO::getStatus);
        List<ReceiptItemDO> listItem = receiptItemMapper.selectList(lambdaQueryWrapper);
        for (ReceiptDO receiptDO : receiptDOS) {
            if (!listItem.isEmpty()) {
                ReceiptItemStatusInReceiptEnum sMxStatus = getReceiptMxStatus(listItem, receiptDO.getId());
                receiptDO.setReceiptItemStatus(sMxStatus.getValue());
            } else {
                receiptDO.setReceiptItemStatus(ReceiptItemStatusInReceiptEnum.NO_INPUT.getValue());
            }
            if (receiptDO.getOpenInvoice().equals(1)) {
                receiptDO.setInvoiceDataStatus(InvoiceDataStatusEnum.SUBMIT.getValue());
                receiptDO.setInvoicingStatus(InvoicingStatusEnum.WAIT.getValue());
            } else if (receiptDO.getOpenInvoice().equals(0)) {
                receiptDO.setInvoiceDataStatus(InvoiceDataStatusEnum.NO_AUDIT.getValue());
                receiptDO.setInvoicingStatus(InvoicingStatusEnum.NO_INVOICE.getValue());
            }
        }
        List<List<ReceiptDO>> receiptDOSList = receiptDOS.stream().collect(Collectors.groupingBy(item -> receiptDOS.indexOf(item) / 500)).values().stream().map(list -> list.stream().limit(1000).collect(Collectors.toList())).collect(Collectors.toList());
        for (List<ReceiptDO> dos : receiptDOSList) {
            receiptMapper.updateReceiptNewFieldBatch(dos);
        }
        log.info("更新收款单新字段：应收明细状态，耗时：{}", System.currentTimeMillis() - l);
        return "success";
    }

    /**
     * 计算收款单明细状态
     * 收款单主表
     * SELECT * from ecw_receipt where receipt_no='SKD00006547'
     * 收款单明细表
     * SELECT * from ecw_receipt_item where receipt_id=6554
     */
    private ReceiptItemStatusInReceiptEnum getReceiptMxStatus(List<ReceiptItemDO> listItem, long ReceiptId) {
        //未录入 收款单下无收款明细 OK
        //未提交 ：收款单下有收款明细且全未提交 OK
        //部分提交  收款单下有收款明细且部分提交(无需判断收款明细是否审批) OK
        //未审核  收款明细全部提交且全部未审批  OK
        //部分审核  收款明细全部提交且部分已审批  OK
        //全部审核  收款明细全部已审批  OK
        ReceiptItemStatusInReceiptEnum sMxStatus = null;
        //计算明细总数
        Long iCount = listItem.stream().filter(i -> i.getReceiptId() == ReceiptId).collect(Collectors.counting());
        if (iCount == 0) {
            sMxStatus = ReceiptItemStatusInReceiptEnum.NO_INPUT;
            return sMxStatus;
        } else {
            //存在记录就判断状态
            //status 状态0待核销，待提交 1已核销
            //2核销审核中，3反核销审核中
            //bmpStatus
            //PROCESS(1, "处理中"),APPROVE(2, "通过"),
            //REJECT(3, "不通过"),CANCEL(4, "已取消");
            long Item0 = listItem.stream().filter(i -> i.getStatus() == 0).filter(i -> i.getReceiptId() == ReceiptId).count();
            long Item1 = listItem.stream().filter(i -> i.getStatus() == 1).filter(i -> i.getReceiptId() == ReceiptId).count();
            long Item2 = listItem.stream().filter(i -> i.getStatus() == 2).filter(i -> i.getReceiptId() == ReceiptId).count();
            long Item3 = listItem.stream().filter(i -> i.getStatus() == 3).filter(i -> i.getReceiptId() == ReceiptId).count();

            if (Item0 == iCount) {
                //0待核销
                //收款单下有收款明细且全未提交
                sMxStatus = ReceiptItemStatusInReceiptEnum.UN_SUBMITTED;
                return sMxStatus;
            }

            if (Item0 > 0 && Item0 < iCount) {
                //部分提交  收款单下有收款明细且部分提交(无需判断收款明细是否审批)
                sMxStatus = ReceiptItemStatusInReceiptEnum.PART_SUBMITTED;
                return sMxStatus;
            }

            //进部分提交，部分审核逻辑
            //未审核  收款明细全部提交且全部未审批
            if (Item2 == iCount) {
                sMxStatus = ReceiptItemStatusInReceiptEnum.UN_APPROVED;
                return sMxStatus;
            }

            if (Item1 == iCount) {
                //1已核销
                sMxStatus = ReceiptItemStatusInReceiptEnum.ALL_APPROVED;
                return sMxStatus;
            }

            if ((Item2 + Item3) == iCount || (Item1 + Item3) == iCount || (Item2 + Item1) == iCount) {
                sMxStatus = ReceiptItemStatusInReceiptEnum.PART_APPROVED;
                return sMxStatus;
            }
        }
        return sMxStatus;
    }
}
