package cn.iocoder.yudao.module.wealth.service;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.enums.WorkFlowEmus;
import cn.iocoder.yudao.module.bpm.resp.TaskResp;
import cn.iocoder.yudao.module.ecw.api.bank.BankApi;
import cn.iocoder.yudao.module.ecw.api.bank.dto.BankAccountDTO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptApproval.ReceiptApprovalDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableWriteOffRecordDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receipt.ReceiptMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.ReceiptItemMapper;
import cn.iocoder.yudao.module.wealth.enums.ReceiptItemStatusEnum;
import cn.iocoder.yudao.module.wealth.enums.ReceiptLinkEnum;
import cn.iocoder.yudao.module.wealth.enums.ReceiptStatusEnum;
import cn.iocoder.yudao.module.wealth.listener.event.ReceiptItemWriteOffEvent;
import cn.iocoder.yudao.module.wealth.service.receipt.ReceiptService;
import cn.iocoder.yudao.module.wealth.service.receiptApproval.ReceiptApprovalService;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.RECEIPT_ITEM_NOT_FOUND;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.RECEIPT_NOT_FOUND;

@Service
@Slf4j
public class FinanceReceiptApproveService {

    @Resource
    private ReceiptMapper receiptMapper;
    @Resource
    private ReceiptService receiptService;
    @Resource
    private ReceiptItemMapper receiptItemMapper;
    @Resource
    private BpmCreateServiceFactory bpmCreateServiceFactory;
    @Resource
    private ReceiptApprovalService receiptApprovalService;
    @Resource
    private ReceivableService receivableService;
    @Resource
    private ApplicationContext applicationContext;
    @Resource
    private BankApi bankApi;

    @Transactional
    public void financeApproveCallBack(String bmpKey, String businessKey, Integer result) {
        ReceiptApprovalDO receiptApprovalDO = receiptApprovalService.getReceiptApproval(Long.parseLong(businessKey));
        // 查询最后审核的详细信息
        TaskResp taskResp =
                bpmCreateServiceFactory.getLastTaskByProcessInstanceId(
                        receiptApprovalDO.getBmpId());
        String comment = "";
        if (Objects.nonNull(taskResp)) {
            comment = taskResp.getComment();
        }
        //根据不同的流程实例进行处理
        if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getKey(), bmpKey)) {
            if (null == receiptApprovalDO.getReceiptId()) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO receiptDO =
                    receiptMapper.selectById(receiptApprovalDO.getReceiptId());
            if (null == receiptDO) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO updateReceipt = new ReceiptDO();
            //添加订单ID的修改，不然订单ID就丢失了
            // updateReceipt.setOrderId(receiptDO.getOrderId());

            updateReceipt.setId(receiptDO.getId());
            updateReceipt.setBmpStatus(result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.RECEIPT_WAITING.getValue());

                LambdaQueryWrapper<ReceivableDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                lambdaQueryWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                List<ReceivableDO> orderList = receivableService.list(lambdaQueryWrapper);
                List orderIds = orderList.stream().map(ReceivableDO::getOrderId).distinct().collect(Collectors.toList());
                receivableService.orderReceivableAddReceiptLog(orderIds, receiptDO.getReceiptNo());
            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.APPROVE_FAIL.getValue());
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.DRAFT.getValue());
            }
            updateReceipt.setComment(comment);
            receiptMapper.updateById(updateReceipt);
            receiptService.recordLog(receiptDO.getId(), ReceiptLinkEnum.APPROVE_RECEIPT, comment, WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getValue());
        } else if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_APPROVE_NO.getKey(), bmpKey)) {
            //收款单反审核
            if (null == receiptApprovalDO.getReceiptId()) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO receiptDO = receiptMapper.selectById(receiptApprovalDO.getReceiptId());
            if (null == receiptDO) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO updateReceipt = new ReceiptDO();
            // 添加订单ID的修改，不然订单ID就丢失了
            // updateReceipt.setOrderId(receiptDO.getOrderId());
            updateReceipt.setId(receiptDO.getId());
            updateReceipt.setBmpStatus(result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.DRAFT.getValue());
            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.RECEIPT_WAITING.getValue());
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.RECEIPT_WAITING.getValue());
            }
            updateReceipt.setComment(comment);
            receiptMapper.updateById(updateReceipt);
            receiptService.recordLog(receiptDO.getId(), ReceiptLinkEnum.REJECT_RECEIPT, comment, WorkFlowEmus.FINANCE_RECEIPT_APPROVE_NO.getValue());
        } else if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF.getKey(), bmpKey)) {
            //收款单核销审核
            if (null == receiptApprovalDO.getReceiptId()) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO receiptDO = receiptMapper.selectById(receiptApprovalDO.getReceiptId());
            if (null == receiptDO) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO updateReceipt = new ReceiptDO();
            // 添加订单ID的修改，不然订单ID就丢失了
            // updateReceipt.setOrderId(receiptDO.getOrderId());
            updateReceipt.setId(receiptDO.getId());
            updateReceipt.setBmpStatus(result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_ALL_SUCCESS.getValue());

                //更新收该收款单下的应收为收款完成状态
                LambdaUpdateWrapper<ReceivableDO> lambdaUpdateWrapper = new LambdaUpdateWrapper();
                lambdaUpdateWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                lambdaUpdateWrapper.set(ReceivableDO::getState, 2);
                receivableService.update(lambdaUpdateWrapper);

                updateReceipt.setWriteOffAt(new Date());
                updateReceipt.setWriteOffProportion(new BigDecimal("100"));//设置核销比例

                //添加订单核销日志
                LambdaQueryWrapper<ReceivableDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                lambdaQueryWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                List<ReceivableDO> orderList = receivableService.list(lambdaQueryWrapper);
                List orderIds = orderList.stream().map(ReceivableDO::getOrderId).distinct().collect(Collectors.toList());

                receivableService.orderReceivableWriteOffLogAdd(orderIds, receiptDO.getReceiptNo());

            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_PART_ING.getValue());
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_WAITING.getValue());
            }
            updateReceipt.setPayedAt(new Date());
            updateReceipt.setComment(comment);
            receiptMapper.updateById(updateReceipt);
            receiptService.recordLog(receiptDO.getId(), ReceiptLinkEnum.WRITE_OFF_RECEIPT, comment, WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF.getValue());
        } else if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF_NO.getKey(), bmpKey)) {
            //收款单核销反审核
            if (null == receiptApprovalDO.getReceiptId()) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO receiptDO = receiptMapper.selectById(receiptApprovalDO.getReceiptId());
            if (null == receiptDO) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO updateReceipt = new ReceiptDO();
            // 添加订单ID的修改，不然订单ID就丢失了
            // updateReceipt.setOrderId(receiptDO.getOrderId());
            updateReceipt.setId(receiptDO.getId());
            updateReceipt.setBmpStatus(result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                //需要根据应收明细核销状态 变更成部分核销或者待核销
                LambdaQueryWrapper<ReceiptItemDO> receiptItemQueryWrapper = new LambdaQueryWrapper();
                receiptItemQueryWrapper.eq(ReceiptItemDO::getReceiptId, receiptDO.getId());
                receiptItemQueryWrapper.eq(ReceiptItemDO::getStatus, ReceiptItemStatusEnum.WRITE_OFF.getValue());
                if (receiptItemMapper.selectCount(receiptItemQueryWrapper) == 0) {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_WAITING.getValue());
                } else {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_PART_ING.getValue());
                }

                //更新收该收款单下的应收为收款完成状态
                LambdaUpdateWrapper<ReceivableDO> lambdaUpdateWrapper = new LambdaUpdateWrapper();
                lambdaUpdateWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                lambdaUpdateWrapper.set(ReceivableDO::getState, 1);
                receivableService.update(lambdaUpdateWrapper);

                //添加订单反核销日志
                LambdaQueryWrapper<ReceivableDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                lambdaQueryWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                List<ReceivableDO> orderList = receivableService.list(lambdaQueryWrapper);
                List orderIds = orderList.stream().map(ReceivableDO::getOrderId).distinct().collect(Collectors.toList());
                receivableService.orderReceivableCancelWriteOffLogAdd(orderIds, receiptDO.getReceiptNo());
            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                //已核销待开票状态已取消
                updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_ALL_SUCCESS.getValue());
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_ALL_SUCCESS.getValue());
            }
            updateReceipt.setComment(comment);
            receiptMapper.updateById(updateReceipt);
            receiptService.recordLog(receiptDO.getId(), ReceiptLinkEnum.REJECT_WRITE_OFF_RECEIPT, comment, WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF_NO.getValue());
        } else if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF.getKey(), bmpKey)) {
            //收款单明细核销审核
            applicationContext.publishEvent(new ReceiptItemWriteOffEvent(receiptApprovalDO.getReceiptItemId(), result, comment));
        } else if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF_NO.getKey(), bmpKey)) {
            //收款单明细核销反审核
            if (null == receiptApprovalDO.getReceiptItemId()) {
                throw exception(RECEIPT_ITEM_NOT_FOUND);
            }
            ReceiptItemDO receiptItemDO = receiptItemMapper.selectById(receiptApprovalDO.getReceiptItemId());
            if (null == receiptItemDO) {
                throw exception(RECEIPT_ITEM_NOT_FOUND);
            }
            LambdaUpdateWrapper<ReceiptItemDO> updateWrapper = Wrappers.lambdaUpdate(ReceiptItemDO.class);
            updateWrapper.eq(ReceiptItemDO::getId, receiptItemDO.getId());
            updateWrapper.set(ReceiptItemDO::getBmpStatus, result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                updateWrapper.set(ReceiptItemDO::getStatus, ReceiptItemStatusEnum.DRAFT.getValue());
                updateWrapper.set(ReceiptItemDO::getApprovalTime, null);
                receiptItemMapper.update(null, updateWrapper);
                //需要根据应收明细核销状态 变更成部分核销或者待核销
                ReceiptDO updateReceipt = new ReceiptDO();
                updateReceipt.setId(receiptItemDO.getReceiptId());
                LambdaQueryWrapper<ReceiptItemDO> receiptItemQueryWrapper = new LambdaQueryWrapper();
                receiptItemQueryWrapper.eq(ReceiptItemDO::getReceiptId, receiptItemDO.getReceiptId());
                receiptItemQueryWrapper.eq(ReceiptItemDO::getStatus, ReceiptItemStatusEnum.WRITE_OFF.getValue());
                if (receiptItemMapper.selectCount(receiptItemQueryWrapper) == 0) {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_WAITING.getValue());
                } else {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_PART_ING.getValue());
                }
                receiptMapper.updateById(updateReceipt);
                // 将应收明细的核销状态和核销金额重置
                List<ReceivableDO> receivableDOS = receivableService.selectList("receipt_id", receiptItemDO.getReceiptId());
                for (ReceivableDO receivableDO : receivableDOS) {
                    ReceivableWriteOffRecordDO receivableWriteOffRecordDO = receivableService.getReceivableWriteOffRecord(receivableDO.getId(), receiptItemDO.getId());
                    if (null != receivableWriteOffRecordDO) {
                        BigDecimal writeOffCurrent = receivableWriteOffRecordDO.getWriteOffAmount().divide(receivableDO.getExchangeRate(), 4, RoundingMode.HALF_UP);
                        receivableDO.setWriteOffAmount(receivableDO.getWriteOffAmount().subtract(writeOffCurrent));
                    }
                    if (receivableDO.getWriteOffAmount().compareTo(BigDecimal.ZERO) <= 0) {
                        receivableDO.setState(0);
                    }
                }
                receivableService.updateBatchById(receivableDOS);
                receiptService.updateReceiptItemStatus(receiptItemDO.getReceiptId());
                // 更新银行账户余额
                bankApi.updateBankAccountBalance(BankAccountDTO.builder()
                        .id(receiptItemDO.getAccountId())
                        .baAccountName(receiptItemDO.getAccountName())
                        .baAccountNum(receiptItemDO.getAccountNo())
                        .baBankName(receiptItemDO.getAccountBankName())
                        .baBalance(receiptItemDO.getAmount()).build(), false);
            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                updateWrapper.set(ReceiptItemDO::getStatus, ReceiptItemStatusEnum.WRITE_OFF.getValue());
                receiptItemMapper.update(null, updateWrapper);
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                updateWrapper.set(ReceiptItemDO::getStatus, ReceiptItemStatusEnum.WRITE_OFF.getValue());
                receiptItemMapper.update(null, updateWrapper);
            }
            receiptService.recordLog(receiptItemDO.getReceiptId(), ReceiptLinkEnum.REJECT_BANK_RECEIPT, comment, WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF_NO.getValue());
        }
        ReceiptApprovalDO updateReceiptApproval = new ReceiptApprovalDO();
        updateReceiptApproval.setId(receiptApprovalDO.getId());
        updateReceiptApproval.setStatus(result);
        receiptApprovalService.updateById(updateReceiptApproval);
    }
}
