package cn.iocoder.yudao.module.wealth.service.payment;

import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.apollo.core.event.Order.OrderNumberLogEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.code.CodeUtils;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.enums.WorkFlowEmus;
import cn.iocoder.yudao.module.wealth.convert.payment.PaymentConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payable.PayableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payment.PaymentDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.paymentApproval.PaymentApprovalDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.payment.PaymentMapper;
import cn.iocoder.yudao.module.wealth.enums.PaymentStatusEnum;
import cn.iocoder.yudao.module.wealth.service.payable.PayableService;
import cn.iocoder.yudao.module.wealth.service.paymentApproval.PaymentApprovalService;
import cn.iocoder.yudao.module.wealth.vo.payable.PayableReqVO;
import cn.iocoder.yudao.module.wealth.vo.payment.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;

/**
 * 付款单 Service 实现类
 *
 * @author 郑屹
 */
@Service
@Validated
public class PaymentServiceImpl extends AbstractService<PaymentMapper, PaymentDO> implements PaymentService {

    @Autowired
    private PaymentMapper paymentMapper;

    @Resource
    private PayableService payableService;

    @Autowired
    private ApplicationContext applicationContext;
    @Resource
    PaymentApprovalService paymentApprovalService;
    @Resource
    BpmCreateServiceFactory bpmCreateServiceFactory;

    private static final String PAY_NEW_NUMBER = "payment:new:number";

    @Override
    public Long createPayment(PaymentCreateReqVO createReqVO) {
        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String creator = String.valueOf(null == user ? "" : user.getId());
        PaymentDO payment = PaymentConvert.INSTANCE.convert(createReqVO);

        OrderNumberLogEvent event = new OrderNumberLogEvent();
        event.setType(3);
        event.setNumberKey(PAY_NEW_NUMBER);
        applicationContext.publishEvent(event);
        String newPaymentNo = CodeUtils.getOddPayNumbers(event.getNumber());
        payment.setPaymentNo(newPaymentNo);

        event.setNewNumber(newPaymentNo);
        applicationContext.publishEvent(event);

        payment.setPaymentAddBy(null == user ? "" : user.getUsername());
        payment.setPaymentAddId(null == user ? null : user.getId());

        payment.setCreateTime(new Date());
        payment.setCreator(creator);
        payment.setUpdateTime(new Date());
        payment.setUpdater(creator);


        if (StringUtils.equals(createReqVO.getAddType(), "0")) {
            payment.setState(PaymentStatusEnum.DRAFT.getValue());
        }

        //把文件List转字符串存储在数据库中
        String sFilePath= JSONUtil.toJsonStr(createReqVO.getFilelist()) ;
        payment.setFilepath(sFilePath);


        paymentMapper.insert(payment);

        if (StringUtils.equals(createReqVO.getAddType(), "1")) {
            validatePaymentBmpStatus(payment.getId());
            //创建收款单并发起审核流程
            PaymentApprovalDO approvalDO = new PaymentApprovalDO();
            approvalDO.setPaymentId(payment.getId());
            approvalDO.setPaymentNo(payment.getPaymentNo());
            approvalDO.setBmpKey(WorkFlowEmus.FINANCE_PAYMENT_APPROVE.getKey());
            approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            approvalDO.setReason(createReqVO.getReason());
            paymentApprovalService.save(approvalDO);
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_PAYMENT_APPROVE.getKey(), payment.getPaymentNo(), createReqVO.getCopyUserList());
            approvalDO.setBmpId(bpmId);
            paymentApprovalService.updateById(approvalDO);

            PaymentDO update = new PaymentDO();
            update.setId(payment.getId());
            update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            update.setState(PaymentStatusEnum.APPROVE_ING.getValue());
            update.setBmpId(bpmId);
            paymentMapper.updateById(update);
        }


        //修改应付明细里面的付款单信息
        List<PayableReqVO> payableReqVOList = createReqVO.getPayableReqVOList();
        for (PayableReqVO payableReqVO : payableReqVOList) {
            //回写应付单信息到应付信息中
            PayableDO payableDO = new PayableDO();
            payableDO.setId(payableReqVO.getId());
            payableDO.setPaymentId(payment.getId());
            payableDO.setInvoiceNumber(payableReqVO.getInvoiceNumber());
            payableDO.setTotal(payableReqVO.getTotal());
            payableDO.setTaxRate(payableReqVO.getTaxRate());
            payableDO.setExchangeRate(payableReqVO.getExchangeRate());
            payableDO.setState(1);
            payableService.updateById(payableDO);

        }


        return payment.getId();
    }

    @Override
    public void updatePayment(PaymentUpdateReqVO updateReqVO) {
        PaymentDO paymentDo = paymentMapper.selectById(updateReqVO.getId());
        if (null == paymentDo) {
            throw exception(PAYMENT_NOT_EXISTS);
        }
        //TODO 判断付款单状态能不能进行更改
        /*if (paymentDo.getState() != 1 && paymentDo.getState() != 3) {
            throw exception(PAYMENT_NOT_UPDATE);
        }*/

        // 更新
        PaymentDO updateObj = PaymentConvert.INSTANCE.convert(updateReqVO);
        if (StringUtils.equals(updateReqVO.getAddType(), "0")) {
            updateReqVO.setState(PaymentStatusEnum.DRAFT.getValue());
        }

        if (StringUtils.equals(updateReqVO.getAddType(), "1")) {
            validatePaymentBmpStatus(paymentDo.getId());
            //创建收款单并发起审核流程
            PaymentApprovalDO approvalDO = new PaymentApprovalDO();
            approvalDO.setPaymentId(paymentDo.getId());
            approvalDO.setPaymentNo(paymentDo.getPaymentNo());
            approvalDO.setBmpKey(WorkFlowEmus.FINANCE_PAYMENT_APPROVE.getKey());
            approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            approvalDO.setReason(updateReqVO.getReason());
            paymentApprovalService.save(approvalDO);
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_PAYMENT_APPROVE.getKey(), paymentDo.getPaymentNo(), updateReqVO.getCopyUserList());
            approvalDO.setBmpId(bpmId);
            paymentApprovalService.updateById(approvalDO);

            updateObj.setBmpId(bpmId);
            updateObj.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            updateObj.setState(PaymentStatusEnum.APPROVE_ING.getValue());

        }

        //
        String sFilePath= JSONUtil.toJsonStr(updateReqVO.getFilelist()) ;
        updateObj.setFilepath(sFilePath);
        //end

        paymentMapper.updateById(updateObj);
        //修改应付明细里面的付款单信息
        List<PayableReqVO> payableReqVOList = updateReqVO.getPayableReqVOList();
        for (PayableReqVO payableReqVO : payableReqVOList) {
            //回写应付单信息到应付信息中
            PayableDO payableDO = new PayableDO();
            payableDO.setId(payableReqVO.getId());
            payableDO.setPaymentId(updateObj.getId());
            payableDO.setInvoiceNumber(payableReqVO.getInvoiceNumber());
            payableDO.setTotal(payableReqVO.getTotal());
            payableDO.setTaxRate(payableReqVO.getTaxRate());
            payableDO.setExchangeRate(payableReqVO.getExchangeRate());
            payableDO.setState(1);
            payableService.updateById(payableDO);
        }
    }

    @Override
    public void deletePayment(Long id) {
        // 校验存在
        this.validatePaymentExists(id);
        // 删除
        paymentMapper.deleteById(id);
    }

    private void validatePaymentExists(Long id) {
        if (paymentMapper.selectById(id) == null) {
            throw exception(PAYMENT_NOT_EXISTS);
        }
    }

    @Override
    public PaymentDO getPayment(Long id) {
        return paymentMapper.selectById(id);
    }

    @Override
    public List<PaymentDO> getPaymentList(Collection<Long> ids) {
        return paymentMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<PaymentDO> getPaymentPage(PaymentQueryVO query, PageVO page) {
        if (Objects.nonNull(query) && StringUtils.isBlank(query.getPaymentNo())){
            // 避免前端传空字符串时，将空字符串带入查询条件
            query.setPaymentNo(null);
        }
        return paymentMapper.selectPage(page, query);
    }

    @Override
    public List<PaymentDO> getPaymentList(PaymentQueryVO query) {
        return paymentMapper.selectList(query);
    }

    @Override
    public void verifyPayment(PaymentVerifyVo paymentVerifyVo) {
        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String creator = String.valueOf(null == user ? "" : user.getId());
        PaymentDO paymentDo = paymentMapper.selectById(paymentVerifyVo.getPaymentId());
        if (null == paymentDo) {
            throw exception(PAYMENT_NOT_EXISTS);
        }
        PaymentDO updatePayment = new PaymentDO();
        updatePayment.setId(paymentDo.getId());
        if (paymentDo.getState() == 1 || paymentDo.getState() == 3) {
            if (StringUtils.equals("y", paymentVerifyVo.getVerifyStatus())) {
                updatePayment.setState(2);
            }
            if (StringUtils.equals("n", paymentVerifyVo.getVerifyStatus())) {
                updatePayment.setState(3);
            }
            updatePayment.setReviewId(user.getId());
            updatePayment.setReviewName(user.getUsername());
            updatePayment.setUpdateTime(new Date());
            updatePayment.setReviewAt(new Date());
            updatePayment.setUpdater(creator);
            this.updateById(updatePayment);
        } else {
            throw exception(PAYMENT_NOT_STATUS);
        }
    }

    @Override
    public void verifyPaymentCancel(FinancePaymentVo financePaymentVo) {
        validatePaymentBmpStatus(financePaymentVo.getPaymentId());
        //创建付款单反审核流程
        PaymentApprovalDO approvalDO = new PaymentApprovalDO();
        approvalDO.setPaymentId(financePaymentVo.getPaymentId());
        approvalDO.setPaymentNo(financePaymentVo.getPaymentNo());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_PAYMENT_APPROVE_NO.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        //反审核理由
        approvalDO.setReason(financePaymentVo.getReason());
        paymentApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_PAYMENT_APPROVE_NO.getKey(), financePaymentVo.getPaymentNo(), financePaymentVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        paymentApprovalService.updateById(approvalDO);

        PaymentDO update = new PaymentDO();
        update.setId(financePaymentVo.getPaymentId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setState(PaymentStatusEnum.APPROVE_NEGATION.getValue());
        update.setBmpId(bpmId);
        this.updateById(update);
    }

    @Override
    public void verification(FinancePaymentVo financePaymentVo) {
        validatePaymentBmpStatus(financePaymentVo.getPaymentId());
        PaymentDO update = new PaymentDO();
        update.setId(financePaymentVo.getPaymentId());
        update.setState(PaymentStatusEnum.WRITE_OFF.getValue());

        //
        //付款备注
        update.setReason(financePaymentVo.getReason());
        //实际付款日期
        update.setSjfkdate(financePaymentVo.getSjfkdate());
        //付款人取核销人，就是当前操作的用户
        update.setFkren(financePaymentVo.getFkren());
        //核销日期
        update.setHexiaodate(new Date());
        this.updateById(update);

    }

    @Override
    public void verificationCancel(FinancePaymentVo financePaymentVo) {
        validatePaymentBmpStatus(financePaymentVo.getPaymentId());
        //创建付款单反核销流程
        PaymentApprovalDO approvalDO = new PaymentApprovalDO();
        approvalDO.setPaymentId(financePaymentVo.getPaymentId());
        approvalDO.setPaymentNo(financePaymentVo.getPaymentNo());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_PAYMENT_WRITE_OFF_NO.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(financePaymentVo.getReason());
        paymentApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_PAYMENT_WRITE_OFF_NO.getKey(), financePaymentVo.getPaymentNo(), financePaymentVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        paymentApprovalService.updateById(approvalDO);

        PaymentDO update = new PaymentDO();
        update.setId(financePaymentVo.getPaymentId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setState(PaymentStatusEnum.WRITE_OFF_NO_APPROVE_ING.getValue());
        update.setBmpId(bpmId);
        this.updateById(update);


        /*if (paymentDO.getState() == 4) {
            LoginUser user = SecurityFrameworkUtils.getLoginUser();
            String creator = String.valueOf(null == user ? "" : user.getId());

            PaymentDO updatePaymentDO = new PaymentDO();
            updatePaymentDO.setId(id);
            updatePaymentDO.setState(1);

            updatePaymentDO.setUpdateTime(new Date());
            updatePaymentDO.setUpdater(creator);

            this.updateById(updatePaymentDO);

        } else {
            throw exception(PAYMENT_VERIFICATION_ERROR);
        }*/

    }

    @Override
    public void cancelFinancePaymentApproval(FinancePaymentCancelVo financePaymentCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        PaymentDO paymentDO = validatePaymentBmpCancel(financePaymentCancelVo.getPaymentId());
        bpmCreateServiceFactory.cancelBmpById(userId, paymentDO.getBmpId(), financePaymentCancelVo.getRemark());
    }

    @Override
    public void cancelFinancePaymentApprovalNo(FinancePaymentCancelVo financePaymentCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        PaymentDO paymentDO = validatePaymentBmpCancel(financePaymentCancelVo.getPaymentId());
        bpmCreateServiceFactory.cancelBmpById(userId, paymentDO.getBmpId(), financePaymentCancelVo.getRemark());
    }

    @Override
    public void cancelFinancePaymentWriteOff(FinancePaymentCancelVo financePaymentCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        PaymentDO paymentDO = validatePaymentBmpCancel(financePaymentCancelVo.getPaymentId());
        bpmCreateServiceFactory.cancelBmpById(userId, paymentDO.getBmpId(), financePaymentCancelVo.getRemark());
    }

    @Override
    public void cancelFinancePaymentWriteOffNo(FinancePaymentCancelVo financePaymentCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        PaymentDO paymentDO = validatePaymentBmpCancel(financePaymentCancelVo.getPaymentId());
        bpmCreateServiceFactory.cancelBmpById(userId, paymentDO.getBmpId(), financePaymentCancelVo.getRemark());
    }

    private void validatePaymentBmpStatus(Long id) {
        PaymentDO paymentDO = paymentMapper.selectById(id);
        if (null == paymentDO) {
            throw exception(PAYMENT_NOT_EXISTS);
        }
        if (null != paymentDO.getBmpStatus() && paymentDO.getBmpStatus() == BpmProcessInstanceResultEnum.PROCESS.getResult()) {
            throw exception(PAYMENT_APPROVING);
        }
    }

    private PaymentDO validatePaymentBmpCancel(Long id) {
        PaymentDO paymentDO = this.getPayment(id);
        if (null == paymentDO) {
            throw exception(PAYMENT_NOT_EXISTS);
        }
        if (null == paymentDO.getBmpStatus()) {
            throw exception(BMP_NOT);
        }

        if (paymentDO.getBmpStatus() != BpmProcessInstanceResultEnum.PROCESS.getResult()) {
            throw exception(BMP_CANCEL);
        }

        return paymentDO;
    }
}

