package cn.iocoder.yudao.module.wealth.service.receipt;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.boot.module.order.api.OrderApi;
import cn.iocoder.boot.module.order.api.dto.OrderCustomerInfoDTO;
import cn.iocoder.yudao.framework.apollo.core.event.Order.OrderNumberLogEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.code.CodeUtils;
import cn.iocoder.yudao.framework.common.util.collectors.CollectorsUtil;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.enums.WorkFlowEmus;
import cn.iocoder.yudao.module.ecw.api.bank.BankApi;
import cn.iocoder.yudao.module.ecw.api.bank.dto.BankAccountDTO;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.ecw.api.region.RegionApi;
import cn.iocoder.yudao.module.ecw.api.region.dto.RegionDTO;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import cn.iocoder.yudao.module.wealth.convert.receipt.ReceiptConvert;
import cn.iocoder.yudao.module.wealth.convert.receiptAccount.ReceiptAccountConvert;
import cn.iocoder.yudao.module.wealth.convert.receiptInvoice.ReceiptInvoiceConvert;
import cn.iocoder.yudao.module.wealth.convert.receivable.ReceivableConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptBankWriteoffRangeDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptAccount.ReceiptAccountDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptApproval.ReceiptApprovalDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptInvoice.ReceiptInvoiceDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptLog.ReceiptLogDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receipt.ReceiptBankWriteoffRangeMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receipt.ReceiptMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptAccount.ReceiptAccountMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptInvoice.ReceiptInvoiceMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.ReceiptItemMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptLog.ReceiptLogMapper;
import cn.iocoder.yudao.module.wealth.enums.*;
import cn.iocoder.yudao.module.wealth.service.receiptAccount.ReceiptAccountService;
import cn.iocoder.yudao.module.wealth.service.receiptApproval.ReceiptApprovalService;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.util.WealthGenCodeUtils;
import cn.iocoder.yudao.module.wealth.vo.receipt.*;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptAccountCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptInvoice.InvoiceInReceiptRespVO;
import cn.iocoder.yudao.module.wealth.vo.receiptInvoice.ReceiptInvoiceCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableBackVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableUpdateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.apache.commons.lang3.StringUtils;
import org.springframework.aop.framework.AopContext;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.NumberFormat;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.USER_NOT_EXISTS;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.wealth.enums.ReceiptItemStatusEnum.*;

/**
 * 收款单 Service 实现类
 *
 * @author 郑屹
 */
@Service
@Validated
public class ReceiptServiceImpl extends AbstractService<ReceiptMapper, ReceiptDO> implements ReceiptService {
    @Resource
    private ReceiptMapper receiptMapper;
    @Resource
    private ReceivableService receivableService;
    @Resource
    private ReceiptAccountService receiptAccountService;
    @Resource
    private ReceiptAccountMapper receiptAccountMapper;
    @Resource
    private ReceiptItemMapper receiptItemMapper;
    @Resource
    private ApplicationContext applicationContext;
    @Resource
    private BpmCreateServiceFactory bpmCreateServiceFactory;
    @Resource
    private ReceiptApprovalService receiptApprovalService;
    @Resource
    private DictDataApi dictDataApi;
    @Resource
    private CurrencyApi currencyApi;
    @Resource
    private ReceiptBankWriteoffRangeMapper receiptBankWriteoffRangeMapper;
    @Resource
    private ReceiptInvoiceMapper receiptInvoiceMapper;
    @Resource
    private WealthGenCodeUtils wealthGenCodeUtils;
    @Resource
    private ReceiptLogMapper receiptLogMapper;
    @Resource
    private AdminUserApi adminUserApi;
    @Resource
    private BankApi bankApi;
    @Resource
    @Lazy
    private OrderApi orderApi;
    @Resource
    private RegionApi regionApi;

    private static final String RECEIPT_NEW_NUMBER = "receipt:new:number";

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createReceipt(ReceiptCreateReqVO createReqVO) {
        validateCreateVO(createReqVO);

        ReceiptService currentProxy = (ReceiptService) AopContext.currentProxy();

        return currentProxy.generateReceiptRelevantData(createReqVO, false);
    }

    /**
     * 发起审批流程
     *
     * @param reason
     * @param copyUserList
     * @param receipt
     */
    private void initApproval(String reason, String[] copyUserList, ReceiptDO receipt, boolean isBatch) {
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(receipt.getId());
        approvalDO.setReceiptNo(receipt.getReceiptNo());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(reason);
        receiptApprovalService.save(approvalDO);
        if (!isBatch) {
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getKey(), receipt.getReceiptNo(), copyUserList);
            approvalDO.setBmpId(bpmId);
            receiptApprovalService.updateById(approvalDO);
            receipt.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            receipt.setState(ReceiptStatusEnum.APPROVE_ING.getValue());
            receipt.setBmpId(bpmId);
        } else {
            approvalDO.setStatus(BpmProcessInstanceResultEnum.APPROVE.getResult());
            receiptApprovalService.updateById(approvalDO);
            receipt.setBmpStatus(BpmProcessInstanceResultEnum.APPROVE.getResult());
            receipt.setState(ReceiptStatusEnum.RECEIPT_WAITING.getValue());
        }
        recordLog(receipt.getId(), ReceiptLinkEnum.SUBMIT_RECEIPT, receipt.getRemark(), WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getValue());
    }

    /**
     * 创建发票
     *
     * @param receiptInvoiceCreateReqVO
     * @param receipt
     */
    private void createInvoice(ReceiptInvoiceCreateReqVO receiptInvoiceCreateReqVO, ReceiptDO receipt) {
        ReceiptInvoiceDO receiptInvoiceDO = ReceiptInvoiceConvert.INSTANCE.convertInvoiceCreateReqVO(receiptInvoiceCreateReqVO);
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        receiptInvoiceDO.setApplyUserId(loginUser.getId());
        receiptInvoiceDO.setApplyUserName(loginUser.getUsername());
        receiptInvoiceDO.setInvoiceHeader(receiptInvoiceCreateReqVO.getInvoice());
        receiptInvoiceDO.setInvoiceAddress(receiptInvoiceCreateReqVO.getAddressPhone());
        receiptInvoiceDO.setInvoiceItem(receiptInvoiceCreateReqVO.getProjectName());
        receiptInvoiceDO.setInvoiceNo(wealthGenCodeUtils.generateInvoiceCode());
        receiptInvoiceDO.setReceiptId(receipt.getId());
        receiptInvoiceDO.setReceiptNo(receipt.getReceiptNo());
        receiptInvoiceMapper.insert(receiptInvoiceDO);
        receipt.setInvoiceId(receiptInvoiceDO.getId());
        receipt.setInvoicingStatus(InvoicingStatusEnum.WAIT.getValue());
        receipt.setInvoiceDataStatus(InvoiceDataStatusEnum.SUBMIT.getValue());
    }

    /**
     * 生成收款账户信息
     *
     * @param receiptAccountCreateReqVOList
     * @param receipt
     */
    private void genReceiptAccount(List<ReceiptAccountCreateReqVO> receiptAccountCreateReqVOList, ReceiptDO receipt) {
        List<ReceiptAccountDO> receiptAccountDOList = receiptAccountCreateReqVOList.stream().map(re -> {
            ReceiptAccountDO receivableDO = ReceiptAccountConvert.INSTANCE.convert(re);
            receivableDO.setReceiptId(receipt.getId());
            receivableDO.setUpdateTime(new Date());
            return receivableDO;
        }).collect(Collectors.toList());
        receiptAccountService.saveBatch(receiptAccountDOList);
    }

    /**
     * 应收明细关联收款单
     *
     * @param receivableUpdateReqVOList
     * @param receipt
     */
    private void receivableAssocReceipt(List<ReceivableUpdateReqVO> receivableUpdateReqVOList, ReceiptDO receipt) {
        List<ReceivableDO> receivableDOList = receivableUpdateReqVOList.stream().filter(re -> null != re.getId()).map(re -> {
            ReceivableDO receivableDO = ReceivableConvert.INSTANCE.convert(re);
            receivableDO.setReceiptNo(receipt.getReceiptNo());
            receivableDO.setReceiptId(receipt.getId());
            receivableDO.setState(1);
            return receivableDO;
        }).collect(Collectors.toList());
        receivableService.updateBatchById(receivableDOList);
    }

    /**
     * 校验创建收款单
     *
     * @param createReqVO
     */
    private void validateCreateVO(ReceiptCreateReqVO createReqVO) {
        // 收款账户信息（对应收款信息/账单信息）
        if (CollectionUtil.isEmpty(createReqVO.getReceiptAccountList())) {
            throw exception(RECEIPT_ACCOUNT_NOT_EXISTS);
        }
        // 校验应收明细
        validateSameCustomerReceivable(createReqVO.getReceivableVOList());
    }

    @Override
    public List<ReceiptBatchGenRespVO> batchGenReceipt(List<ReceiptBatchCreateReqVO> batchCreateReqVOList) {
        if (CollectionUtil.isEmpty(batchCreateReqVOList)) {
            return Collections.emptyList();
        }
        List<ReceiptCreateReqVO> createReqVOList = assembleCreateReqVOList(batchCreateReqVOList);
        ArrayList<ReceiptBatchGenRespVO> respVOS = new ArrayList<>();
        createReqVOList.forEach(createReqVO -> {
            try {
                validateCreateVO(createReqVO);
            } catch (Exception e) {
                respVOS.add(new ReceiptBatchGenRespVO(null, createReqVO.getCustomerName(), e.getMessage()));
            }
        });
        if (!respVOS.isEmpty()) {
            return respVOS;
        }
        ReceiptService currentProxy = (ReceiptService) AopContext.currentProxy();
        ArrayList<Long> receiptIds = new ArrayList<>();
        for (ReceiptCreateReqVO receiptCreateReqVO : createReqVOList) {
            try {
                Long receiptId = currentProxy.generateReceiptRelevantData(receiptCreateReqVO, true);
                receiptIds.add(receiptId);
            } catch (Exception e) {
                respVOS.add(new ReceiptBatchGenRespVO(null, receiptCreateReqVO.getCustomerName(), e.getMessage()));
            }
        }
        if (!respVOS.isEmpty()) {
            return respVOS;
        }
        return null;
    }

    /**
     * 组装批量生成收款单的参数
     *
     * @param batchCreateReqVOList
     * @return
     */
    private List<ReceiptCreateReqVO> assembleCreateReqVOList(List<ReceiptBatchCreateReqVO> batchCreateReqVOList) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        if (null == loginUser) {
            throw exception(USER_NOT_EXISTS);
        }
        ArrayList<ReceiptCreateReqVO> createReqVOs = new ArrayList<>();
        for (ReceiptBatchCreateReqVO receiptBatchCreateReqVO : batchCreateReqVOList) {
            OrderCustomerInfoDTO customerInfo = orderApi.createReceiptGetOrderCustomerInfo(receiptBatchCreateReqVO.getOrderId());
            ReceiptCreateReqVO receiptCreateReqVO = new ReceiptCreateReqVO();
            receiptCreateReqVO.setRateValidateDate(receiptBatchCreateReqVO.getRateValidateDate());
            receiptCreateReqVO.setCustomerId(customerInfo.getCustomerId());
            receiptCreateReqVO.setCustomerName(customerInfo.getCustomerName());
            receiptCreateReqVO.setSalesmanId(loginUser.getId());
            receiptCreateReqVO.setSalesmanName(loginUser.getUsername());
            receiptCreateReqVO.setDepartmentId(loginUser.getDeptId());
            receiptCreateReqVO.setRemark(receiptBatchCreateReqVO.getRemark());
            receiptCreateReqVO.setGeneratePath(1);
            receiptCreateReqVO.setAddType("0");
            // 设置开票信息
            receiptCreateReqVO.setOpenInvoice(receiptBatchCreateReqVO.getOpenInvoice());
            if (receiptBatchCreateReqVO.getOpenInvoice() == 1) {
                receiptCreateReqVO.setReceiptInvoice(ReceiptInvoiceConvert.INSTANCE.convertCreateReqVO(receiptBatchCreateReqVO));
            }
            List<ReceivableBackVO> receivable = receivableService.getReceivableByOrderId(receiptBatchCreateReqVO.getOrderId());
            if (CollectionUtil.isEmpty(receivable)) {
                throw exception(RECEIVABLE_NOT_EXISTS);
            }
            receiptCreateReqVO.setCurrencyId(receivable.get(0).getBaseCurrencyId());
            Map<Long, ReceiptAccountCreateReqVO> receiptAccountCreateReqVOS = new HashMap<>();
            receivable.forEach(receivableBackVO -> {
                // 设置应收信息
                BigDecimal taxPoint = receiptBatchCreateReqVO.getTaxPoint();
                if (taxPoint == null || taxPoint.compareTo(BigDecimal.ZERO) == 0) {
                    receivableBackVO.setTaxPoint(BigDecimal.ZERO);
                } else {
                    receivableBackVO.setTaxPoint(taxPoint);
                    receivableBackVO.setTaxAmount(receivableBackVO.getTaxPoint().divide(BigDecimal.valueOf(100)).multiply(receivableBackVO.getTotalAmount()).add(receivableBackVO.getTotalAmount()));
                    receivableBackVO.setActualAmount(receivableBackVO.getTaxAmount().subtract(receivableBackVO.getDiscountTotal()));
                }
                // 设置账单信息
                if (receiptAccountCreateReqVOS.containsKey(receivableBackVO.getCurrencyId())) {
                    ReceiptAccountCreateReqVO receiptAccountCreateReqVO = receiptAccountCreateReqVOS.get(receivableBackVO.getCurrencyId());
                    receiptAccountCreateReqVO.setReceivableAmount(receiptAccountCreateReqVO.getReceivableAmount().add(receivableBackVO.getActualAmount()));
                    receiptAccountCreateReqVO.setWriteOffAmount(receiptAccountCreateReqVO.getWriteOffAmount().add(receivableBackVO.getBaseAmount()));
                    receiptAccountCreateReqVO.setCollectionAmount(receiptAccountCreateReqVO.getCollectionAmount().add(receivableBackVO.getBaseAmount()));
                } else {
                    ReceiptAccountCreateReqVO receiptAccountCreateReqVO = new ReceiptAccountCreateReqVO();
                    receiptAccountCreateReqVO.setCurrencyId(receivableBackVO.getCurrencyId());
                    receiptAccountCreateReqVO.setReceivableAmount(receivableBackVO.getActualAmount());
                    receiptAccountCreateReqVO.setWriteOffRate(receivableBackVO.getExchangeRate());
                    receiptAccountCreateReqVO.setWriteOffAmount(receivableBackVO.getBaseAmount());
                    receiptAccountCreateReqVO.setCollectionAmount(receivableBackVO.getBaseAmount());
                    receiptAccountCreateReqVO.setCollectionCurrencyId(receivableBackVO.getBaseCurrencyId());
                    receiptAccountCreateReqVO.setCollectionRate(receivableBackVO.getExchangeRate());
                    if (receiptBatchCreateReqVO.getPlatformAccountId() != null) {
                        receiptAccountCreateReqVO.setPlatformAccountId(receiptBatchCreateReqVO.getPlatformAccountId().intValue());
                        BankAccountDTO account = bankApi.getBankAccountByAccountId(receiptBatchCreateReqVO.getPlatformAccountId());
                        receiptAccountCreateReqVO.setPlatformAccountNo(account.getBaAccountNum());
                        receiptAccountCreateReqVO.setPlatformAccountName(account.getBaAccountName());
                        receiptAccountCreateReqVO.setPlatformAccountBlankName(account.getBaBankName());
                    }
                    receiptAccountCreateReqVOS.put(receivableBackVO.getCurrencyId(), receiptAccountCreateReqVO);
                }
            });
            receiptCreateReqVO.setReceivableVOList(ReceivableConvert.INSTANCE.convertToUpdateVO(receivable));
            List<ReceiptAccountCreateReqVO> collect = new ArrayList<>(receiptAccountCreateReqVOS.values());
            receiptCreateReqVO.setReceiptAccountList(collect);
            createReqVOs.add(receiptCreateReqVO);
        }
        return createReqVOs;
    }

    /**
     * 生成收款单相关数据
     *
     * @param createReqVO 创建信息
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long generateReceiptRelevantData(ReceiptCreateReqVO createReqVO, boolean isBatch) {
        ReceiptDO receipt = ReceiptConvert.INSTANCE.convert(createReqVO);
        generateReceiptNo(receipt);
        receipt.setReceiptItemStatus(ReceiptItemStatusInReceiptEnum.NO_INPUT.getValue());
        receiptMapper.insert(receipt);
        recordLog(receipt.getId(), ReceiptLinkEnum.CREATE_RECEIPT, receipt.getRemark(), null);
        receivableAssocReceipt(createReqVO.getReceivableVOList(), receipt);
        genReceiptAccount(createReqVO.getReceiptAccountList(), receipt);
        // 需要开票时生成开票信息
        if (createReqVO.getOpenInvoice().equals(1)) {
            createInvoice(createReqVO.getReceiptInvoice(), receipt);
        } else {
            receipt.setInvoiceDataStatus(InvoiceDataStatusEnum.NO_AUDIT.getValue());
            receipt.setInvoicingStatus(InvoicingStatusEnum.NO_INVOICE.getValue());
        }

        if (StringUtils.equals(createReqVO.getAddType(), "1")) {
            receipt.setState(ReceiptStatusEnum.DRAFT.getValue());
            receiptMapper.updateById(receipt);
        }
        // 不是草稿创建收款单并发起审核流程
        else if (StringUtils.equals(createReqVO.getAddType(), "0")) {
            initApproval(createReqVO.getReason(), createReqVO.getCopyUserList(), receipt, isBatch);
            receiptMapper.updateById(receipt);
        }
        return receipt.getId();
    }

    /**
     * 生成收款单号
     *
     * @param receipt 收款单
     */
    private void generateReceiptNo(ReceiptDO receipt) {
        OrderNumberLogEvent event = new OrderNumberLogEvent();
        event.setType(3);
        event.setNumberKey(RECEIPT_NEW_NUMBER);
        applicationContext.publishEvent(event);
        String newReceiptNo = CodeUtils.getOddReceiptNumbers(event.getNumber());
        receipt.setReceiptNo(newReceiptNo);
        // 更新收款单号最新缓存
        // 待将最新单号持久化
        event.setNewNumber(newReceiptNo);
        applicationContext.publishEvent(event);
    }

    /**
     * 校验应收明细为同一客户同一自编号同目的国下的应收明细。
     * 当自编号为空（某些预付款未装柜的订单），则校验为同一客户同一目的国，忽略自编号规则，视为同一自编号
     *
     * @param receivableUpdateReqVOList 收款明细
     */
    private void validateSameCustomerReceivable(List<ReceivableUpdateReqVO> receivableUpdateReqVOList) {
        if (CollectionUtil.isEmpty(receivableUpdateReqVOList)) {
            throw exception(RECEIVABLE_NOT_NULL);
        }
        Long tempCustomerId = null;
        String tempGuojia = "";
        String tempNum = "";

        for (int i = 0; i < receivableUpdateReqVOList.size(); i++) {
            ReceivableUpdateReqVO temVo = receivableUpdateReqVOList.get(i);

            ReceivableDO receivableDO = receivableService.getReceivable(temVo.getId());

            if (null == receivableDO.getOrderId()) {
                throw exception(RECEIVABLE_ORDER_EXITS_NOT);
            }

            OrderVO orderDo = receiptMapper.getContainerNumberByOrderId(receivableDO.getOrderId());
            if (null == orderDo) {
                throw exception(RECEIVABLE_ORDER_EXITS_NOT);
            }

            if (i == 0) {
                tempCustomerId = orderDo.getCustomerId();//客户

                if (StrUtil.isNotBlank(orderDo.getContainerNumber())) {
                    tempNum = orderDo.getContainerNumber();//自编号
                }

                tempGuojia = orderDo.getCountry();
            } else {
                log.error(tempCustomerId + "");
                log.error(orderDo.getCustomerId() + "");
                if (!StringUtils.equals(tempCustomerId.toString(), orderDo.getCustomerId().toString())) {
                    throw exception(RECEIVABLE_CUSTOMER_DIFF);
                }

                if (StrUtil.isNotBlank(orderDo.getContainerNumber()) && StrUtil.isNotBlank(tempNum)) {
                    if (!StrUtil.equals(tempNum, orderDo.getContainerNumber())) {
                        throw exception(RECEIVABLE_CUSTOMER_DIFF);
                    }
                }

                if (StrUtil.isNotBlank(orderDo.getContainerNumber()) && StrUtil.isBlank(tempNum)) {
                    tempNum = orderDo.getContainerNumber();
                }

                if (!StrUtil.equals(tempGuojia, orderDo.getCountry())) {
                    throw exception(RECEIVABLE_CUSTOMER_DIFF);
                }
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReceipt(ReceiptUpdateReqVO updateReqVO) {
        ReceiptDO oldReceiptDO = validateUpdateVO(updateReqVO);
        List<ReceivableUpdateReqVO> receivableUpdateReqVOList = updateReqVO.getReceivableVOList();
        List<ReceiptAccountCreateReqVO> receiptAccountCreateReqVOList = updateReqVO.getReceiptAccountList();
        ReceiptDO updateObj = ReceiptConvert.INSTANCE.convert(updateReqVO);
        // 删除的应收明细取消关联收款单
        LambdaQueryWrapper<ReceivableDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceivableDO::getReceiptId, updateReqVO.getId());
        List<ReceivableDO> allReceivable = receivableService.list(lambdaQueryWrapper);
        // TODO 校验删除的应收明细是否已收款
        for (ReceivableDO receivableDO : allReceivable) {
            if (receivableUpdateReqVOList.stream().noneMatch(w -> Objects.equals(w.getId(), receivableDO.getId()))) {
                LambdaUpdateWrapper<ReceivableDO> update = new LambdaUpdateWrapper<>();
                update.set(ReceivableDO::getReceiptId, null);
                update.set(ReceivableDO::getReceiptNo, null);
                update.eq(ReceivableDO::getId, receivableDO.getId());
                receivableService.update(update);
            }
        }
        receivableAssocReceipt(receivableUpdateReqVOList, updateObj);

        // 删除旧的收款账户信息，保存新的收款账户信息
        LambdaQueryWrapper<ReceiptAccountDO> removeLambdaWrapper = new LambdaQueryWrapper();
        removeLambdaWrapper.eq(ReceiptAccountDO::getReceiptId, updateObj.getId());
        receiptAccountService.remove(removeLambdaWrapper);
        genReceiptAccount(receiptAccountCreateReqVOList, updateObj);

        // 需要开票时生成开票信息,删除旧的开票信息
        if (updateReqVO.getOpenInvoice().equals(1)) {
            if (oldReceiptDO.getInvoiceId() != null) {
                receiptInvoiceMapper.deleteById(oldReceiptDO.getInvoiceId());
            }
            createInvoice(updateReqVO.getReceiptInvoice(), updateObj);
        } else {
            if (oldReceiptDO.getInvoiceId() != null) {
                receiptInvoiceMapper.deleteById(oldReceiptDO.getInvoiceId());
            }
            updateObj.setInvoiceDataStatus(InvoiceDataStatusEnum.NO_AUDIT.getValue());
            updateObj.setInvoicingStatus(InvoicingStatusEnum.NO_INVOICE.getValue());
        }
        recordLog(updateObj.getId(), ReceiptLinkEnum.EDIT_RECEIPT, updateObj.getRemark(), null);
        if (StringUtils.equals(updateReqVO.getUpdateType(), "0")) {
            updateReqVO.setState(ReceiptStatusEnum.DRAFT.getValue());
        } else if (StringUtils.equals(updateReqVO.getUpdateType(), "1")) {
            //更新收款单并发起审核流程
            initApproval(updateReqVO.getReason(), updateReqVO.getCopyUserList(), updateObj, false);
        }
        receiptMapper.updateById(updateObj);
    }

    private ReceiptDO validateUpdateVO(ReceiptUpdateReqVO updateReqVO) {
        // 校验流程是不是正在审核中
        ReceiptDO receiptDO = validateReceiptBmpStatus(updateReqVO.getId());
        if (CollectionUtil.isEmpty(updateReqVO.getReceivableVOList())) {
            throw exception(RECEIVABLE_NOT_NULL);
        }
        if (CollectionUtil.isEmpty(updateReqVO.getReceiptAccountList())) {
            throw exception(RECEIPT_ACCOUNT_NOT_EXISTS);
        }
        validateSameCustomerReceivable(updateReqVO.getReceivableVOList());
        return receiptDO;
    }

    @Override
    @Transactional
    public void deleteReceipt(Long id) {
        // 校验存在
        this.validateReceiptExists(id);
        // 删除
        receiptMapper.deleteById(id);
        //删除收款单的同时同步删除收款明细
        LambdaQueryWrapper<ReceiptItemDO> removeItemWrapper = new LambdaQueryWrapper<>();
        removeItemWrapper.eq(ReceiptItemDO::getReceiptId, id);
        receiptItemMapper.delete(removeItemWrapper);
        //同步更新应收明细的收款单ID为NULL
        receivableService.lambdaUpdate().eq(ReceivableDO::getReceiptId, id).set(ReceivableDO::getReceiptId, null).set(ReceivableDO::getState, 0).set(ReceivableDO::getReceiptNo, null).update();
    }

    private void validateReceiptExists(Long id) {
        if (receiptMapper.selectById(id) == null) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
    }

    @Override
    public ReceiptDO getReceipt(Long id) {
        return receiptMapper.selectById(id);
    }

    @Override
    public List<ReceiptDO> getReceiptList(Collection<Long> ids) {
        return receiptMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ReceiptBackVO> getReceiptPage(ReceiptQueryVO query, PageVO page) {
        int start = (page.getPage() - 1) * page.getRows();
        int size = page.getRows();
        List<ReceiptBackVO> receiptBackPage = receiptMapper.getReceiptBackPage(start, size, query);
        if (receiptBackPage.isEmpty()) {
            return PageResult.empty();
        }
        int total = receiptMapper.countReceiptBackPage(query);
        receiptBackPage.forEach(receiptBackVO -> {
            // 根据收款单id获取关联订单信息
            List<ReceiptRelationOrderInfoVO> orderInfoList = receiptMapper.getOrderInfoByReceiptId(receiptBackVO.getId());
            receiptBackVO.setOrderInfoList(orderInfoList);
            if (receiptBackVO.getPayedAt() == null) {
                receiptBackVO.setPayedAt(receiptMapper.getReceiptItemPayat(receiptBackVO.getId()));
            }
        });
        List<Long> idList = receiptBackPage.stream().map(ReceiptBackVO::getId).collect(Collectors.toList());
        //一定要判空，不然会报错
        if (CollectionUtil.isNotEmpty(idList)) {
            //根据List获取本页中所有的明细信息
            LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.in(ReceiptItemDO::getReceiptId, idList);
            //所有收款单的明细
            List<ReceiptItemDO> listItem = receiptItemMapper.selectList(lambdaQueryWrapper);

            LambdaQueryWrapper<ReceiptAccountDO> lambdaQueryWrapper2 = new LambdaQueryWrapper<>();
            lambdaQueryWrapper2.in(ReceiptAccountDO::getReceiptId, idList);
            List<ReceiptAccountDO> ReceiptAccountList = receiptAccountMapper.selectList(lambdaQueryWrapper2);

            Map<Integer, CurrencyRespDTO> currencyMap = currencyApi.getAllCurrency();

            for (ReceiptBackVO r : receiptBackPage) {
                String sR = getReceiptMxStatus(listItem, r.getId()).getNameZh();
                r.setMxStatus(sR);

                //应收
                String s1 = getYSShow(ReceiptAccountList, r.getId(), currencyMap);
                r.setYsShow(s1);
                //实收
                String s2 = getSSShow(listItem, r.getId(), currencyMap);
                r.setSsShow(s2);

                //核销比例
                r.setHxBiLi(getHxBili(ReceiptAccountList, listItem, r.getId()));
            }
        }
        return new PageResult<>(receiptBackPage, total);
    }

    private String getPercentage(BigDecimal part, BigDecimal total) {
        BigDecimal percentage = part.divide(total, 5, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("100"));
        // 输出百分比
        String sR = format(percentage) + "%";
        return sR;
    }

    private String format(BigDecimal value) {
        NumberFormat numberFormat = NumberFormat.getInstance();
        numberFormat.setMinimumFractionDigits(2);
        numberFormat.setMaximumFractionDigits(2);
        return numberFormat.format(value.doubleValue());
    }

    /*
     获取核销比例
     */
    private String getHxBili(List<ReceiptAccountDO> ReceiptAccountList, List<ReceiptItemDO> listItemList, long ReceiptId) {
        String sR = "0";
        if (CollectionUtil.isEmpty(ReceiptAccountList)) return sR;
        if (CollectionUtil.isEmpty(listItemList)) {
            return sR;
        }
        List<ReceiptAccountDO> ReceiptAccountList1 = ReceiptAccountList.stream().filter(i -> i.getReceiptId() == ReceiptId).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(ReceiptAccountList1)) return sR;

        BigDecimal bTotal = new BigDecimal(0);
        BigDecimal bPart = new BigDecimal(0);
        for (ReceiptAccountDO do1 : ReceiptAccountList1) {
            bTotal = bTotal.add(do1.getWriteOffAmount());
        }

        //总值为0，直接返回空
        if (bTotal.compareTo(BigDecimal.ZERO) == 0) {
            return sR;
        }

        //已核销状态记录
        List<ReceiptItemDO> list = listItemList.stream().filter(i -> i.getReceiptId() == ReceiptId && (i.getStatus() == 1 || i.getStatus() == 3)).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(list)) {
            //核销金额为0 核销比例为空
            return sR;
        }

        for (ReceiptItemDO do2 : list) {
            bPart = bPart.add(do2.getWriteOffAmount());
        }
        //计算百分比
        sR = getPercentage(bPart, bTotal);
        return sR;
    }

    /*
     应收款
     */
    private String getYSShow(List<ReceiptAccountDO> ReceiptAccountList, long ReceiptId, Map<Integer, CurrencyRespDTO> currencyMap) {
        String sR = "";
        if (CollectionUtil.isEmpty(ReceiptAccountList)) {
            return sR;
        }

        List<ReceiptAccountDO> ReceiptAccountList1 = ReceiptAccountList.stream().filter(i -> i.getReceiptId() == ReceiptId).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(ReceiptAccountList1)) return sR;

        for (ReceiptAccountDO do1 : ReceiptAccountList1) {
            BigDecimal b1 = do1.getReceivableAmount();
            String s1 = b1.toString();
            String s2 = "";
            //获取币种
            if (do1.getCurrencyId() != null) {
                int n1 = Integer.parseInt(do1.getCurrencyId().toString());
                s2 = currencyMap.get(n1).getTitleZh();
            }
            sR += s1 + s2 + "\r\n";
        }
        return sR;
    }

    /*

     实收金额
     */
    private String getSSShow(List<ReceiptItemDO> listItemList, long ReceiptId, Map<Integer, CurrencyRespDTO> currencyMap) {
        String sR = "";
        if (CollectionUtil.isEmpty(listItemList)) {
            return sR;
        }

        List<ReceiptItemDO> list = listItemList.stream().filter(i -> i.getReceiptId() == ReceiptId).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(list)) return sR;

        //根据币种分组求和
        Map<Long, BigDecimal> dataMap = list.stream().filter(t -> t.getCurrencyId() != null).collect(Collectors.groupingBy(ReceiptItemDO::getCurrencyId, CollectorsUtil.summingBigDecimal(ReceiptItemDO::getAmount)));
        //遍历币种信息
        for (Map.Entry<Long, BigDecimal> entry : dataMap.entrySet()) {
            String key = entry.getKey().toString();
            String s1 = entry.getValue().toString();

            String s2 = "";
            //获取币种
            int n1 = Integer.parseInt(key);
            s2 = currencyMap.get(n1).getTitleZh();
            sR += s1 + s2 + "\r\n";
        }
        return sR;
    }

    /**
     * 计算收款单明细状态
     * 收款单主表
     * SELECT * from ecw_receipt where receipt_no='SKD00006547'
     * 收款单明细表
     * SELECT * from ecw_receipt_item where receipt_id=6554
     */
    private ReceiptItemStatusInReceiptEnum getReceiptMxStatus(List<ReceiptItemDO> listItem, long ReceiptId) {
        //未录入 收款单下无收款明细 OK
        //未提交 ：收款单下有收款明细且全未提交 OK
        //部分提交  收款单下有收款明细且部分提交(无需判断收款明细是否审批) OK
        //未审核  收款明细全部提交且全部未审批  OK
        //部分审核  收款明细全部提交且部分已审批  OK
        //全部审核  收款明细全部已审批  OK
        ReceiptItemStatusInReceiptEnum sMxStatus = null;
        //计算明细总数
        Long iCount = listItem.stream().filter(i -> i.getReceiptId() == ReceiptId).collect(Collectors.counting());
        if (iCount == 0) {
            sMxStatus = ReceiptItemStatusInReceiptEnum.NO_INPUT;
            return sMxStatus;
        } else {
            //存在记录就判断状态
            //status 状态0待核销，待提交 1已核销
            //2核销审核中，3反核销审核中
            //bmpStatus
            //PROCESS(1, "处理中"),APPROVE(2, "通过"),
            //REJECT(3, "不通过"),CANCEL(4, "已取消");
            long Item0 = listItem.stream().filter(i -> i.getStatus() == 0 || i.getStatus() == 4).filter(i -> i.getReceiptId() == ReceiptId).collect(Collectors.counting());
            long Item1 = listItem.stream().filter(i -> i.getStatus() == 1).filter(i -> i.getReceiptId() == ReceiptId).collect(Collectors.counting());
            long Item2 = listItem.stream().filter(i -> i.getStatus() == 2).filter(i -> i.getReceiptId() == ReceiptId).collect(Collectors.counting());
            long Item3 = listItem.stream().filter(i -> i.getStatus() == 3).filter(i -> i.getReceiptId() == ReceiptId).collect(Collectors.counting());

            if (Item0 == iCount) {
                //0待核销
                //收款单下有收款明细且全未提交
                sMxStatus = ReceiptItemStatusInReceiptEnum.UN_SUBMITTED;
                return sMxStatus;
            }

            if (Item0 > 0 && Item0 < iCount) {
                //部分提交  收款单下有收款明细且部分提交(无需判断收款明细是否审批)
                sMxStatus = ReceiptItemStatusInReceiptEnum.PART_SUBMITTED;
                return sMxStatus;
            }

            //进部分提交，部分审核逻辑
            //未审核  收款明细全部提交且全部未审批
            if (Item2 == iCount) {
                sMxStatus = ReceiptItemStatusInReceiptEnum.UN_APPROVED;
                return sMxStatus;
            }

            if (Item1 == iCount) {
                //1已核销
                sMxStatus = ReceiptItemStatusInReceiptEnum.ALL_APPROVED;
                return sMxStatus;
            }

            if ((Item2 + Item3) == iCount || (Item1 + Item3) == iCount || (Item2 + Item1) == iCount) {
                sMxStatus = ReceiptItemStatusInReceiptEnum.PART_APPROVED;
                return sMxStatus;
            }
        }
        return sMxStatus;
    }

    @Override
    public List<ReceiptDO> getReceiptList(ReceiptQueryVO query) {
        return receiptMapper.selectList(query);
    }

    @Override
    public void verification(Long id) {
        ReceiptDO receiptDO = receiptMapper.selectById(id);
        if (receiptDO == null) {
            throw exception(RECEIPTITEM_NOT_EXISTS);
        }
        if (receiptDO.getState() == null || receiptDO.getState() == 1) {
            ReceiptDO updateReceiptDO = new ReceiptDO();
            updateReceiptDO.setId(receiptDO.getId());
            if (receiptDO.getOpenInvoice() == 0) {
                updateReceiptDO.setState(2);
            }
            if (receiptDO.getOpenInvoice() == 1) {
                updateReceiptDO.setState(3);
            }
            receiptMapper.updateById(updateReceiptDO);
        } else {
            throw exception(RECEIPT_VERIFICATION);
        }
    }

    @Override
    public void verificationCancel(Long id) {
        ReceiptDO receiptDO = receiptMapper.selectById(id);
        if (receiptDO == null) {
            throw exception(RECEIPTITEM_NOT_EXISTS);
        }
        if (receiptDO.getState() == null || receiptDO.getState() == 1) {
            throw exception(RECEIPT_VERIFICATION_NOT);
        } else {
            ReceiptDO updateReceiptDO = new ReceiptDO();
            updateReceiptDO.setId(receiptDO.getId());
            updateReceiptDO.setState(1);
            receiptMapper.updateById(updateReceiptDO);
        }

    }

    @Override
    public void createFinanceApproveNo(FinanceReceiptVo financeReceiptVo) {
        validateReceiptBmpStatus(financeReceiptVo.getReceiptId());
        //创建收款单反审核流程
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(financeReceiptVo.getReceiptId());
        approvalDO.setReceiptNo(financeReceiptVo.getReceiptNo());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_APPROVE_NO.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(financeReceiptVo.getRemark());
        receiptApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_APPROVE_NO.getKey(), financeReceiptVo.getReceiptNo(), financeReceiptVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        receiptApprovalService.updateById(approvalDO);

        ReceiptDO update = new ReceiptDO();
        update.setId(financeReceiptVo.getReceiptId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setState(ReceiptStatusEnum.APPROVE_NEGATION.getValue());
        update.setBmpId(bpmId);
        receiptMapper.updateById(update);
    }

    @Override
    public void createFinanceReceiptWriteOff(FinanceReceiptVo financeReceiptVo) {

        ReceiptDO receiptDO = validateReceiptBmpStatus(financeReceiptVo.getReceiptId());
        //判断收款单下的收款明细是不是全部核销完成
        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceiptItemDO::getReceiptId, financeReceiptVo.getReceiptId());
        lambdaQueryWrapper.in(ReceiptItemDO::getStatus, 0, 2, 3);
        Long count = receiptItemMapper.selectCount(lambdaQueryWrapper);
        if (count != 0) {
            throw exception(RECEIPT_ITEM_WRITE_OFF_ING);
        }

        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper1 = new LambdaQueryWrapper<>();
        lambdaQueryWrapper1.eq(ReceiptItemDO::getReceiptId, financeReceiptVo.getReceiptId());
        lambdaQueryWrapper1.eq(ReceiptItemDO::getStatus, 1);
        List<ReceiptItemDO> receiptItemDOList = receiptItemMapper.selectList(lambdaQueryWrapper1);
        if (null == receiptItemDOList || receiptItemDOList.size() == 0) {
            throw exception(RECEIPT_NOT_WRITE_OFF_ITEM_NOT_EXISTS);
        }

        BigDecimal totalAmount = receiptItemDOList.stream().map(ReceiptItemDO::getWriteOffAmount).reduce(BigDecimal.ZERO, BigDecimal::add);

        String financeRemark = "";
        Long currencyId = receiptDO.getCurrencyId();

        if (null == currencyId) {
            throw exception(RECEIPT_CURRENCY_NOT_EXISTS);
        }
        BigDecimal diff = null;
        if (totalAmount.compareTo(receiptDO.getReceivableTotalAmount()) < 0) {
            DictDataRespDTO dictDataRespDTO = dictDataApi.parseDictDataFromCache("write_off_diff", String.valueOf(currencyId));
            if (null == dictDataRespDTO) {
                throw exception(RECEIPT_CURRENCY_DIFF_NOT_EXISTS);
            }
            //核销差额大于配置提示
            diff = receiptDO.getReceivableTotalAmount().subtract(totalAmount);
            if (diff.compareTo(new BigDecimal(dictDataRespDTO.getValue())) > 0) {
                throw exception(RECEIPT_CURRENCY_DIFF_BIG);
            } else {
                String message = "收款单已全部核销，总金额差值小于{}{},允许全部核销，核销比例应为100%";
                Map<Integer, CurrencyRespDTO> currencyMap = currencyApi.getAllCurrency();
                CurrencyRespDTO currencyRespDTO = currencyMap.get(currencyId.intValue());
                financeRemark = StrUtil.format(message, dictDataRespDTO.getValue(), currencyRespDTO.getFuhao());
            }
        }

        //创建收款单核销流程
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(financeReceiptVo.getReceiptId());
        approvalDO.setReceiptNo(financeReceiptVo.getReceiptNo());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(financeReceiptVo.getRemark());
        receiptApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF.getKey(), financeReceiptVo.getReceiptNo(), financeReceiptVo.getCopyUserList());

        approvalDO.setBmpId(bpmId);
        receiptApprovalService.updateById(approvalDO);

        ReceiptDO update = new ReceiptDO();
        update.setId(financeReceiptVo.getReceiptId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        //计算出核销比例
        BigDecimal proportion = totalAmount.divide(receiptDO.getReceivableTotalAmount(), 8, RoundingMode.HALF_UP).multiply(new BigDecimal("100")).setScale(6, BigDecimal.ROUND_HALF_UP);
        update.setWriteOffProportion(proportion);
        update.setState(ReceiptStatusEnum.WRITE_OFF_ALL_ING.getValue());
        update.setBmpId(bpmId);
        update.setFinanceRemark(financeRemark);
        // 更新核销误差兑额外费用主币种金额
        if (null == diff) {
            diff = totalAmount.subtract(receiptDO.getReceivableTotalAmount());
        }
        List<ReceiptRelationOrderInfoVO> relationOrder = receiptMapper.getOrderInfoByReceiptId(receiptDO.getId());
        OrderVO orderVO = receiptMapper.getContainerNumberByOrderId(relationOrder.get(0).getOrderId());
        if (null != orderVO) {
            RegionDTO region = regionApi.getRegionById(Long.parseLong(orderVO.getCountry()));
            Long importCurrency3 = Long.parseLong(region.getImportCurrency3());
            if (!currencyId.equals(importCurrency3)) {
                diff = diff.multiply(currencyApi.getCurrencyRate(currencyId, importCurrency3).getCurrencyRate());
            }
            update.setWriteOffDiffMainCurrency(diff.setScale(2, RoundingMode.HALF_UP) + " " + currencyApi.getAllCurrency().get(importCurrency3.intValue()).getFuhao());
        }
        receiptMapper.updateById(update);
    }

    @Override
    public void createFinanceReceiptWriteOffNo(FinanceReceiptVo financeReceiptVo) {
        validateReceiptBmpStatus(financeReceiptVo.getReceiptId());
        //创建收款单反核销流程
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(financeReceiptVo.getReceiptId());
        approvalDO.setReceiptNo(financeReceiptVo.getReceiptNo());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF_NO.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(financeReceiptVo.getRemark());
        receiptApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF_NO.getKey(), financeReceiptVo.getReceiptNo(), financeReceiptVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        receiptApprovalService.updateById(approvalDO);

        ReceiptDO update = new ReceiptDO();
        update.setId(financeReceiptVo.getReceiptId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setState(ReceiptStatusEnum.WRITE_OFF_ALL_NEGATION.getValue());
        update.setBmpId(bpmId);
        receiptMapper.updateById(update);
    }

    @Override
    public void cancelFinanceReceiptApproval(FinanceReceiptCancelVo financeReceiptCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptDO receiptDO = validateReceiptBmpCancel(financeReceiptCancelVo.getReceiptId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptDO.getBmpId(), financeReceiptCancelVo.getRemark());
    }

    @Override
    public void cancelFinanceReceiptApprovalNo(FinanceReceiptCancelVo financeReceiptCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptDO receiptDO = validateReceiptBmpCancel(financeReceiptCancelVo.getReceiptId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptDO.getBmpId(), financeReceiptCancelVo.getRemark());
    }

    @Override
    public void cancelFinanceReceiptWriteOff(FinanceReceiptCancelVo financeReceiptCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptDO receiptDO = validateReceiptBmpCancel(financeReceiptCancelVo.getReceiptId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptDO.getBmpId(), financeReceiptCancelVo.getRemark());
    }

    @Override
    public void cancelFinanceReceiptWriteOffNo(FinanceReceiptCancelVo financeReceiptCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptDO receiptDO = validateReceiptBmpCancel(financeReceiptCancelVo.getReceiptId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptDO.getBmpId(), financeReceiptCancelVo.getRemark());
    }

    private ReceiptDO validateReceiptBmpStatus(Long id) {
        ReceiptDO receiptDO = receiptMapper.selectById(id);
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        if (null != receiptDO.getBmpStatus() && receiptDO.getBmpStatus().equals(BpmProcessInstanceResultEnum.PROCESS.getResult())) {
            throw exception(RECEIPT_APPROVING);
        }

        if (Objects.equals(receiptDO.getState(), ReceiptStatusEnum.WRITE_OFF_ALL_ING.getValue())) {
            throw exception(RECEIPT_APPROVING);
        }
        return receiptDO;
    }

    private ReceiptDO validateReceiptBmpCancel(Long id) {
        ReceiptDO receiptDO = receiptMapper.selectById(id);
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        if (null == receiptDO.getBmpStatus()) {

            throw exception(BMP_NOT);
        }

        if (receiptDO.getBmpStatus() != BpmProcessInstanceResultEnum.PROCESS.getResult()) {
            throw exception(BMP_CANCEL);
        }

        return receiptDO;
    }

    @Override
    public ReceiptExportDto getReceiptExportByOrderId(Long orderId) {
        return receiptMapper.getReceiptExportByOrderId(orderId);
    }

    @Override
    public Boolean judgingProportion(ReceiptDO receiptDO, String proportion) {
        //判断收款单下的收款明细是不是全部核销
        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceiptItemDO::getReceiptId, receiptDO.getId());
        lambdaQueryWrapper.in(ReceiptItemDO::getStatus, 0, 2, 3);
        Long count = receiptItemMapper.selectCount(lambdaQueryWrapper);
        if (count != 0) {
            return Boolean.FALSE;
        }


        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper1 = new LambdaQueryWrapper<>();
        lambdaQueryWrapper1.eq(ReceiptItemDO::getReceiptId, receiptDO.getId());
        lambdaQueryWrapper1.eq(ReceiptItemDO::getStatus, 1);
        List<ReceiptItemDO> receiptItemDOList = receiptItemMapper.selectList(lambdaQueryWrapper1);
        if (null == receiptItemDOList || receiptItemDOList.size() == 0) {
            return Boolean.FALSE;
        }


        BigDecimal totalAmount = receiptItemDOList.stream().map(ReceiptItemDO::getWriteOffAmount).reduce(BigDecimal.ZERO, BigDecimal::add);

        Long currencyId = receiptDO.getCurrencyId();

        if (null == currencyId) {
            log.error("自动核销判断核销币种为空");
            return Boolean.FALSE;
        }

        //计算出实际收款比例
        BigDecimal sjbl = NumberUtil.div(totalAmount, receiptDO.getReceivableTotalAmount()).multiply(new BigDecimal("100"));
        //实际收款比例大于规定阀值时,反回可以自己动核销
        if (sjbl.compareTo(new BigDecimal(proportion)) >= 0) {
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void autoWriteOffReceipt(ReceiptDO receiptDO, String userId, String userName) {
        ReceiptDO updateReceipt = new ReceiptDO();
        // 添加订单ID的修改，不然订单ID就丢失了
        // updateReceipt.setOrderId(receiptDO.getOrderId());
        updateReceipt.setId(receiptDO.getId());

        updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_ALL_SUCCESS.getValue());

        updateReceipt.setWriteOffAt(new Date());
        updateReceipt.setWriteOffProportion(new BigDecimal("100"));//设置核销比例
        updateReceipt.setRemark("自动核销,核销比例>=100%");
        updateReceipt.setWriteOffId(Long.parseLong(userId));
        updateReceipt.setWriteOffName(userName);
        receiptMapper.updateById(updateReceipt);

        //更新收该收款单下的应收为收款完成状态
        LambdaUpdateWrapper<ReceivableDO> lambdaUpdateWrapper = new LambdaUpdateWrapper();
        lambdaUpdateWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
        lambdaUpdateWrapper.set(ReceivableDO::getState, 2);
        receivableService.update(lambdaUpdateWrapper);

        //添加订单核销日志
        LambdaQueryWrapper<ReceivableDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
        List<ReceivableDO> orderList = receivableService.list(lambdaQueryWrapper);
        List orderIds = orderList.stream().map(ReceivableDO::getOrderId).distinct().collect(Collectors.toList());
        receivableService.orderReceivableWriteOffLogAdd(orderIds, receiptDO.getReceiptNo());
    }

    @Override
    public ReceiptBankWriteoffRangeVO getBankWriteoffRange() {
        ReceiptBankWriteoffRangeDO rangeDO = receiptBankWriteoffRangeMapper.selectOne("id", 1);
        return BeanUtil.toBean(rangeDO, ReceiptBankWriteoffRangeVO.class);
    }

    @Override
    public Boolean setBankWriteoffRange(ReceiptBankWriteoffRangeVO range) {
        receiptBankWriteoffRangeMapper.updateById(BeanUtil.toBean(range, ReceiptBankWriteoffRangeDO.class));
        return true;
    }

    @Override
    public void recordLog(Long receiptId, ReceiptLinkEnum linkEnum, String remark, String bpmStatus) {
        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String username = "";
        if (user != null && user.getUsername() != null) {
            username = user.getUsername();
        }
        ReceiptLogDO receiptLogDO = ReceiptLogDO.builder()
                .receiptId(receiptId)
                .operator(username)
                .remark(remark)
                .receiptLink(linkEnum.getValue())
                .bpmStatus(bpmStatus)
                .createTime(new Date())
                .build();
        receiptLogMapper.insert(receiptLogDO);
    }

    @Override
    public ReceiptBackVO getReceiptDetail(Long id) {
        ReceiptDO receiptDO = receiptMapper.selectById(id);
        if (receiptDO == null) {
            return null;
        }
        ReceiptBackVO backVO = ReceiptConvert.INSTANCE.convert(receiptDO);
        List<ReceiptLogDO> receiptLogDOS = receiptLogMapper.selectList(ReceiptLogDO::getReceiptId, id);
        if (receiptLogDOS != null && !receiptLogDOS.isEmpty()) {
            List<ReceiptLogBackVO> receiptLogVOS = ReceiptConvert.INSTANCE.convertLogList(receiptLogDOS);
            backVO.setLogList(receiptLogVOS);
        }
        if (receiptDO.getOpenInvoice() != null && receiptDO.getOpenInvoice() == 1 && receiptDO.getInvoiceId() != null) {
            ReceiptInvoiceDO receiptInvoiceDO = receiptInvoiceMapper.selectById(receiptDO.getInvoiceId());
            if (receiptInvoiceDO != null) {
                InvoiceInReceiptRespVO invoiceInReceiptRespVO = ReceiptInvoiceConvert.INSTANCE.convertRespVO(receiptInvoiceDO);
                if (receiptInvoiceDO.getInvoicerId() != null) {
                    invoiceInReceiptRespVO.setInvoicerName(adminUserApi.getUserName(receiptInvoiceDO.getInvoicerId().toString()));
                }
                invoiceInReceiptRespVO.setInvoice(receiptInvoiceDO.getInvoiceHeader());
                invoiceInReceiptRespVO.setAddressPhone(receiptInvoiceDO.getInvoiceAddress());
                invoiceInReceiptRespVO.setProjectName(receiptInvoiceDO.getInvoiceItem());
                backVO.setInvoiceVO(invoiceInReceiptRespVO);
            }
        }
        if (receiptDO.getOpenInvoice() != null && receiptDO.getOpenInvoice() == 0) {
            backVO.setInvoiceId(null);
        }
        backVO.setCreator(adminUserApi.getUserName(receiptDO.getCreator()));
        backVO.setUpdater(adminUserApi.getUserName(receiptDO.getUpdater()));
        return backVO;
    }

    @Override
    public String getReceiptApprover(Long receiptId) {
        List<ReceiptApprovalDO> approvalDOS = receiptApprovalService.selectList(Wrappers.lambdaQuery(ReceiptApprovalDO.class)
                .eq(ReceiptApprovalDO::getReceiptId, receiptId)
                .eq(ReceiptApprovalDO::getStatus, BpmProcessInstanceResultEnum.APPROVE.getResult())
                .eq(ReceiptApprovalDO::getBmpKey, WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getKey()));
        if (CollectionUtil.isEmpty(approvalDOS)) {
            return null;
        }
        ReceiptApprovalDO receiptApprovalDO = approvalDOS.stream().max(Comparator.comparing(ReceiptApprovalDO::getCreateTime)).orElse(null);
        AdminUserRespDTO user = adminUserApi.getUser(Long.parseLong(receiptApprovalDO.getUpdater()));
        return user.getNickname();
    }

    /**
     * 更新收款单明细状态
     */
    @Override
    public void updateReceiptItemStatus(Long receiptId) {
        List<ReceiptItemDO> listItem = receiptItemMapper.selectList(ReceiptItemDO::getReceiptId, receiptId);
        //ReceiptItemStatusInReceiptEnum receiptItemStatus = null;
        //int countItem = listItem.size();
        //if (countItem == 0) {
        //    receiptItemStatus = ReceiptItemStatusInReceiptEnum.NO_INPUT;
        //} else {
        //    long draft = listItem.stream().filter(i -> Objects.equals(i.getStatus(), DRAFT.getValue()) || Objects.equals(i.getStatus(), WRITE_OFF_APPROVE_REJECT.getValue())).count();
        //    long writeOff = listItem.stream().filter(i -> Objects.equals(i.getStatus(), WRITE_OFF.getValue())).count();
        //    long writeOffApproving = listItem.stream().filter(i -> Objects.equals(i.getStatus(), WRITE_OFF_APPROVE_ING.getValue())).count();
        //    long writeOffNoApproving = listItem.stream().filter(i -> Objects.equals(i.getStatus(), WRITE_OFF_NO_APPROVE_ING.getValue())).count();
        //    if (draft == countItem) {
        //        receiptItemStatus = ReceiptItemStatusInReceiptEnum.UN_SUBMITTED;
        //    } else if (draft > 0 && draft < countItem) {
        //        receiptItemStatus = ReceiptItemStatusInReceiptEnum.PART_SUBMITTED;
        //    } else if (writeOffApproving == countItem) {
        //        receiptItemStatus = ReceiptItemStatusInReceiptEnum.UN_APPROVED;
        //    } else if (writeOff == countItem) {
        //        receiptItemStatus = ReceiptItemStatusInReceiptEnum.ALL_APPROVED;
        //    } else if ((writeOffApproving + writeOffNoApproving) == countItem || (writeOff + writeOffNoApproving) == countItem || (writeOffApproving + writeOff) == countItem) {
        //        receiptItemStatus = ReceiptItemStatusInReceiptEnum.PART_APPROVED;
        //    }
        //}
        //if (receiptItemStatus != null) {
        //    ReceiptDO receiptDO = new ReceiptDO();
        //    receiptDO.setId(receiptId);
        //    receiptDO.setReceiptItemStatus(receiptItemStatus.getValue());
        //    receiptMapper.updateById(receiptDO);
        //}
        ReceiptItemStatusInReceiptEnum receiptMxStatus = getReceiptMxStatus(listItem, receiptId);
        if (receiptMxStatus != null) {
            ReceiptDO receiptDO = new ReceiptDO();
            receiptDO.setId(receiptId);
            receiptDO.setReceiptItemStatus(receiptMxStatus.getValue());
            receiptMapper.updateById(receiptDO);
        }
    }
}
