package cn.iocoder.yudao.module.wealth.service.receiptInvoice;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptInvoice.ReceiptInvoiceDO;
import cn.iocoder.yudao.module.wealth.vo.receiptInvoice.*;

/**
 * 收款单开票信息 Service 接口
 *
 * @author 系统管理员
 */
public interface ReceiptInvoiceService extends IService<ReceiptInvoiceDO> {

    /**
     * 创建收款单开票信息
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createReceiptInvoice(@Valid InvoiceCreateReqVO createReqVO);

    /**
     * 更新收款单开票信息
     * @param updateReqVO 更新信息
     */
    void updateReceiptInvoiceData(@Valid ReceiptInvoiceUpdateDataReqVO updateReqVO);

    /**
     * 获得收款单开票信息
     *
     * @param id 编号
     * @return 收款单开票信息
     */
    ReceiptInvoiceDetailVO getReceiptInvoiceDetail(Long id);

    /**
     * 获得收款单开票信息列表
     * @param ids 编号
     * @return 收款单开票信息列表
     */
    List<ReceiptInvoiceDO> getReceiptInvoiceList(Collection<Long> ids);

    /**
     * 获得收款单开票信息分页
     * @param page 分页查询
     * @param query 查询
     * @return 收款单开票信息分页
     */
    PageResult<ReceiptInvoicePageBackVO> getReceiptInvoicePage(ReceiptInvoiceQueryVO query, PageVO page);

    /**
     * 获得收款单开票信息列表, 用于 Excel 导出
     * @param query 查询
     * @return 收款单开票信息列表
     */
    List<ReceiptInvoiceDO> getReceiptInvoiceList(ReceiptInvoiceQueryVO query);

    Boolean rejectInvoiceInfo(@Valid ReceiptInvoiceOperateReqVO reqVO);

    Boolean cancelInvoice(@Valid ReceiptInvoiceOperateReqVO reqVO);

    Boolean deleteInvoiceInfo(@Valid ReceiptInvoiceOperateReqVO reqVO);

    void updateReceiptInvoiceInfo(@Valid ReceiptInvoiceUpdateInfoReqVO updateReqVO);

    ReceiptInvoiceDetailVO getReceiptDetailForCreateInvoice(Long receiptId);
}
