package cn.iocoder.yudao.module.wealth.service.receiptItem;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.io.FileModel;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import cn.iocoder.yudao.module.wealth.dal.mysql.receipt.ReceiptMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.BankReceiptDetailsMapper;
import cn.iocoder.yudao.module.wealth.vo.receipt.ReceiptRelationOrderInfoVO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Validated
@Slf4j
public class BankReceiptDetailsImpl implements BankReceiptDetailsService {

    @Resource
    private BankReceiptDetailsMapper bankReceiptDetailsMapper;
    @Resource
    private AdminUserService adminUserService;
    @Resource
    private ReceiptMapper receiptMapper;

    public PageResult<ReceiptItemRespVO> getBankReceiptDetailsList(ReceiptItemReq req) {
        IPage<ReceiptItemRespVO> mpPage = MyBatisUtils.buildPage(req);
        long total = bankReceiptDetailsMapper.GetCount(req);
        if (total == 0) {
            return PageResult.empty();
        }
        int start = (req.getPageNo() - 1) * req.getPageSize();

        req.setStart(start);
        List<AdminUserDO> ListUser = adminUserService.getUsersList();
        Map<Long, String> userMap = ListUser.stream().collect(Collectors.toMap((AdminUserDO::getId), AdminUserDO::getNickname));

        List<ReceiptItemRespVO> list = bankReceiptDetailsMapper.getBankReceiptDetailsList(req);
        for (ReceiptItemRespVO r : list) {
            if (r.getCreator() == null || r.getCreator().isEmpty()) {
                r.setCreator("");
            } else {
                Long l = Long.parseLong(r.getCreator());
                r.setCreator(userMap.get(l));
            }
            r.setAttrList(getFileList(r.getAttr()));

            if (r.getCreateBpm() == null || r.getCreator().isEmpty()) {
                r.setCreateBpm("");
            } else {
                Long l2 = Long.parseLong(r.getCreateBpm());
                r.setCreateBpm(userMap.get(l2));
            }
            List<ReceiptRelationOrderInfoVO> orderInfoList = receiptMapper.getOrderInfoByReceiptId(r.getReceiptId());
            r.setOrderInfoList(orderInfoList);
        }
        return new PageResult<>(list, total, mpPage.getSize(), req.getPageNo(), (total + mpPage.getSize() - 1) / mpPage.getSize());
    }

    @Override
    public BankIncomePageResult<BankIncomeItemResp> getBankIncomeItemPage(BankIncomeItemReq req) {
        if (req.getIsIncome() != null && req.getIsIncome() == 1) {
            return BankIncomePageResult.empty();
        }
        IPage<BankIncomeItemResp> mpPage = MyBatisUtils.buildPage(req);
        bankReceiptDetailsMapper.getBankIncomeItemPage(mpPage, req);
        if (mpPage.getTotal() == 0) {
            return BankIncomePageResult.empty();
        }
        // 计算账号余额
        List<BankIncomeItemResp> records = mpPage.getRecords();
        HashMap<String, List<CurrencyAmount>> accountBalanceMap = new HashMap<>();
        for (BankIncomeItemResp record : records) {
            // 目前只有收
            record.setIsIncome(0);
            if (accountBalanceMap.containsKey(record.getAccountNo())) {
                record.setAccountBalance(accountBalanceMap.get(record.getAccountNo()));
            } else {
                record.setAccountBalance(bankReceiptDetailsMapper.calculateAccountBalance(record.getAccountId()));
                accountBalanceMap.put(record.getAccountNo(), record.getAccountBalance());
            }
            List<ReceiptRelationOrderInfoVO> orderInfoByReceiptId = receiptMapper.getOrderInfoByReceiptId(Long.parseLong(record.getReceiptId()));
            record.setOrderInfoList(orderInfoByReceiptId);
        }
        // 汇总不同币种总额
        BankIncomePageResult<BankIncomeItemResp> result = BankIncomePageResult.of(mpPage);
        result.setCurrentAmount(bankReceiptDetailsMapper.calculateCurrentAmount(req));
        result.setTotalAmount(bankReceiptDetailsMapper.calculateTotalAmount());
        return result;
    }

    /*
     组装附件信息
     */
    private List<FileModel> getFileList(String sFileStr) {
        if (sFileStr == null || sFileStr.length() == 0) {
            return null;
        } else {
            String[] l = sFileStr.split(",");
            List<FileModel> l1 = new ArrayList<>();
            for (String f : l
            ) {
                FileModel m = new FileModel();
                m.setName(getFileName(f));
                m.setUrl(f);
                l1.add(m);
            }
            return l1;
        }
    }

    /*
     分解Url路径
     */
    public String getFileName(String s) {
        //String s="https://jiedao-test1.oss-cn-shenzhen.aliyuncs.com/2024/05/31/66592b5b556179746796617e.jpg";
        int l = s.lastIndexOf('/');
        String n = s.substring(l + 1);
        return n;
    }
}
