package cn.iocoder.yudao.module.wealth.service.receiptItem;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.boot.module.order.api.OrderApi;
import cn.iocoder.boot.module.order.api.dto.OrderRespDTO;
import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.enums.WorkFlowEmus;
import cn.iocoder.yudao.module.ecw.api.bank.BankApi;
import cn.iocoder.yudao.module.ecw.api.bank.dto.BankAccountDTO;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.ecw.api.currency.dto.ExchangeRateRespDTO;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.wealth.convert.receiptItem.ReceiptItemConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptBankWriteoffRangeDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptAccount.ReceiptAccountDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptApproval.ReceiptApprovalDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableWriteOffRecordDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receipt.ReceiptBankWriteoffRangeMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receipt.ReceiptMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptAccount.ReceiptAccountMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.ReceiptItemMapper;
import cn.iocoder.yudao.module.wealth.enums.ReceiptItemStatusEnum;
import cn.iocoder.yudao.module.wealth.enums.ReceiptLinkEnum;
import cn.iocoder.yudao.module.wealth.enums.ReceiptStatusEnum;
import cn.iocoder.yudao.module.wealth.service.receipt.ReceiptService;
import cn.iocoder.yudao.module.wealth.service.receiptApproval.ReceiptApprovalService;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.util.WealthGenCodeUtils;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptItemCancelVo;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptItemVo;
import cn.iocoder.yudao.module.wealth.vo.receipt.ReceiptRelationOrderInfoVO;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptItemBatchCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptItemBatchRespVO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.*;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableInItemVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableWriteOffReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.jetbrains.annotations.Nullable;
import org.springframework.aop.framework.AopContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;

/**
 * 收款单明细 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
@Slf4j
public class ReceiptItemServiceImpl extends AbstractService<ReceiptItemMapper, ReceiptItemDO> implements ReceiptItemService {

    @Resource
    private ReceiptItemMapper receiptItemMapper;
    @Resource
    private ReceiptService receiptService;
    @Resource
    private ReceiptMapper receiptMapper;
    @Resource
    private ReceivableService receivableService;
    @Resource
    private DictDataApi dictDataApi;
    @Resource
    private BpmCreateServiceFactory bpmCreateServiceFactory;
    @Resource
    private ReceiptApprovalService receiptApprovalService;
    @Resource
    private WealthGenCodeUtils wealthGenCodeUtils;
    @Resource
    private OrderApi orderApi;
    @Resource
    private CurrencyApi currencyApi;
    @Resource
    private BankApi bankApi;
    @Resource
    private ReceiptBankWriteoffRangeMapper receiptBankWriteoffRangeMapper;
    @Resource
    private ReceiptAccountMapper receiptAccountMapper;
    @Resource
    private BpmProcessInstanceApi bpmProcessInstanceApi;
    @Resource
    private AdminUserApi adminUserApi;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createReceiptItem(ReceiptItemCreateReqVO createReqVO, boolean isBatch) {
        ReceiptDO receiptDO = validateCreateVO(createReqVO);
        // 插入收款明细
        ReceiptItemDO receiptItem = ReceiptItemConvert.INSTANCE.convert(createReqVO);
        receiptItem.setSerialNumber(wealthGenCodeUtils.generateReceiptItemCode());
        receiptItemMapper.insert(receiptItem);
        // 生成应收明细核销记录
        BankAccountDTO account = bankApi.getBankAccountByAccountId(receiptItem.getAccountId());
        Integer incomeBelong = account.getBaIncomeBelong();
        for (ReceivableWriteOffReqVO receivableWriteOffReqVO : createReqVO.getReceivableWriteOffList()) {
            receivableService.createWriteOffRecord(receivableWriteOffReqVO, receiptItem, incomeBelong);
        }
        // 创建收款明细的时候要判断收款单状态进行更新
        if (receiptDO.getState() == 1) {
            ReceiptDO update = new ReceiptDO();
            update.setId(receiptDO.getId());
            update.setPayedAt(new Date());
            update.setState(2);
            receiptService.updateById(update);
        }
        receiptService.updateReceiptItemStatus(receiptItem.getReceiptId());
        if (isBatch) {
            createFinanceReceiptItemWriteOff(FinanceReceiptItemVo.builder()
                    .receiptId(receiptItem.getReceiptId())
                    .receiptNo(receiptDO.getReceiptNo())
                    .receiptItemId(receiptItem.getId())
                    .remark("批量创建收款明细自动提交审批")
                    .build());
            // TODO 更新汇率
        }
        // 返回
        return receiptItem.getId();
    }

    private ReceiptDO validateCreateVO(ReceiptItemCreateReqVO createReqVO) {
        if (null == createReqVO.getReceiptId()) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        if (createReqVO.getReceivableWriteOffList() == null || createReqVO.getReceivableWriteOffList().isEmpty()) {
            throw exception(RECEIPT_ITEM_RECEIVABLE_WRITE_OFF_LIST_EMPTY);
        }
        ReceiptDO receiptDO = receiptService.getReceipt(createReqVO.getReceiptId());
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        if (receiptDO.getState() == 0) {
            throw exception(RECEIPT_STATUS_NOT_ADD_ITEM);
        }
        return receiptDO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReceiptItem(ReceiptItemUpdateReqVO updateReqVO) {
        ReceiptItemDO receiptItemDO = receiptItemMapper.selectById(updateReqVO.getId());
        if (receiptItemDO == null) {
            throw exception(RECEIPT_ITEM_NOT_EXISTS);
        }
        if (!receiptItemDO.getStatus().equals(ReceiptItemStatusEnum.DRAFT.getValue())) {
            throw exception(RECEIPT_ITEM_NOT_EXISTS);
        }
        if (updateReqVO.getReceivableWriteOffList() == null || updateReqVO.getReceivableWriteOffList().isEmpty()) {
            throw exception(RECEIPT_ITEM_RECEIVABLE_WRITE_OFF_LIST_EMPTY);
        }
        // 删除之前的核销记录
        receivableService.deleteWriteOffRecord(receiptItemDO);
        // 更新
        ReceiptItemDO updateObj = ReceiptItemConvert.INSTANCE.convert(updateReqVO);
        receiptItemMapper.updateById(updateObj);
        BankAccountDTO account = bankApi.getBankAccountByAccountId(updateObj.getAccountId());
        Integer incomeBelong = account.getBaIncomeBelong();
        // 生成应收明细核销记录
        for (ReceivableWriteOffReqVO receivableWriteOffReqVO : updateReqVO.getReceivableWriteOffList()) {
            receivableService.createWriteOffRecord(receivableWriteOffReqVO, receiptItemDO, incomeBelong);
        }
    }

    /*
      状态:0:草稿， 1待收款， 2待核销，3已部分核销，4已核销，5已核销待开票，6已开票
      收款单状态
     */
    @Override
    public void deleteReceiptItem(Long id) {
        // 校验存在
        ReceiptItemDO receiptItemDO = receiptItemMapper.selectById(id);
        if (null == receiptItemDO) {
            throw exception(RECEIPT_ITEM_NOT_EXISTS);
        }
        // 删除
        receiptItemMapper.deleteById(id);
        //判断当前收款单下还有没有收款明细
        if (null != receiptItemDO.getReceiptId()) {
            LambdaQueryWrapper<ReceiptItemDO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(ReceiptItemDO::getReceiptId, receiptItemDO.getReceiptId());
            long count = this.count(queryWrapper);
            if (count == 0) {
                ReceivableDO updateReceivableDO = new ReceivableDO();
                updateReceivableDO.setState(0);
                //更新当前收款单下的应收明细为待收款
                UpdateWrapper updateWrapper = new UpdateWrapper();
                updateWrapper.eq("receipt_id", receiptItemDO.getReceiptId());
                this.receivableService.update(updateReceivableDO, updateWrapper);

                //添加的逻辑点
                //当银行收款明细为0是要改变收款单的状态为 待收款
                ReceiptDO update = new ReceiptDO();
                update.setId(receiptItemDO.getReceiptId());
                update.setState(1);
                receiptService.updateById(update);
                //end 添加的逻辑点
            }
        }
        receiptService.updateReceiptItemStatus(receiptItemDO.getReceiptId());
    }

    private void validateReceiptItemExists(Long id) {
        if (receiptItemMapper.selectById(id) == null) {
            throw exception(RECEIPT_ITEM_NOT_EXISTS);
        }
    }

    @Override
    public ReceiptItemDO getReceiptItem(Long id) {
        return receiptItemMapper.selectById(id);
    }

    @Override
    public List<ReceiptItemDO> getReceiptItemList(Collection<Long> ids) {
        return receiptItemMapper.selectBatchIds(ids);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void verification(Long id, Integer bpmResult, String comment) {
        //收款单明细核销审核
        ReceiptItemDO receiptItemDO = receiptItemMapper.selectById(id);
        receiptItemDO.setBmpStatus(bpmResult);
        // 核销通过
        if (Objects.equals(bpmResult, BpmProcessInstanceResultEnum.APPROVE.getResult())) {
            receiptItemDO.setStatus(ReceiptItemStatusEnum.WRITE_OFF.getValue());
            receiptItemDO.setApprovalTime(new Date());

            // 更新收款单
            ReceiptDO receiptDO = receiptMapper.selectById(receiptItemDO.getReceiptId());
            //判断收款单状态如果是待提交状态-变成部分核销状态，其它 状态不变
            if (receiptDO.getState().equals(ReceiptStatusEnum.WRITE_OFF_WAITING.getValue())) {
                receiptDO.setState(ReceiptStatusEnum.WRITE_OFF_PART_ING.getValue());
            }
            // TODO 是否更新核销金额，核销比例
            BigDecimal writeOffAmount = receiptItemMapper.getWriteOffAmountByReceiptId(receiptDO.getId());
            if (null != receiptDO.getReceivableTotalAmount() && receiptDO.getReceivableTotalAmount().compareTo(BigDecimal.ZERO) > 0) {
                receiptDO.setWriteOffProportion(writeOffAmount.divide(receiptDO.getReceivableTotalAmount(), 8, RoundingMode.HALF_UP).multiply(new BigDecimal("100")).setScale(6, BigDecimal.ROUND_HALF_UP));
            }
            try {
                receiptService.updateById(receiptDO);
            } catch (Exception e) {
                log.error("更新收款单状态失败", e);
            }
            //更新应收明细
            List<ReceivableDO> receivableDOS = receivableService.selectList("receipt_id", receiptItemDO.getReceiptId());
            for (ReceivableDO receivableDO : receivableDOS) {
                receivableDO.setState(1);
                ReceivableWriteOffRecordDO receivableWriteOffRecordDO = receivableService.getReceivableWriteOffRecord(receivableDO.getId(), receiptItemDO.getId());
                if (null != receivableWriteOffRecordDO) {
                    BigDecimal writeOffCurrent = receivableWriteOffRecordDO.getWriteOffAmount().divide(receivableDO.getExchangeRate(), 4, RoundingMode.HALF_UP);
                    BigDecimal oldWriteOffAmount = receivableDO.getWriteOffAmount() == null ? BigDecimal.ZERO : receivableDO.getWriteOffAmount();
                    receivableDO.setWriteOffAmount(oldWriteOffAmount.add(writeOffCurrent));
                }
            }
            receivableService.updateBatchById(receivableDOS);
            receiptService.updateReceiptItemStatus(receiptItemDO.getReceiptId());
            // 更新银行账户余额
            bankApi.updateBankAccountBalance(BankAccountDTO.builder()
                    .id(receiptItemDO.getAccountId())
                    .baAccountName(receiptItemDO.getAccountName())
                    .baAccountNum(receiptItemDO.getAccountNo())
                    .baBankName(receiptItemDO.getAccountBankName())
                    .baBalance(receiptItemDO.getAmount()).build(), true);
        }
        // 审核拒绝
        else if (Objects.equals(bpmResult, BpmProcessInstanceResultEnum.REJECT.getResult())) {
            receiptItemDO.setStatus(ReceiptItemStatusEnum.WRITE_OFF_APPROVE_REJECT.getValue());
        }
        // 取消审核
        else if (Objects.equals(bpmResult, BpmProcessInstanceResultEnum.CANCEL.getResult())) {
            receiptItemDO.setStatus(ReceiptItemStatusEnum.DRAFT.getValue());
        }
        receiptItemMapper.updateById(receiptItemDO);
        receiptService.recordLog(receiptItemDO.getReceiptId(), ReceiptLinkEnum.APPROVE_BANK_RECEIPT, comment, WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF.getValue());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchVerification(Collection<Long> ids) {
        Long receiptId = null;
        for (Long id : ids) {
            ReceiptItemDO receiptItemDO = this.getReceiptItem(id);
            if (null == receiptItemDO) {
                throw exception(RECEIPTITEM_NOT_EXISTS);
            }
            receiptId = receiptItemDO.getReceiptId();
            if (receiptItemDO.getStatus() == 0) {
                ReceiptItemDO updateReceiptDO = new ReceiptItemDO();
                updateReceiptDO.setId(receiptItemDO.getId());

                updateReceiptDO.setStatus(1);
                this.updateById(updateReceiptDO);
            } else {
                throw exception(RECEIPTITEM_VERIFICATION);
            }
        }


        if (null == receiptId) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        ReceiptDO receiptDO = receiptService.getById(receiptId);
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        //设置应收明细为收款中
        LambdaUpdateWrapper<ReceivableDO> lambdaReceivableUpdateWrapper = new LambdaUpdateWrapper();
        lambdaReceivableUpdateWrapper.set(ReceivableDO::getState, 1);
        lambdaReceivableUpdateWrapper.eq(ReceivableDO::getReceiptId, receiptId);
        lambdaReceivableUpdateWrapper.eq(ReceivableDO::getState, 0);
        receivableService.update(lambdaReceivableUpdateWrapper);

        //需要计算出核销比例
        BigDecimal writeOffAmount = receiptItemMapper.getWriteOffAmountByReceiptId(receiptDO.getId());

        ReceiptDO updateReceiptDO = new ReceiptDO();
        updateReceiptDO.setId(receiptDO.getId());
        //需要计算出核销比例
        if (receiptDO.getState() == 2) {
            updateReceiptDO.setState(3);
        }

        if (null != receiptDO.getReceivableTotalAmount() && receiptDO.getReceivableTotalAmount().compareTo(BigDecimal.ZERO) == 1) {
            updateReceiptDO.setWriteOffProportion(writeOffAmount.divide(receiptDO.getReceivableTotalAmount(), 8, RoundingMode.HALF_UP).multiply(new BigDecimal("100")).setScale(6, BigDecimal.ROUND_HALF_UP));
        }

        receiptService.updateById(updateReceiptDO);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void allVerification(Long id) {
        //进行验证判断核销是否达到100%
        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceiptItemDO::getStatus, 1);
        lambdaQueryWrapper.eq(ReceiptItemDO::getReceiptId, id);
        List<ReceiptItemDO> receiptItemDOList = this.list(lambdaQueryWrapper);
        if (null == receiptItemDOList || receiptItemDOList.size() == 0) {
            throw exception(RECEIPT_NOT_WRITE_OFF_ITEM_NOT_EXISTS);
        }

        BigDecimal totalAmount = receiptItemDOList.stream().map(ReceiptItemDO::getWriteOffAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        //核销全部收款明细的时候要判断收款单状态进行更新
        ReceiptDO receiptDO = receiptService.getReceipt(id);
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        if (receiptDO.getState() != 2 && receiptDO.getState() != 3) {
            throw exception(RECEIPT_WRITE_OFF_STATUS_NOT);
        }


        Long currencyId = receiptDO.getCurrencyId();
        if (null == currencyId) {
            throw exception(RECEIPT_CURRENCY_NOT_EXISTS);
        }
        if (totalAmount.compareTo(receiptDO.getReceivableTotalAmount()) == -1) {
            DictDataRespDTO dictDataRespDTO = dictDataApi.parseDictDataFromCache("write_off_diff", String.valueOf(currencyId));
            if (null == dictDataRespDTO) {
                throw exception(RECEIPT_CURRENCY_DIFF_NOT_EXISTS);
            }
            //核销差额大于配置提示
            if (receiptDO.getReceivableTotalAmount().subtract(totalAmount).compareTo(new BigDecimal(dictDataRespDTO.getValue())) == 1) {
                throw exception(RECEIPT_CURRENCY_DIFF_BIG);
            }
        }

        //设置应收明细为已收款状态
        LambdaUpdateWrapper<ReceivableDO> lambdaReceivableUpdateWrapper = new LambdaUpdateWrapper();
        lambdaReceivableUpdateWrapper.set(ReceivableDO::getState, 2);
        lambdaReceivableUpdateWrapper.eq(ReceivableDO::getReceiptId, id);
        receivableService.update(lambdaReceivableUpdateWrapper);


        LambdaUpdateWrapper<ReceiptItemDO> lambdaUpdateWrapper = new LambdaUpdateWrapper();
        lambdaUpdateWrapper.set(ReceiptItemDO::getStatus, 1);
        lambdaUpdateWrapper.eq(ReceiptItemDO::getReceiptId, id);
        lambdaUpdateWrapper.eq(ReceiptItemDO::getStatus, 0);
        this.update(lambdaUpdateWrapper);

        //需要计算出核销比例
        ReceiptDO update = new ReceiptDO();
        update.setId(receiptDO.getId());
        if (receiptDO.getOpenInvoice() == 1) {
            update.setState(5);
        } else {
            update.setState(4);
        }
        update.setWriteOffAt(new Date());
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        if (null != loginUser) {
            update.setWriteOffId(loginUser.getId());
            update.setWriteOffName(loginUser.getNickname());
        }
        update.setWriteOffProportion(new BigDecimal("100"));//设置核销比例
        receiptService.updateById(update);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void verificationCancel(Long id) {
        ReceiptItemDO receiptItemDO = this.getReceiptItem(id);
        if (receiptItemDO == null) {
            throw exception(RECEIPT_ITEM_NOT_EXISTS);
        }
        if (receiptItemDO.getStatus() != 1) {
            throw exception(RECEIPTITEM_VERIFICATION);
        }

        ReceiptItemDO updateReceiptItemDO = new ReceiptItemDO();
        updateReceiptItemDO.setId(receiptItemDO.getId());
        updateReceiptItemDO.setStatus(0);
        this.updateById(updateReceiptItemDO);

        if (null == receiptItemDO.getReceiptId()) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        ReceiptDO receiptDO = receiptService.getById(receiptItemDO.getReceiptId());
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(ReceiptItemDO::getReceiptId, receiptItemDO.getReceiptId());
        lambdaQueryWrapper.eq(ReceiptItemDO::getStatus, 1);
        long writeCount = this.count(lambdaQueryWrapper);

        ReceiptDO updateReceiptDO = new ReceiptDO();
        updateReceiptDO.setId(receiptDO.getId());
        if (writeCount == 0) {
            updateReceiptDO.setState(2);
        } else {
            updateReceiptDO.setState(3);
        }

        //需要计算出核销比例

        BigDecimal writeOffAmount = receiptItemMapper.getWriteOffAmountByReceiptId(receiptItemDO.getReceiptId());

        log.error("------------------------------{}", writeOffAmount.toString());

        if (null != receiptDO.getReceivableTotalAmount() && receiptDO.getReceivableTotalAmount().compareTo(BigDecimal.ZERO) == 1) {
            updateReceiptDO.setWriteOffProportion(writeOffAmount.divide(receiptDO.getReceivableTotalAmount(), 8, RoundingMode.HALF_UP).multiply(new BigDecimal("100")).setScale(6, BigDecimal.ROUND_HALF_UP));
        }

        receiptService.updateById(updateReceiptDO);

        //TODO 收款单明细反核销需要发起流程
    }

    @Override
    public ReceiptApprovalDO createFinanceReceiptItemWriteOff(FinanceReceiptItemVo financeReceiptItemVo) {
        ReceiptItemDO receiptItemDO = this.getById(financeReceiptItemVo.getReceiptItemId());
        if (null == receiptItemDO) {
            throw exception(RECEIPT_ITEM_NOT_FOUND);
        }
        if (receiptItemDO.getStatus() == 1) {
            throw exception(RECEIPT_ITEM_STATE_WRITE_OFF_SUCCESS_NOT_APPROVE);
        }
        //校验收款单是否核销中
        validateReceipt(receiptItemDO.getReceiptId());
        receiptService.recordLog(receiptItemDO.getReceiptId(), ReceiptLinkEnum.SUBMIT_BANK_RECEIPT, financeReceiptItemVo.getRemark(), WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF.getValue());
        // 判断是否符合自动审批规则
        boolean autoApprove = judgmentAutoApprove(receiptItemDO);
        //创建收款单明细核销流程
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(financeReceiptItemVo.getReceiptId());
        approvalDO.setReceiptNo(financeReceiptItemVo.getReceiptNo());
        approvalDO.setReceiptItemId(financeReceiptItemVo.getReceiptItemId());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF.getKey());
        approvalDO.setReason(financeReceiptItemVo.getRemark());
        if (autoApprove) {
            approvalDO.setStatus(BpmProcessInstanceResultEnum.APPROVE.getResult());
            receiptApprovalService.save(approvalDO);
            receiptItemDO.setBmpStatus(BpmProcessInstanceResultEnum.APPROVE.getResult());
            receiptItemDO.setStatus(ReceiptItemStatusEnum.WRITE_OFF.getValue());
            receiptItemDO.setApprovalTime(new Date());
            this.updateById(receiptItemDO);
            // 更新收款单
            ReceiptDO receiptDO = receiptMapper.selectById(receiptItemDO.getReceiptId());
            //判断收款单状态如果是待提交状态-变成部分核销状态，其它 状态不变
            if (receiptDO.getState().equals(ReceiptStatusEnum.WRITE_OFF_WAITING.getValue())) {
                receiptDO.setState(ReceiptStatusEnum.WRITE_OFF_PART_ING.getValue());
            }
            BigDecimal writeOffAmount = receiptItemMapper.getWriteOffAmountByReceiptId(receiptDO.getId());
            if (null != receiptDO.getReceivableTotalAmount() && receiptDO.getReceivableTotalAmount().compareTo(BigDecimal.ZERO) > 0) {
                receiptDO.setWriteOffProportion(writeOffAmount.divide(receiptDO.getReceivableTotalAmount(), 8, RoundingMode.HALF_UP).multiply(new BigDecimal("100")).setScale(6, BigDecimal.ROUND_HALF_UP));
            }
            try {
                receiptService.updateById(receiptDO);
            } catch (Exception e) {
                log.error("更新收款单失败", e);
            }
            //更新应收明细
            List<ReceivableDO> receivableDOS = receivableService.selectList("receipt_id", receiptItemDO.getReceiptId());
            for (ReceivableDO receivableDO : receivableDOS) {
                receivableDO.setState(1);
                ReceivableWriteOffRecordDO receivableWriteOffRecordDO = receivableService.getReceivableWriteOffRecord(receivableDO.getId(), receiptItemDO.getId());
                if (null != receivableWriteOffRecordDO) {
                    BigDecimal writeOffCurrent = receivableWriteOffRecordDO.getWriteOffAmount().divide(receivableDO.getExchangeRate(), 4, RoundingMode.HALF_UP);
                    BigDecimal oldWriteOffAmount = receivableDO.getWriteOffAmount() == null ? BigDecimal.ZERO : receivableDO.getWriteOffAmount();
                    receivableDO.setWriteOffAmount(oldWriteOffAmount.add(writeOffCurrent));
                }
            }
            receivableService.updateBatchById(receivableDOS);
            // 更新银行账户余额
            bankApi.updateBankAccountBalance(BankAccountDTO.builder()
                    .id(receiptItemDO.getAccountId())
                    .baAccountName(receiptItemDO.getAccountName())
                    .baAccountNum(receiptItemDO.getAccountNo())
                    .baBankName(receiptItemDO.getAccountBankName())
                    .baBalance(receiptItemDO.getAmount()).build(), true);
        } else {
            approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            receiptApprovalService.save(approvalDO);
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF.getKey(), financeReceiptItemVo.getReceiptNo(), financeReceiptItemVo.getCopyUserList());
            approvalDO.setBmpId(bpmId);
            receiptApprovalService.updateById(approvalDO);

            receiptItemDO.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            receiptItemDO.setStatus(ReceiptItemStatusEnum.WRITE_OFF_APPROVE_ING.getValue());
            receiptItemDO.setBmpId(bpmId);
            this.updateById(receiptItemDO);
            return approvalDO;
        }
        receiptService.updateReceiptItemStatus(receiptItemDO.getReceiptId());
        return null;
    }

    /**
     * 判断是否符合自动审批条件
     * 当银行收款明细提交后,当前收款单下有且只有一个银行收款明细,且总核销比例在当前页面设置
     * 的核销比例范围内,银行收款明细自动审批通过,审批通过人为admin,日志备注核销比例为XX,
     * 符合核销比例99.99%-100.99%
     */
    private boolean judgmentAutoApprove(ReceiptItemDO receiptItemDO) {
        List<ReceiptItemDO> receiptItemDOS = receiptItemMapper.selectList("receipt_id", receiptItemDO.getReceiptId());
        if (receiptItemDOS.size() > 1 || !Objects.equals(receiptItemDOS.get(0).getId(), receiptItemDO.getId())) {
            return false;
        }
        List<ReceiptBankWriteoffRangeDO> receiptBankWriteoffRangeDOS = receiptBankWriteoffRangeMapper.selectList();
        if (CollectionUtil.isEmpty(receiptBankWriteoffRangeDOS)) {
            return false;
        }
        List<ReceiptAccountDO> receiptAccounts = receiptAccountMapper.selectList("receipt_id", receiptItemDO.getReceiptId());
        if (CollectionUtil.isEmpty(receiptAccounts)) {
            return false;
        }
        // 计算账单总金额
        BigDecimal totalAmount = receiptAccounts.stream().map(ReceiptAccountDO::getWriteOffAmount).reduce(BigDecimal.ZERO, BigDecimal::add);

        // 计算核销比例
        BigDecimal writeOffDown = new BigDecimal(receiptBankWriteoffRangeDOS.get(0).getWriteOffDown());
        BigDecimal writeOffUp = new BigDecimal(receiptBankWriteoffRangeDOS.get(0).getWriteOffUp());
        BigDecimal writeOffRate = receiptItemDO.getWriteOffAmount().multiply(new BigDecimal(10000)).divide(totalAmount, 8, RoundingMode.HALF_UP).setScale(6, RoundingMode.HALF_UP);
        if (writeOffRate.compareTo(writeOffDown) < 0 || writeOffRate.compareTo(writeOffUp) > 0) {
            return false;
        }
        String remark = "核销比例为: " + writeOffRate.divide(new BigDecimal("100"), 2, RoundingMode.HALF_UP) + "%, " + "符合核销比例" + writeOffDown.divide(new BigDecimal("100"), 2, RoundingMode.HALF_UP) + "%-" + writeOffUp.divide(new BigDecimal("100"), 2, RoundingMode.HALF_UP) + "%";
        receiptService.recordLog(receiptItemDO.getReceiptId(), ReceiptLinkEnum.APPROVE_BANK_RECEIPT, remark, WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF.getValue());
        return true;
    }

    @Override
    public void createFinanceReceiptItemWriteOffNo(FinanceReceiptItemVo financeReceiptItemVo) {
        //如果收款单状态是全部核销中或者全部核销通过不能发起反核销
        ReceiptItemDO receiptItemDO = this.getById(financeReceiptItemVo.getReceiptItemId());
        if (null == receiptItemDO) {
            throw exception(RECEIPT_ITEM_NOT_FOUND);
        }
        if (receiptItemDO.getStatus() == 0) {
            throw exception(RECEIPT_ITEM_STATE_WRITE_OFF__NOT_APPROVE);
        }
        if (null == receiptItemDO.getReceiptId()) {
            throw exception(RECEIPT_NOT_FOUND);
        }
        //校验收款单是否在在全部核销中
        validateReceipt(receiptItemDO.getReceiptId());

        ReceiptDO receiptDO = receiptService.getReceipt(receiptItemDO.getReceiptId());
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_FOUND);
        }


        if (ReceiptStatusEnum.WRITE_OFF_ALL_ING.getValue() == receiptDO.getState()) {
            throw exception(RECEIPT_WRITE_OFF_ALL_ING_ITEM_NOT_APPROVE);
        }
        if (ReceiptStatusEnum.WRITE_OFF_ALL_SUCCESS.getValue() == receiptDO.getState()) {
            throw exception(RECEIPT_WRITE_OFF_ALL_SUCCESS_ITEM_NOT_APPROVE);
        }
        if (ReceiptStatusEnum.OPEN_BILL.getValue() == receiptDO.getState()) {
            throw exception(RECEIPT_WRITE_OFF_ALL_SUCCESS_ITEM_NOT_APPROVE);
        }


        //创建收款单明细反核销流程
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(financeReceiptItemVo.getReceiptId());
        approvalDO.setReceiptNo(financeReceiptItemVo.getReceiptNo());
        approvalDO.setReceiptItemId(financeReceiptItemVo.getReceiptItemId());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF_NO.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(financeReceiptItemVo.getRemark());
        receiptApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF_NO.getKey(), financeReceiptItemVo.getReceiptNo(), financeReceiptItemVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        receiptApprovalService.updateById(approvalDO);

        ReceiptItemDO update = new ReceiptItemDO();
        update.setId(financeReceiptItemVo.getReceiptItemId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setStatus(ReceiptItemStatusEnum.WRITE_OFF_NO_APPROVE_ING.getValue());
        update.setBmpId(bpmId);
        this.updateById(update);
    }

    @Override
    public void cancelFinanceReceiptItemWriteOff(FinanceReceiptItemCancelVo financeReceiptItemVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptItemDO receiptItemDO = validateReceiptItemBmpCancel(financeReceiptItemVo.getReceiptItemId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptItemDO.getBmpId(), financeReceiptItemVo.getRemark());
        receiptService.updateReceiptItemStatus(receiptItemDO.getReceiptId());
    }

    @Override
    public void cancelFinanceReceiptItemWriteOffNo(FinanceReceiptItemCancelVo financeReceiptItemVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptItemDO receiptItemDO = validateReceiptItemBmpCancel(financeReceiptItemVo.getReceiptItemId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptItemDO.getBmpId(), financeReceiptItemVo.getRemark());
    }

    private ReceiptItemDO validateReceiptItemBmpCancel(Long id) {
        ReceiptItemDO receiptItemDO = this.getReceiptItem(id);
        if (null == receiptItemDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        if (null == receiptItemDO.getBmpStatus()) {

            throw exception(BMP_NOT);
        }

        if (!receiptItemDO.getBmpStatus().equals(BpmProcessInstanceResultEnum.PROCESS.getResult())) {
            throw exception(BMP_CANCEL);
        }

        return receiptItemDO;
    }

    private void validateReceipt(Long receiptId) {
        ReceiptDO receiptDO = receiptService.getById(receiptId);
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        } else {
            if (BpmProcessInstanceResultEnum.PROCESS.getResult().equals(receiptDO.getBmpStatus())) {
                throw exception(RECEIPT_ALL_WRITE_OFF_ING);
            }
        }
    }

    @Override
    public List<ReceiptItemBatchRespVO> receiptItemImport(List<ReceiptItemBatchCreateReqVO> createReqVOlist, Boolean ignoreItem) {
        // 校验收款账户信息并生成创建明细
        ArrayList<ReceiptItemCreateReqVO> receiptItemCreateReqVOS = new ArrayList<>();
        List<ReceiptItemBatchRespVO> respVOS = createReqVOlist.stream()
                .map(createReqVO -> validateBatchReceiptItem(createReqVO, ignoreItem, receiptItemCreateReqVOS))
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(respVOS)) {
            boolean haveError = respVOS.stream().anyMatch(respVO -> respVO.getErrorType() == 0);
            StringBuilder errorMsg = new StringBuilder();
            // 拼接错误信息,每个错误一行
            respVOS.forEach(respVO -> {
                errorMsg.append("<div>")
                        .append(respVO.getErrorType() == 0 ? "错误：" : "提示：").append(respVO.getErrorMsg()).append(", ")
                        .append("订单号/提单号: ").append(respVO.getOrderNo()).append(", ")
                        .append("付款人: ").append(respVO.getPayer())
                        .append("</div>");
            });
            throw exception(new ErrorCode(haveError ? 1004520042 : 1004520043, errorMsg.toString()));
        }
        ReceiptItemService proxy = (ReceiptItemService) AopContext.currentProxy();
        for (ReceiptItemCreateReqVO receiptItemCreateReqVO : receiptItemCreateReqVOS) {
            // 计算每条应收明细的核销金额
            List<ReceivableInItemVO> listForCreateReceiptItem = receivableService.getListForCreateReceiptItem(receiptItemCreateReqVO.getReceiptId());
            List<ReceivableWriteOffReqVO> receivableWriteOffList = getReceiptItemDetail(listForCreateReceiptItem, receiptItemCreateReqVO);
            receiptItemCreateReqVO.setReceivableWriteOffList(receivableWriteOffList);
            // 插入数据
            proxy.createReceiptItem(receiptItemCreateReqVO, true);
        }
        return respVOS;
    }

    /**
     * 自动填写收款明细核销金额
     */
    public List<ReceivableWriteOffReqVO> getReceiptItemDetail(List<ReceivableInItemVO> listForCreateReceiptItem, ReceiptItemCreateReqVO receiptItemCreateReqVO) {
        List<ReceivableInItemVO> receivableInItemVOS = listForCreateReceiptItem.stream()
                .filter(receivableInItemVO -> receivableInItemVO.getNotWriteOffAmountUsd().compareTo(BigDecimal.ZERO) > 0)
                .sorted(Comparator.comparing(ReceivableInItemVO::getNotWriteOffAmountUsd))
                .collect(Collectors.toList());
        ArrayList<ReceivableWriteOffReqVO> receivableWriteOffReqVOS = new ArrayList<>();
        // 当剩余核销金额<未填写核销金额，默认本次核销金额=剩余核销金额，依次填写应收明细
        // 当填充到最后一个应收，默认把剩余未填写核销金额填充到本次核销金额（允许本次核销金额>剩余核销金额）
        BigDecimal writeOffAmount = receiptItemCreateReqVO.getWriteOffAmount();
        for (ReceivableInItemVO receivableInItemVO : receivableInItemVOS) {
            if (writeOffAmount.compareTo(receivableInItemVO.getNotWriteOffAmountUsd()) > 0) {
                ReceivableWriteOffReqVO writeOffReqVO = ReceivableWriteOffReqVO.builder()
                        .receivableId(receivableInItemVO.getId())
                        .writeOffAmount(receivableInItemVO.getNotWriteOffAmountUsd())
                        .build();
                receivableWriteOffReqVOS.add(writeOffReqVO);
                writeOffAmount = writeOffAmount.subtract(receivableInItemVO.getNotWriteOffAmountUsd());
                continue;
            }
            if (writeOffAmount.compareTo(receivableInItemVO.getNotWriteOffAmountUsd()) <= 0) {
                ReceivableWriteOffReqVO writeOffReqVO = ReceivableWriteOffReqVO.builder()
                        .receivableId(receivableInItemVO.getId())
                        .writeOffAmount(writeOffAmount)
                        .build();
                receivableWriteOffReqVOS.add(writeOffReqVO);
                break;
            }
        }
        return receivableWriteOffReqVOS;
    }

    @Override
    public ReceiptItemBackVO getReceiptItemDetail(Long id) {
        ReceiptItemDO receiptItemDO = receiptItemMapper.selectById(id);
        if (null == receiptItemDO) {
            throw exception(RECEIPT_ITEM_NOT_FOUND);
        }
        ReceiptItemBackVO itemBackVO = ReceiptItemConvert.INSTANCE.convert(receiptItemDO);
        ReceiptDO receiptDO = receiptMapper.selectById(receiptItemDO.getReceiptId());
        itemBackVO.setReceiptNo(receiptDO.getReceiptNo());
        List<ReceivableInItemVO> receivableList = receivableService.getReceivableByReceiptItem(receiptItemDO.getId());
        itemBackVO.setReceivableList(receivableList);
        return itemBackVO;
    }

    @Override
    public ReceiptItemReviewPageResult<ReceiptItemReviewPageVO> getReceiptItemReviewPage(ReceiptItemReviewPageReqVO pageReqVO, PageVO page) {
        IPage<ReceiptItemReviewPageVO> mpPage = MyBatisUtils.buildPage(page);
        receiptItemMapper.getReceiptItemReviewPage(mpPage, pageReqVO);
        List<ReceiptItemReviewPageVO> records = mpPage.getRecords();
        if (records.isEmpty()) {
            return ReceiptItemReviewPageResult.empty();
        }
        for (ReceiptItemReviewPageVO record : records) {
            List<ReceiptRelationOrderInfoVO> orderInfoByReceiptId = receiptMapper.getOrderInfoByReceiptId(record.getReceiptId());
            record.setOrderInfoList(orderInfoByReceiptId);
            List<ReceivableDO> receivableDOS = receivableService.selectList("receipt_id", record.getReceiptId());
            record.setFeeStatistics(getReceivableFeeStatistics(receivableDOS));
        }
        ReceiptItemReviewPageResult<ReceiptItemReviewPageVO> pageResult = ReceiptItemReviewPageResult.of(mpPage);
        List<CurrencyAmount> actualAmount = receiptItemMapper.getReceiptItemActualAmount(pageReqVO);
        List<CurrencyAmount> totalActualAmount = receiptItemMapper.countTotalActualAmount();
        List<CurrencyAmount> totalWriteOffAmount = receiptItemMapper.countTotalWriteOffAmount();
        pageResult.setActualAmount(actualAmount);
        pageResult.setTotalActualAmount(totalActualAmount);
        pageResult.setTotalWriteOffAmount(totalWriteOffAmount);
        return pageResult;
    }

    private ReceiptItemReviewPageVO.FeeStatistics getReceivableFeeStatistics(List<ReceivableDO> receivableDOS) {
        HashMap<Long, BigDecimal> summaryMap = new HashMap<>();
        BigDecimal baseCurrencySummary = BigDecimal.ZERO;
        HashMap<Integer, Map<Long, BigDecimal>> detailsMap = new HashMap<>();

        for (ReceivableDO receivableDO : receivableDOS) {
            BigDecimal actualAmount = receivableDO.getTaxAmount().subtract(receivableDO.getDiscountTotal());

            // 更新 summaryMap
            summaryMap.merge(receivableDO.getCurrencyId(), actualAmount, BigDecimal::add);

            // 更新 baseCurrencySummary
            baseCurrencySummary = baseCurrencySummary.add(receivableDO.getBaseAmount());

            // 更新 detailsMap
            detailsMap.computeIfAbsent(receivableDO.getFeeType(), k -> new HashMap<>())
                    .merge(receivableDO.getCurrencyId(), actualAmount, BigDecimal::add);
        }

        Map<Integer, CurrencyRespDTO> allCurrency = currencyApi.getAllCurrency();
        ReceiptItemReviewPageVO.FeeStatistics feeStatistics = new ReceiptItemReviewPageVO.FeeStatistics();

        if (!receivableDOS.isEmpty()) {
            CurrencyRespDTO baseCurrency = allCurrency.get(receivableDOS.get(0).getBaseCurrencyId().intValue());
            feeStatistics.setBaseCurrencySummary(baseCurrency.getFuhao() + baseCurrencySummary);
        }

        StringBuilder summary = new StringBuilder(summaryMap.size() * 20);
        for (Map.Entry<Long, BigDecimal> entry : summaryMap.entrySet()) {
            CurrencyRespDTO currency = allCurrency.get(entry.getKey().intValue());
            summary.append(currency.getFuhao()).append(entry.getValue()).append(",");
        }
        if (summary.length() > 0) {
            feeStatistics.setSummary(summary.substring(0, summary.length() - 1));
        }

        List<ReceiptItemReviewPageVO.Fee> fees = new ArrayList<>(detailsMap.size());
        for (Map.Entry<Integer, Map<Long, BigDecimal>> entry : detailsMap.entrySet()) {
            StringBuilder detail = new StringBuilder(entry.getValue().size() * 20);
            for (Map.Entry<Long, BigDecimal> detailEntry : entry.getValue().entrySet()) {
                CurrencyRespDTO currency = allCurrency.get(detailEntry.getKey().intValue());
                detail.append(currency.getFuhao()).append(detailEntry.getValue()).append(",");
            }
            fees.add(new ReceiptItemReviewPageVO.Fee(entry.getKey(), detail.substring(0, detail.length() - 1)));
        }
        feeStatistics.setDetails(fees);

        return feeStatistics;
    }

    /**
     * 批量创建收款单校验
     *
     * @param batchCreateReqVO
     * @param ignoreItem
     * @param receiptItemCreateReqVOS
     * @return
     */
    private ReceiptItemBatchRespVO validateBatchReceiptItem(ReceiptItemBatchCreateReqVO batchCreateReqVO, Boolean ignoreItem, ArrayList<ReceiptItemCreateReqVO> receiptItemCreateReqVOS) {
        // 校验单个字段合法性
        ReceiptItemBatchRespVO commonFieldRes = validateBatchReceiptItemCommonField(batchCreateReqVO);
        if (commonFieldRes != null) {
            return commonFieldRes;
        }
        // 根据订单号或提单号是否能查到订单
        OrderRespDTO orderByNo = orderApi.getOrderByNo(batchCreateReqVO.getOrderNo());
        if (null == orderByNo) {
            return new ReceiptItemBatchRespVO(batchCreateReqVO.getOrderNo(), batchCreateReqVO.getPayer(), 0, "订单不存在");
        }
        // 根据订单号或提单号+付款人匹配状态≠已核销/草稿状态的收款单是否存在，存在多条匹配创建时间最晚的一条
        List<ReceivableDO> receivableDOS = receivableService.selectList(Wrappers.lambdaQuery(ReceivableDO.class)
                .eq(ReceivableDO::getOrderId, orderByNo.getOrderId())
                .eq(ReceivableDO::getPaymentUser, "发货人".equals(batchCreateReqVO.getPayer()) ? 1 : 2));
        if (receivableDOS == null || receivableDOS.isEmpty()) {
            return new ReceiptItemBatchRespVO(batchCreateReqVO.getOrderNo(), batchCreateReqVO.getPayer(), 0, "收款单不存在");
        }
        if (!receivableDOS.get(0).getBaseCurrencyId().equals(Long.parseLong(batchCreateReqVO.getSettleCurrency()))) {
            return new ReceiptItemBatchRespVO(batchCreateReqVO.getOrderNo(), batchCreateReqVO.getPayer(), 0, "结算币种与收款单基准币种不一致");
        }
        Map<Long, List<ReceivableDO>> receivableCollect = receivableDOS.stream().filter(receivableDO -> receivableDO.getReceiptId() != null).collect(Collectors.groupingBy(ReceivableDO::getReceiptId));
        ReceiptDO matchReceiptDO = receivableCollect.keySet().stream()
                .map(receiptId -> receiptService.selectOne("id", receiptId))
                .filter(receivableDO -> !Objects.equals(receivableDO.getState(), ReceiptStatusEnum.DRAFT.getValue()) && !Objects.equals(receivableDO.getState(), ReceiptStatusEnum.WRITE_OFF_ALL_SUCCESS.getValue()))
                .max(Comparator.comparingLong(r -> r.getCreateTime().getTime()))
                .orElse(null);
        if (null == matchReceiptDO) {
            return new ReceiptItemBatchRespVO(batchCreateReqVO.getOrderNo(), batchCreateReqVO.getPayer(), 0, "收款单不存在");
        }
        // 查询订单是否存在多条银行收款数据
        if (!ignoreItem) {
            LambdaQueryWrapper<ReceiptItemDO> queryWrapper = Wrappers.lambdaQuery(ReceiptItemDO.class).in(ReceiptItemDO::getReceiptId, receivableCollect.keySet());
            List<ReceiptItemDO> receiptItemDOS = receiptItemMapper.selectList(queryWrapper);
            if (receiptItemDOS.size() > 1) {
                return new ReceiptItemBatchRespVO(batchCreateReqVO.getOrderNo(), batchCreateReqVO.getPayer(), 1, "订单存在多条银行收款数据");
            }
            if (receiptItemDOS.size() == 1) {
                return new ReceiptItemBatchRespVO(batchCreateReqVO.getOrderNo(), batchCreateReqVO.getPayer(), 1, "收款单已存在银行收款明细");
            }
        }
        // 生成创建明细
        ReceiptItemCreateReqVO receiptItemCreateReqVO = new ReceiptItemCreateReqVO();
        receiptItemCreateReqVO.setReceiptId(matchReceiptDO.getId());
        receiptItemCreateReqVO.setAccountId(batchCreateReqVO.getAccountId());
        receiptItemCreateReqVO.setAccountNo(batchCreateReqVO.getPayAccount());
        receiptItemCreateReqVO.setAccountName(batchCreateReqVO.getAccountName());
        receiptItemCreateReqVO.setAccountBankName(batchCreateReqVO.getAccountBankName());
        receiptItemCreateReqVO.setAmount(new BigDecimal(batchCreateReqVO.getPayAmount()));
        receiptItemCreateReqVO.setCurrencyId(Long.parseLong(batchCreateReqVO.getPayCurrency()));
        receiptItemCreateReqVO.setRate(batchCreateReqVO.getSettleRate());
        receiptItemCreateReqVO.setWriteOffCurrencyId(Long.parseLong(batchCreateReqVO.getSettleCurrency()));
        receiptItemCreateReqVO.setWriteOffAmount(receiptItemCreateReqVO.getAmount().multiply(new BigDecimal(batchCreateReqVO.getSettleRate())));
        receiptItemCreateReqVO.setRemark(batchCreateReqVO.getRemark());
        receiptItemCreateReqVO.setOrderNo(orderByNo.getOrderNo());
        try {
            receiptItemCreateReqVO.setAmountDate(DateUtils.parseDate(batchCreateReqVO.getPayDate(), FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND));
        } catch (Exception e) {
            return new ReceiptItemBatchRespVO(batchCreateReqVO.getOrderNo(), batchCreateReqVO.getPayer(), 0, "收款日期格式错误");
        }
        receiptItemCreateReqVO.setBillNo(batchCreateReqVO.getWaterNo());
        receiptItemCreateReqVOS.add(receiptItemCreateReqVO);
        return null;
    }

    private @Nullable ReceiptItemBatchRespVO validateBatchReceiptItemCommonField(ReceiptItemBatchCreateReqVO createReqVO) {
        if (StringUtils.isBlank(createReqVO.getOrderNo())) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "订单号/提单号不能为空");
        }
        if (StringUtils.isBlank(createReqVO.getPayer())) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "付款人不能为空");
        }
        if (!"发货人".equals(createReqVO.getPayer()) && !"收货人".equals(createReqVO.getPayer())) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "付款人需填写发货人或收货人");
        }
        if (createReqVO.getPayDate() == null) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "收款日期不能为空");
        }
        // 校验收款日期格式
        try {
            DateUtils.parseDate(createReqVO.getPayDate(), FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND);
        } catch (Exception e) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "收款日期格式不正确");
        }
        if (StringUtils.isBlank(createReqVO.getPayAccount())) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "收款账号不能为空");
        }
        if (StringUtils.isBlank(createReqVO.getPayAmount())) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "收款金额不能为空");
        }
        BigDecimal payAmount = new BigDecimal(createReqVO.getPayAmount());
        if (payAmount.compareTo(BigDecimal.ZERO) < 0) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "收款金额必须大于0");
        }
        if (StringUtils.isBlank(createReqVO.getPayCurrency())) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "实收金额币种不能为空");
        }
        if (StringUtils.isBlank(createReqVO.getSettleCurrency())) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "结算币种不能为空");
        }
        if (StringUtils.isBlank(createReqVO.getSubmitter())) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "提交人不能为空");
        }
        Map<Integer, CurrencyRespDTO> allCurrency = currencyApi.getAllCurrency();
        List<String> currencySymbols = allCurrency.values().stream().map(CurrencyRespDTO::getFuhao).collect(Collectors.toList());
        if (!currencySymbols.contains(createReqVO.getPayCurrency())) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "实收金额币种不存在");
        }
        if (!currencySymbols.contains(createReqVO.getSettleCurrency())) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "结算币种不存在");
        }
        ExchangeRateRespDTO rate = currencyApi.getCurrencyRateByCode(createReqVO.getPayCurrency(), createReqVO.getSettleCurrency());
        if (null == rate) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "汇率不存在");
        }
        createReqVO.setSettleRate(rate.getCurrencyRate().toString());
        for (CurrencyRespDTO value : allCurrency.values()) {
            if (value.getFuhao().equals(createReqVO.getPayCurrency())) {
                createReqVO.setPayCurrency(value.getId().toString());
            }
            if (value.getFuhao().equals(createReqVO.getSettleCurrency())) {
                createReqVO.setSettleCurrency(value.getId().toString());
            }
        }
        List<BankAccountDTO> accounts = bankApi.getBankAccountByAccountNo(createReqVO.getPayAccount());
        if (CollectionUtil.isEmpty(accounts)) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "收款账号不存在");
        }
        if (accounts.size() > 1) {
            return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "匹配到多个收款账号");
        }
        BankAccountDTO account = accounts.get(0);
        createReqVO.setAccountId(account.getId());
        createReqVO.setAccountName(account.getBaAccountName());
        createReqVO.setAccountBankName(account.getBaBankName());
        //if (account.getBaCurrency() != null && !allCurrency.get(account.getBaCurrency()).getFuhao().equals(createReqVO.getPayCurrency())) {
        //    return new ReceiptItemBatchRespVO(createReqVO.getOrderNo(), createReqVO.getPayer(), 0, "收款账号币种与实收金额币种不一致");
        //}
        return null;
    }
}
