package cn.iocoder.yudao.module.wealth.service.receivable;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.boot.module.order.api.OrderApi;
import cn.iocoder.yudao.framework.apollo.core.event.Order.OrderOperateLogEvent;
import cn.iocoder.yudao.framework.apollo.core.vo.ApplyInfoVO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.ecw.api.currency.dto.ExchangeRateRespDTO;
import cn.iocoder.yudao.module.ecw.api.customer.CustomerApi;
import cn.iocoder.yudao.module.ecw.api.customer.dto.CustomerDTO;
import cn.iocoder.yudao.module.ecw.api.region.RegionApi;
import cn.iocoder.yudao.module.ecw.api.region.dto.RegionDTO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import cn.iocoder.yudao.module.wealth.convert.receivable.ReceivableConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableWriteOffRecordDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivableDiscount.ReceivableDiscountDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.ReceiptItemMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receivable.ReceivableMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receivable.ReceivableWriteOffRecordMapper;
import cn.iocoder.yudao.module.wealth.service.receipt.ReceiptService;
import cn.iocoder.yudao.module.wealth.service.receivableDiscount.ReceivableDiscountService;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.CurrencyAmount;
import cn.iocoder.yudao.module.wealth.vo.receivable.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.BoxReportConstant.*;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;

/**
 * 应收款 Service 实现类
 *
 * @author 郑屹
 */
@Service
@Validated
@Slf4j
public class ReceivableServiceImpl extends AbstractService<ReceivableMapper, ReceivableDO> implements ReceivableService {

    @Resource
    private ReceivableMapper receivableMapper;
    @Resource
    @Lazy
    private ReceiptService receiptService;
    @Resource
    private ReceivableDiscountService receivableDiscountService;
    @Resource
    private AdminUserApi adminUserApi;
    @Resource
    private ApplicationContext applicationContext;
    @Resource
    private CurrencyApi currencyApi;
    @Resource
    private CustomerApi customerApi;
    @Resource
    private RegionApi regionApi;
    @Resource
    @Lazy
    private OrderApi orderApi;
    @Resource
    private ReceivableWriteOffRecordMapper receivableWriteOffRecordMapper;
    @Resource
    private ReceiptItemMapper receiptItemMapper;
    @Resource
    private RedisHelper redisHelper;

    @Override
    public Long createReceivable(ReceivableCreateReqVO createReqVO) {
        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String creator = String.valueOf(null == user ? "" : user.getId());
        // TODO 具体业务待完善
//        if (null != createReqVO.getOrderItemId()){
//
//        }
        // 插入
        ReceivableDO receivable = ReceivableConvert.INSTANCE.convert(createReqVO);
        receivable.setCreateTime(new Date());
        receivable.setCreator(creator);
        receivable.setUpdateTime(new Date());
        receivable.setUpdater(creator);
        // 设置相关币种
        Long countryId = orderApi.getDestCountryByOrderId(createReqVO.getOrderId());
        RegionDTO regionInfo = regionApi.getRegionById(countryId);
        if (regionInfo != null) {
            receivable.setBaseCurrencyId(Long.parseLong(regionInfo.getImportCurrency5()));
            BigDecimal baseAmount = currencyApi.getCurrencyRate(receivable.getCurrencyId(), receivable.getBaseCurrencyId()).getCurrencyRate().multiply(receivable.getTotalAmount());
            receivable.setBaseAmount(baseAmount);
            int feeType = getFeeType(receivable.getFeeType());
            // 运费
            if (feeType == 0) {
                receivable.setDestCountryCurrencyId(Long.parseLong(regionInfo.getImportCurrency1()));
                receivable.setDestCountryRate(currencyApi.getCurrencyRate(receivable.getCurrencyId(), receivable.getDestCountryCurrencyId()).getCurrencyRate());
            }
            // 清关费
            if (feeType == 1) {
                receivable.setDestCountryCurrencyId(Long.parseLong(regionInfo.getImportCurrency2()));
                receivable.setDestCountryRate(currencyApi.getCurrencyRate(receivable.getCurrencyId(), receivable.getDestCountryCurrencyId()).getCurrencyRate());
            }
            // 额外费用
            if (feeType == 2) {
                receivable.setDestCountryCurrencyId(Long.parseLong(regionInfo.getImportCurrency3()));
                receivable.setDestCountryRate(currencyApi.getCurrencyRate(receivable.getCurrencyId(), receivable.getDestCountryCurrencyId()).getCurrencyRate());
            }
            receivable.setDestCountrySubRate(currencyApi.getCurrencyRate(receivable.getCurrencyId(), Long.parseLong(regionInfo.getImportCurrency4())).getCurrencyRate());
        }
        this.save(receivable);
        // 返回
        return receivable.getId();
    }

    @Override
    public void updateReceivable(ReceivableUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateReceivableExists(updateReqVO.getId());
        // 更新
        ReceivableDO updateObj = ReceivableConvert.INSTANCE.convert(updateReqVO);
        this.updateById(updateObj);
    }

    @Override
    public void deleteReceivable(Long id) {
        // 校验存在
        ReceivableDO receivableDO = this.getReceivable(id);
        if (receivableDO == null) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        if (receivableDO.getReceiptId() != null) {
            throw exception(RECEIVABLE_REMOVE_BIND_RECEIPT);
        }
        // 删除
        this.removeById(id);
    }

    @Override
    public void deleteReceivableByOrderId(Long orderId) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>().eq(ReceivableDO::getOrderId, orderId));
    }

    @Override
    public void deleteReceivableByOrderIdAndFeeSource(Long orderId, Collection<Integer> feeSourceList) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>()
                .eq(ReceivableDO::getOrderId, orderId)
                .in(CollectionUtil.isNotEmpty(feeSourceList), ReceivableDO::getFeeSource, feeSourceList));
    }

    @Override
    public void deleteReceivableByOrderIdAndFeeSource(Long orderId, Integer feeSource) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>().eq(ReceivableDO::getOrderId, orderId).eq(ReceivableDO::getFeeSource, feeSource));
    }

    @Override
    public void deleteReceivableByOrderItemId(Long orderItemId) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>().eq(ReceivableDO::getOrderItemId, orderItemId));
    }

    @Override
    public ReceivableDO getReceivableByOrderItemId(Long orderItemId, Integer feeType) {
        return receivableMapper.selectOne(new LambdaQueryWrapper<ReceivableDO>()
                .eq(ReceivableDO::getOrderItemId, orderItemId)
                .eq(ReceivableDO::getFeeType, feeType)
                .eq(ReceivableDO::getFeeSource, 1));
    }

    @Override
    public void cancelReceivable(Long id) {
        ReceivableDO receivable = this.getReceivable(id);
        if (null == receivable) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        if (receivable.getDeleted()) {
            throw exception(RECEIVABLE_DELETED);
        }
        if (receivable.getState() != 0) {
            throw exception(RECEIVABLE_WRITE_OFF_ING);
        }
        this.deleteReceivable(id);
    }

    @Override
    public void updateReceivableById(ReceivableDO updateReqVO) {
        if (null == updateReqVO.getId()) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        ReceivableDO receivable = this.getReceivable(updateReqVO.getId());
        if (null == receivable) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        if (receivable.getDeleted()) {
            throw exception(RECEIVABLE_DELETED);
        }
        //收款状态（0未收款，1收款中，2已收款）
        if (receivable.getState() != 0) {
            // 取消此逻辑
            //throw exception(RECEIVABLE_WRITE_OFF_ING);
        }
        updateReqVO.setUpdateTime(new Date());
        updateReqVO.setUpdater(String.valueOf(SecurityFrameworkUtils.getLoginUserId()));
        // 更新
        this.updateById(updateReqVO);
    }

    @Override
    public void verificationReceivableIsWriteOff(Long orderId, Long orderItemId) {
        if (Objects.nonNull(orderId)) {
            // 校验整个订单的应收单是否有核销的
            long count = this.count(new LambdaQueryWrapper<ReceivableDO>().ne(ReceivableDO::getState, 0).eq(BaseDO::getDeleted, false).eq(ReceivableDO::getOrderId, orderId));
            if (count > 0) {
                throw exception(RECEIVABLE_WRITE_OFF_ING);
            }
        } else {
            // 校验订单项的应收是否核销
            if (Objects.isNull(orderItemId)) {
                throw exception(ORDER_ITEM_ID_NOT_NULL);
            }
            long count = this.count(new LambdaQueryWrapper<ReceivableDO>().ne(ReceivableDO::getState, 0).eq(BaseDO::getDeleted, false).eq(ReceivableDO::getOrderItemId, orderItemId));
            if (count > 0) {
                throw exception(RECEIVABLE_WRITE_OFF_ING);
            }
        }
    }

    private void validateReceivableExists(Long id) {
        if (this.getReceivable(id) == null) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
    }

    @Override
    public ReceivableDO getReceivable(Long id) {
        return this.getById(id);
    }

    @Override
    public List<ReceivableDO> getReceivableList(Collection<Long> ids) {
        return receivableMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ReceivableDO> getReceivablePage(ReceivableQueryVO query, PageVO page) {
        return receivableMapper.selectPage(page, query);
    }

    @Override
    public PageResult<ReceivableBackVO> receivablePage(ReceivableQueryVO query, PageVO page) {
        query.setLang(I18nMessage.getLang());
        page.setField("r.id");
        IPage<ReceivableBackVO> mpPage = MyBatisUtils.buildPage(page);
        receivableMapper.receivablePage(mpPage, query);
        mpPage.getRecords().forEach(receivableBackVO -> {
            if (receivableBackVO.getDiscountTotal() == null) {
                receivableBackVO.setDiscountTotal(BigDecimal.ZERO);
            }
            receivableBackVO.setNotIncludedTaxAmount(receivableBackVO.getTotalAmount());
            receivableBackVO.setActualAmount(receivableBackVO.getNotIncludedTaxAmount().subtract(receivableBackVO.getDiscountTotal()));
            ExchangeRateRespDTO currencyRate = currencyApi.getCurrencyRate(receivableBackVO.getCurrencyId(), receivableBackVO.getBaseCurrencyId());
            receivableBackVO.setExchangeRate(currencyRate.getCurrencyRate());
            receivableBackVO.setBaseAmount(receivableBackVO.getActualAmount().multiply(currencyRate.getCurrencyRate()));
        });
        return PageResult.of(mpPage);
    }

    @Override
    public List<WealthMoneyAmountVO> receivablePageAmount(ReceivableQueryVO query) {
        List<WealthMoneyAmountVO> voList = receivableMapper.receivablePageAmount(query);
        Map<Integer, CurrencyRespDTO> allCurrency = currencyApi.getAllCurrency();
        voList.stream().filter(vo -> allCurrency.containsKey(vo.getCurrencyId())).forEach(vo -> {
            vo.setCurrencySymbol(allCurrency.get(vo.getCurrencyId()).getFuhao());
            vo.setCurrencyName(allCurrency.get(vo.getCurrencyId()).getTitleZh());
        });
        return voList;
    }

    @Override
    public List<ReceivableBackVO> getReceivableByOrderId(Long orderId) {
        List<ReceivableBackVO> receivableBackVOList = receivableMapper.getReceivableByOrderId(orderId);
        receivableBackVOList.forEach(receivableBackVO -> {
            if (receivableBackVO.getDiscountTotal() == null) {
                receivableBackVO.setDiscountTotal(BigDecimal.ZERO);
            }
            receivableBackVO.setNotIncludedTaxAmount(receivableBackVO.getTotalAmount());
            receivableBackVO.setActualAmount(receivableBackVO.getTaxAmount().subtract(receivableBackVO.getDiscountTotal()));
            ExchangeRateRespDTO currencyRate = currencyApi.getCurrencyRate(receivableBackVO.getCurrencyId(), receivableBackVO.getBaseCurrencyId());
            receivableBackVO.setExchangeRate(currencyRate.getCurrencyRate());
            receivableBackVO.setBaseAmount(receivableBackVO.getActualAmount().multiply(currencyRate.getCurrencyRate()));
        });
        return receivableBackVOList;
    }


    @Override
    public List<ReceivableDO> getReceivableList(ReceivableQueryVO query) {
        return receivableMapper.selectList(query);
    }


    @Override
    public Long createReceipt(Long receivableId) {
        return null;
    }

    @Override
    public List<ReceivableBackVO> receivableList(List<Long> ids) {
        return receivableMapper.receivableList(ids);
    }

    @Override
    public List<ReceivableBackVO> getReceivableListByReceiptId(Long id) {
        List<ReceivableBackVO> receivableBackVOList = receivableMapper.getReceivableBackVoListByReceiptId(id);
        receivableBackVOList.forEach(receivableBackVO -> {
            if (receivableBackVO.getDiscountTotal() == null) {
                receivableBackVO.setDiscountTotal(BigDecimal.ZERO);
            }
            receivableBackVO.setNotIncludedTaxAmount(receivableBackVO.getTotalAmount());
            receivableBackVO.setActualAmount(receivableBackVO.getTaxAmount().subtract(receivableBackVO.getDiscountTotal()));
            if (receivableBackVO.getWriteOffAmount() == null) {
                receivableBackVO.setWriteOffAmount(BigDecimal.ZERO);
            }
            receivableBackVO.setNotWriteOffAmount(receivableBackVO.getActualAmount().subtract(receivableBackVO.getWriteOffAmount()));
            if (receivableBackVO.getExchangeRate() != null) {
                receivableBackVO.setWriteOffAmountUsd(receivableBackVO.getWriteOffAmount().multiply(receivableBackVO.getExchangeRate()));
                receivableBackVO.setNotWriteOffAmountUsd(receivableBackVO.getNotWriteOffAmount().multiply(receivableBackVO.getExchangeRate()));
            } else {
                receivableBackVO.setWriteOffAmountUsd(BigDecimal.ZERO);
                receivableBackVO.setNotWriteOffAmountUsd(BigDecimal.ZERO);
            }
            if (receivableBackVO.getBaseAmount() == null) {
                receivableBackVO.setBaseAmount(BigDecimal.ZERO);
            }
            // 核销比例 = 已核销金额 / 实际金额，保留两位小数字符串
            if (receivableBackVO.getWriteOffAmount().compareTo(BigDecimal.ZERO) <= 0 || receivableBackVO.getActualAmount().compareTo(BigDecimal.ZERO) <= 0) {
                receivableBackVO.setWriteOffProportion("0.00%");
            } else {
                receivableBackVO.setWriteOffProportion(receivableBackVO.getWriteOffAmount().divide(receivableBackVO.getActualAmount(), 2, RoundingMode.HALF_UP).multiply(new BigDecimal("100")) + "%");
            }
        });
        return receivableBackVOList;
    }

    @Override
    public ReceivableBackVO getReceivableBackVo(Long id) {
        return receivableMapper.getReceivableBackVo(id);
    }

    @Override
    public List<ReceivableBackVO> getReceivableListByOrderId(Long orderId) {
        return receivableMapper.getReceivableListByOrderId(orderId);
    }

    @Override
    public List<ReceivableBackVO> getFirstReceivableListByOrderId(Long orderId) {
        return receivableMapper.getFirstReceivableListByOrderId(orderId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReceivableDiscountById(ReceivableUpdateReqVO receivableUpdateReqVO) {
        ReceivableDO receivableDO = this.getReceivable(receivableUpdateReqVO.getId());
        if (null == receivableDO) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }

        if (null != receivableDO.getReceiptId()) {
            ReceiptDO receiptDO = receiptService.getReceipt(receivableDO.getReceiptId());
            if (null != receiptDO) {
                /*if (receiptDO.getState() == 4 || receiptDO.getState() == 5 || receiptDO.getState() == 6) {
                    throw exception(RECEIPT_STATUS_NOT_EXISTS);
                }*/
            }
        }

        ReceivableDO updateReceivableDO = new ReceivableDO();
        updateReceivableDO.setId(receivableDO.getId());
        updateReceivableDO.setDiscountRemark(receivableUpdateReqVO.getDiscountRemark());
        updateReceivableDO.setDiscountTotal(receivableUpdateReqVO.getDiscountTotal());
        this.updateById(updateReceivableDO);
        //存上应收款的优惠修改记录
        ReceivableDiscountDO receivableDiscountDO = new ReceivableDiscountDO();
        receivableDiscountDO.setReceivableId(receivableDO.getId());
        receivableDiscountDO.setDiscountRemark(receivableUpdateReqVO.getDiscountRemark());
        receivableDiscountDO.setDiscountTotal(receivableUpdateReqVO.getDiscountTotal());

        AdminUserRespDTO userRespDTO = adminUserApi.getUser(SecurityFrameworkUtils.getLoginUser().getId());
        if (null != userRespDTO) {
            receivableDiscountDO.setCreatorName(userRespDTO.getNickname());
        }

        receivableDiscountService.save(receivableDiscountDO);
    }

    @Override
    public List<ReceivableOrderPayedFeeTypeVO> getPayedMoneyByFeeType(Long orderId) {

        return receivableMapper.getPayedMoneyByFeeType(orderId);
    }


    @Override
    public void orderReceivableAddReceiptLog(Collection<Long> orderIdList, String receiptNo) {
        ApplyInfoVO applyInfoVO = new ApplyInfoVO();
        applyInfoVO.setName("收款单号");
        applyInfoVO.setNewValue(receiptNo);
        for (Long orderId : orderIdList) {
            // 当前订单有应收进行反核销
            applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收加入收款单", Collections.singletonList(applyInfoVO)));
        }
    }

    @Override
    public void orderReceivableWriteOffLogAdd(Collection<Long> orderIdList, String receiptNo) {
        ApplyInfoVO applyInfoVO = new ApplyInfoVO();
        applyInfoVO.setName("收款单号");
        applyInfoVO.setNewValue(receiptNo);
        for (Long orderId : orderIdList) {
            List<ReceivableDO> list = receivableMapper.selectList(new LambdaQueryWrapper<ReceivableDO>().eq(ReceivableDO::getOrderId, orderId));
            // 如果没有找到订单应收，是不正常的，不需要去添加应收核销操作日志
            if (CollectionUtil.isNotEmpty(list)) {
                // 判断当前订单的应收是否有未核销完成的
                long count = list.stream().filter(r -> r.getState() != 2).count();
                if (count == 0) {
                    // 当前订单所有已生成的应收已经核销完成
                    log.error("-------------------------------");
                    applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收全部核销", Collections.singletonList(applyInfoVO)));
                } else {
                    log.error("222222222222222222222222222222222");
                    applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收部分核销", Collections.singletonList(applyInfoVO)));
                }
            }
        }
    }

    @Override
    public void orderReceivableCancelWriteOffLogAdd(Collection<Long> orderIdList, String receiptNo) {
        ApplyInfoVO applyInfoVO = new ApplyInfoVO();
        applyInfoVO.setName("收款单号");
        applyInfoVO.setNewValue(receiptNo);
        for (Long orderId : orderIdList) {
            // 当前订单有应收进行反核销
            applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收反核销", Collections.singletonList(applyInfoVO)));
        }
    }

    @Override
    public List<ReceivableDO> receivableListByOrderId(Long orderId) {
        return receivableMapper.receivableListByOrderId(orderId);
    }


    @Override
    public List<ReceivableDO> allByOrderId(Long orderId) {
        return receivableMapper.allByOrderId(orderId);
    }

    @Override
    public List<ReceivableDO> getOrderPayList(Collection<Long> orderIdList) {
        return this.list(new LambdaQueryWrapper<ReceivableDO>()
                .ne(ReceivableDO::getState, 0)
                .eq(BaseDO::getDeleted, false)
                .in(ReceivableDO::getOrderId, orderIdList)
        );
    }

    @Override
    public void revokeDeleteReceivableById(Long receivableId) {
        receivableMapper.revokeDeleteReceivableById(receivableId);
    }

    @Override
    public CostDetailBackVO getCostDetailPage(CostDetailPageQueryVO query, PageVO page) {
        Map<Integer, CurrencyRespDTO> allCurrency = currencyApi.getAllCurrency();

        CompletableFuture<IPage<CostDetailPageVO>> costDetailPageFuture = CompletableFuture.supplyAsync(() -> {
            IPage<CostDetailPageVO> pageInfo = new Page<>(page.getPage(), page.getRows());
            return receivableMapper.getCostDetailPage(pageInfo, query);
        });
        CompletableFuture<List<WealthMoneyAmountVO>> searchStatisticsFuture = CompletableFuture.supplyAsync(() -> receivableMapper.costDetailSearchAmount(query));
        CompletableFuture<List<WealthMoneyAmountVO>> totalStatisticsFuture = CompletableFuture.supplyAsync(() -> receivableMapper.costDetailAmount());

        try {
            CompletableFuture<Void> allFutures = CompletableFuture.allOf(costDetailPageFuture, searchStatisticsFuture, totalStatisticsFuture);
            allFutures.join();

            IPage<CostDetailPageVO> costDetailPage = costDetailPageFuture.get();
            List<WealthMoneyAmountVO> searchStatistics = searchStatisticsFuture.get();
            List<WealthMoneyAmountVO> totalStatistics = totalStatisticsFuture.get();

            if (CollectionUtil.isNotEmpty(searchStatistics)) {
                searchStatistics = searchStatistics.stream().filter(vo -> allCurrency.containsKey(vo.getCurrencyId())).collect(Collectors.toList());
                calculateStatisticalAmount(allCurrency, searchStatistics);
            }
            totalStatistics = totalStatistics.stream().filter(vo -> allCurrency.containsKey(vo.getCurrencyId())).collect(Collectors.toList());
            calculateStatisticalAmount(allCurrency, totalStatistics);

            CostDetailBackVO.CostDetailBackVOBuilder voBuilder = CostDetailBackVO.builder()
                    .searchStatistics(searchStatistics)
                    .totalStatistics(totalStatistics);
            List<CostDetailPageVO> records = costDetailPage.getRecords();

            if (CollectionUtil.isNotEmpty(records)) {
                records.forEach(vo -> {
                    vo.setUpdater(vo.getUpdaterName());
                    vo.setCurrencySymbol(allCurrency.get(vo.getCurrencyId()).getFuhao());
                    vo.setNotIncludedTaxAmount(vo.getTotalAmount());
                    if (vo.getDiscountTotal() == null) {
                        vo.setDiscountTotal(BigDecimal.ZERO);
                    }
                    vo.setReceivableTotalAmount(vo.getTaxAmount().subtract(vo.getDiscountTotal()));
                    if (vo.getWriteOffAmount() == null) {
                        vo.setWriteOffAmount(BigDecimal.ZERO);
                    }
                    vo.setNotWriteOffAmount(vo.getReceivableTotalAmount().subtract(vo.getWriteOffAmount()));
                    if (vo.getWriteOffAmount().compareTo(BigDecimal.ZERO) <= 0 || vo.getReceivableTotalAmount().compareTo(BigDecimal.ZERO) <= 0) {
                        vo.setWriteOffProportion("0.00%");
                    } else {
                        vo.setWriteOffProportion(vo.getWriteOffAmount().divide(vo.getReceivableTotalAmount(), 4, RoundingMode.HALF_UP).multiply(new BigDecimal("100")) + "%");
                    }
                    // 存在应收明细关联的定单不存在的情况
                    if (vo.getDestinationCity() != null && vo.getDestinationCountry() != null) {
                        RegionDTO regionCity = regionApi.getRegionById(vo.getDestinationCity().longValue());
                        vo.setDestinationCityZh(regionCity.getTitleZh());
                        vo.setDestinationCityEn(regionCity.getTitleEn());
                        RegionDTO regionCountry = regionApi.getRegionById(vo.getDestinationCountry().longValue());
                        vo.setDestinationCountryZh(regionCountry.getTitleZh());
                        vo.setDestinationCountryEn(regionCountry.getTitleEn());
                    }
                });
                return voBuilder
                        .list(records)
                        .total(costDetailPage.getTotal())
                        .rows(costDetailPage.getSize())
                        .page(costDetailPage.getCurrent())
                        .pages(costDetailPage.getPages())
                        .build();
            }
            return voBuilder
                    .list(Collections.EMPTY_LIST)
                    .total(costDetailPage.getTotal())
                    .rows(costDetailPage.getSize())
                    .page(costDetailPage.getCurrent())
                    .pages(costDetailPage.getPages())
                    .build();
        } catch (InterruptedException | ExecutionException e) {
            throw new RuntimeException("查询异常");
        }
    }

    /**
     * 实际金额 = 含税金额 - 优惠金额
     * 未核销金额 = 实际金额 - 核销金额
     * TODO 暂定 不含税金额 = 总金额
     *
     * @param allCurrency
     * @param statistics
     */
    private void calculateStatisticalAmount(Map<Integer, CurrencyRespDTO> allCurrency, List<WealthMoneyAmountVO> statistics) {
        statistics.forEach(vo -> {
            vo.setCurrencySymbol(allCurrency.get(vo.getCurrencyId()).getFuhao());
            vo.setCurrencyName(allCurrency.get(vo.getCurrencyId()).getTitleZh());
            if (vo.getWriteOffAmount() == null) {
                vo.setWriteOffAmount(BigDecimal.ZERO);
            }
            if (vo.getDiscountTotal() == null) {
                vo.setDiscountTotal(BigDecimal.ZERO);
            }
            vo.setNotIncludedTaxAmount(vo.getTotalAmount());
            vo.setReceivableTotalAmount(vo.getTaxAmount().subtract(vo.getDiscountTotal()));
            vo.setNotWriteOffAmount(vo.getReceivableTotalAmount().subtract(vo.getWriteOffAmount()));
        });
    }

    @Override
    public PageResult<BatchGenReceiptPageVO> batchGenReceiptPage(BatchGenReceiptPageQueryVO query, PageVO page) {
        int start = (page.getPage() - 1) * page.getRows();
        int size = page.getRows();
        List<BatchGenReceiptPageVO> batchGenReceiptPageVOS = receivableMapper.batchGenReceiptPage(start, size, query);
        int total = receivableMapper.countBatchGenReceiptPage(query);
        batchGenReceiptPageVOS.forEach(vo -> {
            List<CurrencyAmount> receivableAmount = receivableMapper.calculateReceivableAmountByOrderId(vo.getOrderId());
            vo.setReceivableAmount(receivableAmount);
            CustomerDTO customer;
            if (vo.getDrawee().equals(1)) {
                customer = customerApi.getCustomerInfo(vo.getConsignorId());
            } else {
                customer = customerApi.getCustomerInfo(vo.getConsigneeId());
            }
            vo.setPayerId(customer.getId());
            vo.setOpenInvoice(customer.getDefaultBilling());
            vo.setCustomerName(customer.getName());
            vo.setCustomerNo(customer.getNumber());
            vo.setPayerName(vo.getPayerName());
        });
        return new PageResult<>(batchGenReceiptPageVOS, total, page.getRows(), page.getPage(), (total + page.getRows() - 1) / page.getRows());
    }

    @Override
    public Long getDestCountryCurrencyId(Long orderId) {
        Long countryId = orderApi.getDestCountryByOrderId(orderId);
        if (countryId == null) {
            return null;
        }
        RegionDTO regionById = regionApi.getRegionById(countryId);
        return Long.parseLong(regionById.getImportCurrency5());
    }

    @Override
    public void createWriteOffRecord(ReceivableWriteOffReqVO receivableWriteOffReqVO, ReceiptItemDO receiptItem, Integer incomeBelong) {
        ReceivableDO receivableDO = receivableMapper.selectById(receivableWriteOffReqVO.getReceivableId());
        if (receivableDO == null) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        BigDecimal receivableWriteOffAmount;
        ExchangeRateRespDTO currencyRate = currencyApi.getCurrencyRate(receiptItem.getWriteOffCurrencyId(), receivableDO.getCurrencyId());
        if (currencyRate == null) {
            receivableWriteOffAmount = receivableWriteOffReqVO.getWriteOffAmount();
        } else {
            receivableWriteOffAmount = receivableWriteOffReqVO.getWriteOffAmount().multiply(currencyRate.getCurrencyRate());
        }
        ReceivableWriteOffRecordDO writeOffRecord = ReceivableWriteOffRecordDO.builder()
                .receivableId(receivableWriteOffReqVO.getReceivableId())
                .writeOffAmount(receivableWriteOffReqVO.getWriteOffAmount())
                .receiptItemId(receiptItem.getId())
                .incomeBelong(incomeBelong)
                .receivableWriteOffAmount(receivableWriteOffAmount)
                .orderId(receivableDO.getOrderId())
                .build();
        receivableWriteOffRecordMapper.insert(writeOffRecord);
    }

    @Override
    public ReceivableWriteOffRecordDO getReceivableWriteOffRecord(Long receivableId, Long receiptItemId) {
        return receivableWriteOffRecordMapper.selectOne("receivable_id", receivableId, "receipt_item_id", receiptItemId);
    }

    @Override
    public List<ReceivableInItemVO> getListForCreateReceiptItem(Long receiptId) {
        List<ReceivableDO> receivableDOS = this.selectList("receipt_id", receiptId);
        List<ReceivableInItemVO> receivableInItemVOS = ReceivableConvert.INSTANCE.convertToItemVOList(receivableDOS);
        for (ReceivableInItemVO receivableInItemVO : receivableInItemVOS) {
            receivableInItemVO.setActualAmount(receivableInItemVO.getTaxAmount().subtract(receivableInItemVO.getDiscountTotal() == null ? BigDecimal.ZERO : receivableInItemVO.getDiscountTotal()));
            BigDecimal totalWriteOffAmount = receivableWriteOffRecordMapper.getTotalAmountForReceivable(receivableInItemVO.getId());
            if (totalWriteOffAmount == null) {
                totalWriteOffAmount = BigDecimal.ZERO;
            }
            receivableInItemVO.setWriteOffAmountUsd(totalWriteOffAmount);
            receivableInItemVO.setNotWriteOffAmountUsd(receivableInItemVO.getBaseAmount().subtract(totalWriteOffAmount));
        }
        receivableInItemVOS.sort(Comparator.comparing(ReceivableInItemVO::getNotWriteOffAmountUsd));
        return receivableInItemVOS;
    }


    @Override
    public List<ReceivableInItemVO> getReceivableByReceiptItem(Long receiptItemId) {
        ReceiptItemDO receiptItemDO = receiptItemMapper.selectById(receiptItemId);
        List<ReceivableDO> receivableDOS = this.selectList("receipt_id", receiptItemDO.getReceiptId());
        List<ReceivableInItemVO> receivableInItemVOS = ReceivableConvert.INSTANCE.convertToItemVOList(receivableDOS);
        for (ReceivableInItemVO receivableInItemVO : receivableInItemVOS) {
            receivableInItemVO.setActualAmount(receivableInItemVO.getTaxAmount().subtract(receivableInItemVO.getDiscountTotal() == null ? BigDecimal.ZERO : receivableInItemVO.getDiscountTotal()));
            ReceivableWriteOffRecordDO receivableWriteOffRecordDO = this.getReceivableWriteOffRecord(receivableInItemVO.getId(), receiptItemId);
            if (null == receivableWriteOffRecordDO) {
                receivableInItemVO.setWriteOffAmount(BigDecimal.ZERO);
            } else {
                receivableInItemVO.setWriteOffAmount(receivableWriteOffRecordDO.getWriteOffAmount());
            }
            BigDecimal totalWriteOffAmount = receivableWriteOffRecordMapper.getTotalAmountForReceivable(receivableInItemVO.getId());
            if (totalWriteOffAmount == null) {
                receivableInItemVO.setWriteOffAmountUsd(BigDecimal.ZERO);
            } else {
                receivableInItemVO.setWriteOffAmountUsd(totalWriteOffAmount.subtract(receivableInItemVO.getWriteOffAmount()));
            }
            receivableInItemVO.setNotWriteOffAmountUsd(receivableInItemVO.getBaseAmount().subtract(receivableInItemVO.getWriteOffAmountUsd()));
        }
        receivableInItemVOS.sort(Comparator.comparing(ReceivableInItemVO::getNotWriteOffAmountUsd));
        return receivableInItemVOS;
    }

    @Override
    public ReceivablePageResult<BatchGenReceiptReceivablePageVO> batchGenReceiptReceivablePage(BatchGenReceiptReceivablePageQueryVO query, PageVO page) {
        IPage<BatchGenReceiptReceivablePageVO> mpPage = MyBatisUtils.buildPage(page);
        receivableMapper.batchGenReceiptReceivablePage(mpPage, query);
        ReceivablePageResult<BatchGenReceiptReceivablePageVO> pageResult = ReceivablePageResult.of(mpPage);
        List<CurrencyAmount> totalActualAmount = receivableMapper.batchGenReceiptReceivablAmount(query);
        pageResult.setTotalActualAmount(totalActualAmount);
        return pageResult;
    }

    @Override
    public void deleteWriteOffRecord(ReceiptItemDO receiptItemDO) {
        receivableWriteOffRecordMapper.deleteWriteOffRecord(receiptItemDO.getId());
    }

    @Override
    public ReceivableBoxReportVO getReceivableBoxReport(ReceivableBoxReportQueryVO query, PageVO page) {
        assembleBoxReportQuery(query);
        // 打开页面默认选择到港时间为本年度的数据，按结算时间倒序，卸柜/到仓时间倒序，清关时间倒序，到港时间倒序
        ReceivableBoxReportVO receivableBoxReport = null;
        // 查询报表缓存
        String queryJson = JSON.toJSONString(query);
        if (!query.getForceUpdateCache() && receivableBoxReport == null) {
            receivableBoxReport = getReceivableBoxReportCache(queryJson);
        }
        if (receivableBoxReport != null) {
            if (receivableBoxReport.getPage().intValue() == page.getPage() && receivableBoxReport.getRows() == page.getRows()) {
                return receivableBoxReport;
            }
            // 如果只是分页条件不同,重新获取分页列表
            receivableBoxReport.setPage(page.getPage().longValue());
            receivableBoxReport.setRows((long) page.getRows());
            List<ReceivableBoxReportPageVO> records = receivableMapper.getReceivableBoxPage(query);
            records = records.stream()
                    .sorted(Comparator.comparing(ReceivableBoxReportPageVO::getSlSettledTime, Comparator.nullsLast(query.getJsTimeDesc() ? (Comparator<Date>) Comparator.reverseOrder() : (Comparator<Date>) Comparator.naturalOrder()))
                            .thenComparing(ReceivableBoxReportPageVO::getUlWarehouseTime, Comparator.nullsLast(query.getUlWarehouseTimeDesc() ? (Comparator<Date>) Comparator.reverseOrder() : (Comparator<Date>) Comparator.naturalOrder()))
                            .thenComparing(ReceivableBoxReportPageVO::getQgDate, Comparator.nullsLast(query.getQingguanTimeDesc() ? (Comparator<Date>) Comparator.reverseOrder() : (Comparator<Date>) Comparator.naturalOrder()))
                            .thenComparing(ReceivableBoxReportPageVO::getDgDate, Comparator.nullsLast(query.getDaogangTimeDesc() ? (Comparator<Date>) Comparator.reverseOrder() : (Comparator<Date>) Comparator.naturalOrder())))
                    .collect(Collectors.toList());
            int startIndex = Math.max(0, (page.getPage() - 1) * page.getRows());
            int endIndex = Math.min(startIndex + page.getRows(), records.size());
            List<ReceivableBoxReportPageVO> pageVOList = records.subList(startIndex, endIndex);
            if (CollectionUtil.isEmpty(records) || CollectionUtil.isEmpty(pageVOList)) {
                receivableBoxReport.setList(pageVOList);
                return receivableBoxReport;
            }
            // 获取当页自编号对应的订单
            List<Long> shipmentIdList = pageVOList.stream().map(ReceivableBoxReportPageVO::getId).collect(Collectors.toList());
            List<ReceivebleBoxRelationOrder> relationOrders = receivableMapper.getReceivableBoxRelationOrders(shipmentIdList);
            Map<Long, List<ReceivebleBoxRelationOrder>> relationOrderMap = relationOrders.stream().collect(Collectors.groupingBy(ReceivebleBoxRelationOrder::getShipmentId));
            Map<Integer, CurrencyRespDTO> allCurrency = currencyApi.getAllCurrency();
            List<ExchangeRateRespDTO> allCurrencyRate = currencyApi.getAllCurrencyRate();
            ReceivableBoxReportVO.Amount pageAmount = new ReceivableBoxReportVO.Amount();
            assembleReportAmount(query.getForceUpdateCache(), pageVOList, relationOrderMap, allCurrencyRate, allCurrency, null, pageVOList, pageAmount);
            receivableBoxReport.setList(pageVOList);
            receivableBoxReport.setPageAmount(pageAmount);
            redisHelper.set(BOX_REPORT_QUERY_CACHE, JSON.toJSONString(receivableBoxReport));
            return receivableBoxReport;
        }
        // 该查询条件所有的自编号列表
        List<ReceivableBoxReportPageVO> records = receivableMapper.getReceivableBoxPage(query);
        records = records.stream()
                .sorted(Comparator.comparing(ReceivableBoxReportPageVO::getSlSettledTime, Comparator.nullsLast(query.getJsTimeDesc() ? (Comparator<Date>) Comparator.reverseOrder() : (Comparator<Date>) Comparator.naturalOrder()))
                        .thenComparing(ReceivableBoxReportPageVO::getUlWarehouseTime, Comparator.nullsLast(query.getUlWarehouseTimeDesc() ? (Comparator<Date>) Comparator.reverseOrder() : (Comparator<Date>) Comparator.naturalOrder()))
                        .thenComparing(ReceivableBoxReportPageVO::getQgDate, Comparator.nullsLast(query.getQingguanTimeDesc() ? (Comparator<Date>) Comparator.reverseOrder() : (Comparator<Date>) Comparator.naturalOrder()))
                        .thenComparing(ReceivableBoxReportPageVO::getDgDate, Comparator.nullsLast(query.getDaogangTimeDesc() ? (Comparator<Date>) Comparator.reverseOrder() : (Comparator<Date>) Comparator.naturalOrder())))
                .collect(Collectors.toList());
        int startIndex = Math.max(0, (page.getPage() - 1) * page.getRows());
        int endIndex = Math.min(startIndex + page.getRows(), records.size());
        List<ReceivableBoxReportPageVO> pageVOList = records.subList(startIndex, endIndex);
        ReceivableBoxReportVO.ReceivableBoxReportVOBuilder voBuilder = ReceivableBoxReportVO.builder()
                .list(pageVOList)
                .total((long) records.size())
                .rows((long) page.getRows())
                .page((long) page.getPage())
                .pages((long) (records.size() / page.getRows() + 1));
        if (CollectionUtil.isEmpty(records)) {
            return voBuilder.build();
        }
        // 获取所有自编号对应的订单
        List<Long> shipmentIdList = records.stream().map(ReceivableBoxReportPageVO::getId).collect(Collectors.toList());
        List<ReceivebleBoxRelationOrder> relationOrders = receivableMapper.getReceivableBoxRelationOrders(shipmentIdList);
        Map<Long, List<ReceivebleBoxRelationOrder>> relationOrderMap = relationOrders.stream().collect(Collectors.groupingBy(ReceivebleBoxRelationOrder::getShipmentId));
        Map<Integer, CurrencyRespDTO> allCurrency = currencyApi.getAllCurrency();
        List<ExchangeRateRespDTO> allCurrencyRate = currencyApi.getAllCurrencyRate();
        ReceivableBoxReportVO.Amount pageAmount = new ReceivableBoxReportVO.Amount();
        ReceivableBoxReportVO.Amount queryAmount = new ReceivableBoxReportVO.Amount();
        assembleReportAmount(query.getForceUpdateCache(),records, relationOrderMap, allCurrencyRate, allCurrency, queryAmount, pageVOList, pageAmount);
        ReceivableBoxReportVO reportVO = voBuilder
                .pageAmount(pageAmount)
                .queryAmount(queryAmount)
                .build();

        redisHelper.set(BOX_REPORT_QUERY_CONDITION_CACHE, queryJson, 60 * 60);
        redisHelper.set(BOX_REPORT_QUERY_CACHE, JSON.toJSONString(reportVO), 60 * 60);
        return reportVO;
    }

    private void assembleBoxReportQuery(ReceivableBoxReportQueryVO query) {
        if (StringUtils.isNotBlank(query.getSelfNo()) && (query.getSelfNo().contains(",") || query.getSelfNo().contains("，"))) {
            String[] selfNoArray = query.getSelfNo().split(",");
            query.setSelfNoList(Arrays.asList(selfNoArray));
            query.setSelfNo(null);
        }
        if (CollectionUtil.isNotEmpty(query.getDestCity()) && CollectionUtil.isEmpty(query.getDestWarehouseIdList())) {
            query.setDestWarehouseIdList(receivableMapper.getDestWarehouseIdListByDestCity(query.getDestCity()));
        }
    }

    /**
     * 整合报表金额数据
     *
     * @param records
     * @param relationOrderMap
     * @param allCurrencyRate
     * @param allCurrency
     * @param queryAmount
     * @param pageVOList
     * @param pageAmount
     */

    private void assembleReportAmount(boolean forceUpdateCache, List<ReceivableBoxReportPageVO> records, Map<Long, List<ReceivebleBoxRelationOrder>> relationOrderMap, List<ExchangeRateRespDTO> allCurrencyRate, Map<Integer, CurrencyRespDTO> allCurrency, ReceivableBoxReportVO.Amount queryAmount, List<ReceivableBoxReportPageVO> pageVOList, ReceivableBoxReportVO.Amount pageAmount) {
        for (ReceivableBoxReportPageVO record : records) {
            setLadingBillStatus(record);
            ReceivableBoxAmount receivableBoxAmount;
            String receivableBoxAmountCache = redisHelper.get(BOX_AMOUNT_CACHE + record.getId());
            if (receivableBoxAmountCache != null && !forceUpdateCache) {
                receivableBoxAmount = JSONObject.parseObject(receivableBoxAmountCache, ReceivableBoxAmount.class);
            } else {
                List<ReceivebleBoxRelationOrder> orders = relationOrderMap.get(record.getId());
                if (CollectionUtil.isEmpty(orders)) {
                    continue;
                }
                Set<Long> orderIdSets = orders.stream().map(ReceivebleBoxRelationOrder::getOrderId).collect(Collectors.toSet());
                RegionDTO regionInfo = regionApi.getRegionById(record.getDestCountryId());
                if (null == regionInfo) {
                    continue;
                }
                // 查询该自编号所有的明细
                List<ReceivableDO> boxReceivables = receivableMapper.selectBoxReceivablesByOrderIds(orderIdSets);
                if (CollectionUtil.isEmpty(boxReceivables)) {
                    continue;
                }
                List<ReceivableIncomeBelong> receivableIncomeBelongs = receiptItemMapper.selectAllWriteOffItemByOrders(orderIdSets);
                Map<Long, List<ReceivableIncomeBelong>> receivableIncomeBelongMap = new HashMap<>();
                if (CollectionUtil.isNotEmpty(receivableIncomeBelongs)) {
                    // 根据明细id分组
                    receivableIncomeBelongMap = receivableIncomeBelongs.stream().collect(Collectors.groupingBy(ReceivableIncomeBelong::getReceivableId));
                }
                receivableBoxAmount = this.computeReceivableBoxSummary(boxReceivables, regionInfo, receivableIncomeBelongMap, allCurrencyRate);
                redisHelper.set(BOX_AMOUNT_CACHE + record.getId(), JSONObject.toJSONString(receivableBoxAmount), 60 * 60 * 24);
            }
            // 整合数据给返回赋值
            assembleRecord(receivableBoxAmount, record, allCurrency);
            if (queryAmount != null) {
                mergeAmount(queryAmount, receivableBoxAmount);
            }
            if (pageVOList.contains(record)) {
                mergeAmount(pageAmount, receivableBoxAmount);
            }
        }
    }

    /**
     * 更新某个自编号的缓存
     */
    @Override
    public void updateBoxAmountCache(List<Long> shipmentIdList, boolean isForceUpdate) {
        if (CollectionUtil.isEmpty(shipmentIdList)) {
            return;
        }
        List<ReceivebleBoxRelationOrder> relationOrders = receivableMapper.getReceivableBoxRelationOrders(shipmentIdList);
        Map<Long, List<ReceivebleBoxRelationOrder>> relationOrderMap = relationOrders.stream().collect(Collectors.groupingBy(ReceivebleBoxRelationOrder::getShipmentId));
        List<ExchangeRateRespDTO> allCurrencyRate = currencyApi.getAllCurrencyRate();
        for (Long boxId : shipmentIdList) {
            if (redisHelper.hasKey(BOX_AMOUNT_CACHE + boxId) && !isForceUpdate) {
                continue;
            }
            List<ReceivebleBoxRelationOrder> orders = relationOrderMap.get(boxId);
            if (CollectionUtil.isEmpty(orders)) {
                continue;
            }
            Set<Long> orderIdSets = orders.stream().map(ReceivebleBoxRelationOrder::getOrderId).collect(Collectors.toSet());
            RegionDTO regionInfo = regionApi.getRegionById(orders.get(0).getDestCountryId());
            if (null == regionInfo) {
                continue;
            }
            // 查询该自编号所有的明细
            List<ReceivableDO> boxReceivables = receivableMapper.selectBoxReceivablesByOrderIds(orderIdSets);
            if (CollectionUtil.isEmpty(boxReceivables)) {
                continue;
            }
            List<ReceivableIncomeBelong> receivableIncomeBelongs = receiptItemMapper.selectAllWriteOffItemByOrders(orderIdSets);
            Map<Long, List<ReceivableIncomeBelong>> receivableIncomeBelongMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(receivableIncomeBelongs)) {
                // 根据明细id分组
                receivableIncomeBelongMap = receivableIncomeBelongs.stream().collect(Collectors.groupingBy(ReceivableIncomeBelong::getReceivableId));
            }
            ReceivableBoxAmount receivableBoxAmount = this.computeReceivableBoxSummary(boxReceivables, regionInfo, receivableIncomeBelongMap, allCurrencyRate);
            redisHelper.set(BOX_AMOUNT_CACHE + boxId, JSONObject.toJSONString(receivableBoxAmount), 60 * 60 * 24);
        }
    }

    /**
     * 查询报表缓存
     */
    private ReceivableBoxReportVO getReceivableBoxReportCache(String queryJson) {
        String boxReportQueryConditionCache = redisHelper.get(BOX_REPORT_QUERY_CONDITION_CACHE);
        if (boxReportQueryConditionCache != null) {
            if (boxReportQueryConditionCache.equals(queryJson)) {
                String boxReportCache = redisHelper.get(BOX_REPORT_QUERY_CACHE);
                if (boxReportCache != null) {
                    return JSON.parseObject(boxReportCache, ReceivableBoxReportVO.class);
                }
            }
        }
        return null;
    }

    /**
     * 计算自编号收款
     *
     * @param boxReceivables
     * @param regionInfo
     */
    private ReceivableBoxAmount computeReceivableBoxSummary(List<ReceivableDO> boxReceivables,
                                                            RegionDTO regionInfo,
                                                            Map<Long, List<ReceivableIncomeBelong>> receivableIncomeBelongMap,
                                                            List<ExchangeRateRespDTO> allCurrencyRate) {
        Long freightCurrency = Long.valueOf(regionInfo.getImportCurrency1());
        Long clearanceCurrency = Long.valueOf(regionInfo.getImportCurrency2());
        Long otherFeeCurrency = Long.valueOf(regionInfo.getImportCurrency3());
        Long otherSubCurrency = Long.valueOf(regionInfo.getImportCurrency4());
        Long receivableCurrency = Long.valueOf(regionInfo.getImportCurrency5());

        ReceivableBoxAmount receivableBoxAmount = new ReceivableBoxAmount(freightCurrency, clearanceCurrency, otherFeeCurrency, otherSubCurrency, receivableCurrency);
        ReceivableCurrencyAmount receivableBaseCurrency = receivableBoxAmount.getReceivableBaseCurrency();
        ReceivableCurrencyAmount receivableBaseCurrencySettle = receivableBoxAmount.getReceivableBaseCurrencySettle();
        for (ReceivableDO boxReceivable : boxReceivables) {
            List<ReceivableIncomeBelong> receivableIncomeBelongs = receivableIncomeBelongMap.get(boxReceivable.getId());
            BigDecimal baseCurrencyAmount;
            if (boxReceivable.getBaseAmount() == null) {
                BigDecimal exchangeRate = getExchangeRate(allCurrencyRate, boxReceivable.getCurrencyId(), boxReceivable.getBaseCurrencyId());
                baseCurrencyAmount = boxReceivable.getTotalAmount().multiply(exchangeRate);
            } else {
                baseCurrencyAmount = boxReceivable.getBaseAmount();
            }
            receivableBaseCurrency.addAmount(baseCurrencyAmount);
            if (CollectionUtil.isNotEmpty(receivableIncomeBelongs)) {
                BigDecimal writeOffBaseAmount = receivableIncomeBelongs.stream().map(ReceivableIncomeBelong::getWriteOffBaseAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                receivableBaseCurrencySettle.addAmount(writeOffBaseAmount);
            }
            // 应收款-汇总
            ReceivableCurrencyAmount currencyAmount = new ReceivableCurrencyAmount(boxReceivable.getCurrencyId(), boxReceivable.getTaxAmount() == null ? boxReceivable.getTotalAmount() : boxReceivable.getTaxAmount().subtract(boxReceivable.getDiscountTotal() == null ? BigDecimal.ZERO : boxReceivable.getDiscountTotal()));
            int index = receivableBoxAmount.receivableTotalSummary.indexOf(currencyAmount);
            if (index >= 0) {
                receivableBoxAmount.receivableTotalSummary.get(index).addAmount(currencyAmount.getAmount());
            } else {
                receivableBoxAmount.receivableTotalSummary.add(currencyAmount);
            }
            // 应收款-结算币种
            if (boxReceivable.getCurrencyId().equals(receivableCurrency)) {
                receivableBoxAmount.receivableTotalSettle.addAmount(currencyAmount.getAmount());
            } else {
                receivableBoxAmount.receivableTotalSettle.addAmount(currencyAmount.getAmount().multiply(getExchangeRate(allCurrencyRate, boxReceivable.getCurrencyId(), receivableCurrency)));
            }
            // 判断属于哪种费用 0运费,1 清关费,2 额外费用
            Integer feeType = this.getFeeType(boxReceivable.getFeeType());
            // TODO 折扣金额计算的成交单价和原价暂时不知道哪拿,先只取优惠金额
            BigDecimal discount = boxReceivable.getDiscountTotal() == null ? BigDecimal.ZERO : boxReceivable.getDiscountTotal();
            BigDecimal notWrittenOff = Objects.equals(boxReceivable.getWriteOffAmount(), BigDecimal.ZERO) ? currencyAmount.getAmount() : currencyAmount.getAmount().subtract(boxReceivable.getWriteOffAmount());

            BigDecimal freightRate = getExchangeRate(allCurrencyRate, boxReceivable.getCurrencyId(), freightCurrency);
            BigDecimal clearanceRate = getExchangeRate(allCurrencyRate, boxReceivable.getCurrencyId(), clearanceCurrency);
            BigDecimal otherFeeRate = getExchangeRate(allCurrencyRate, boxReceivable.getCurrencyId(), otherFeeCurrency);
            BigDecimal otherSubRate = getExchangeRate(allCurrencyRate, freightCurrency, otherSubCurrency);
            switch (feeType) {
                case 0:
                    if (boxReceivable.getCurrencyId().equals(freightCurrency)) {
                        receivableBoxAmount.receivableDetailFreight.addAmount(currencyAmount.getAmount());
                        receivableBoxAmount.receivableDiscountFreight.addAmount(discount);
                        receivableBoxAmount.receivableUnsettledFreight.addAmount(notWrittenOff);
                    } else {
                        receivableBoxAmount.receivableDetailFreight.addAmount(currencyAmount.getAmount().multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : freightRate));
                        receivableBoxAmount.receivableDiscountFreight.addAmount(discount.multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : freightRate));
                        receivableBoxAmount.receivableUnsettledFreight.addAmount(notWrittenOff.multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : freightRate));
                    }
                    // 计算始发国或目的国实收
                    if (receivableIncomeBelongs != null) {
                        for (ReceivableIncomeBelong receivableIncomeBelong : receivableIncomeBelongs) {
                            if (receivableIncomeBelong.getIncomeBelong() == 1) {
                                receivableBoxAmount.receivableStartCountryFreight.addAmount(receivableIncomeBelong.getWriteOffAmount().multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : freightRate));
                            } else if (receivableIncomeBelong.getIncomeBelong() == 0) {
                                receivableBoxAmount.receivableDestCountryFreight.addAmount(receivableIncomeBelong.getWriteOffAmount().multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : freightRate));
                            }
                        }
                    }
                    break;
                case 1:
                    if (boxReceivable.getCurrencyId().equals(clearanceCurrency)) {
                        receivableBoxAmount.receivableDetailClearance.addAmount(currencyAmount.getAmount());
                        receivableBoxAmount.receivableDiscountClearance.addAmount(discount);
                        receivableBoxAmount.receivableUnsettledClearance.addAmount(notWrittenOff);
                    } else {
                        receivableBoxAmount.receivableDetailClearance.addAmount(currencyAmount.getAmount().multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : clearanceRate));
                        receivableBoxAmount.receivableDiscountClearance.addAmount(discount.multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : clearanceRate));
                        receivableBoxAmount.receivableUnsettledClearance.addAmount(notWrittenOff.multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : clearanceRate));
                    }
                    // 计算始发国或目的国实收
                    if (receivableIncomeBelongs != null) {
                        for (ReceivableIncomeBelong receivableIncomeBelong : receivableIncomeBelongs) {
                            if (receivableIncomeBelong.getIncomeBelong() == 1) {
                                receivableBoxAmount.receivableStartCountryClearance.addAmount(receivableIncomeBelong.getWriteOffAmount().multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : clearanceRate));
                            } else if (receivableIncomeBelong.getIncomeBelong() == 0) {
                                receivableBoxAmount.receivableDestCountryClearance.addAmount(receivableIncomeBelong.getWriteOffAmount().multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : clearanceRate));
                            }
                        }
                    }
                    break;
                case 2:
                    if (boxReceivable.getCurrencyId().equals(otherFeeCurrency)) {
                        receivableBoxAmount.receivableDetailOtherFee.addAmount(currencyAmount.getAmount());
                        receivableBoxAmount.receivableDiscountOtherFee.addAmount(discount);
                        receivableBoxAmount.receivableUnsettledOtherFee.addAmount(notWrittenOff);
                    } else {
                        receivableBoxAmount.receivableDetailOtherFee.addAmount(currencyAmount.getAmount().multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : otherFeeRate));
                        receivableBoxAmount.receivableDiscountOtherFee.addAmount(discount.multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : otherFeeRate));
                        receivableBoxAmount.receivableUnsettledOtherFee.addAmount(notWrittenOff.multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : otherFeeRate));
                    }
                    // 计算始发国或目的国实收
                    if (receivableIncomeBelongs != null) {
                        for (ReceivableIncomeBelong receivableIncomeBelong : receivableIncomeBelongs) {
                            if (receivableIncomeBelong.getIncomeBelong() == 1) {
                                receivableBoxAmount.receivableStartCountryOtherFee.addAmount(receivableIncomeBelong.getWriteOffAmount().multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : otherFeeRate));
                                receivableBoxAmount.receivableStartCountryOtherFeeSub.addAmount(receivableIncomeBelong.getWriteOffAmount().multiply(boxReceivable.getDestCountrySubRate() != null ? boxReceivable.getDestCountrySubRate() : otherSubRate));
                            } else if (receivableIncomeBelong.getIncomeBelong() == 0) {
                                receivableBoxAmount.receivableDestCountryOtherFee.addAmount(receivableIncomeBelong.getWriteOffAmount().multiply(boxReceivable.getDestCountryRate() != null ? boxReceivable.getDestCountryRate() : otherFeeRate));
                                receivableBoxAmount.receivableDestCountryOtherFeeSub.addAmount(receivableIncomeBelong.getWriteOffAmount().multiply(boxReceivable.getDestCountrySubRate() != null ? boxReceivable.getDestCountrySubRate() : otherSubRate));
                            }
                        }
                    }
                    break;
                default:
                    break;
            }
        }
        return receivableBoxAmount;
    }

    private void mergeAmount(ReceivableBoxReportVO.Amount resAmount, ReceivableBoxAmount receivableBoxAmount) {
        List<ReceivableCurrencyAmount> pageReceivableTotalSummary = resAmount.getReceivableTotalSummary();
        for (ReceivableCurrencyAmount receivableCurrencyAmount : receivableBoxAmount.receivableTotalSummary) {
            if (pageReceivableTotalSummary.contains(receivableCurrencyAmount)) {
                pageReceivableTotalSummary.get(pageReceivableTotalSummary.indexOf(receivableCurrencyAmount)).addAmount(receivableCurrencyAmount.getAmount());
            } else {
                pageReceivableTotalSummary.add(receivableCurrencyAmount.clone());
            }
        }
        List<ReceivableCurrencyAmount> pageReceivableTotalSettle = resAmount.getReceivableTotalSettle();
        if (pageReceivableTotalSettle.contains(receivableBoxAmount.receivableTotalSettle)) {
            pageReceivableTotalSettle.get(pageReceivableTotalSettle.indexOf(receivableBoxAmount.receivableTotalSettle)).addAmount(receivableBoxAmount.receivableTotalSettle.getAmount());
        } else {
            pageReceivableTotalSettle.add(receivableBoxAmount.receivableTotalSettle.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableDetailFreight = resAmount.getReceivableDetailFreight();
        if (pageReceivableDetailFreight.contains(receivableBoxAmount.receivableDetailFreight)) {
            pageReceivableDetailFreight.get(pageReceivableDetailFreight.indexOf(receivableBoxAmount.receivableDetailFreight)).addAmount(receivableBoxAmount.receivableDetailFreight.getAmount());
        } else {
            pageReceivableDetailFreight.add(receivableBoxAmount.receivableDetailFreight.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableDetailClearance = resAmount.getReceivableDetailClearance();
        if (pageReceivableDetailClearance.contains(receivableBoxAmount.receivableDetailClearance)) {
            pageReceivableDetailClearance.get(pageReceivableDetailClearance.indexOf(receivableBoxAmount.receivableDetailClearance)).addAmount(receivableBoxAmount.receivableDetailClearance.getAmount());
        } else {
            pageReceivableDetailClearance.add(receivableBoxAmount.receivableDetailClearance.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableDetailOtherFee = resAmount.getReceivableDetailOtherFee();
        if (pageReceivableDetailOtherFee.contains(receivableBoxAmount.receivableDetailOtherFee)) {
            pageReceivableDetailOtherFee.get(pageReceivableDetailOtherFee.indexOf(receivableBoxAmount.receivableDetailOtherFee)).addAmount(receivableBoxAmount.receivableDetailOtherFee.getAmount());
        } else {
            pageReceivableDetailOtherFee.add(receivableBoxAmount.receivableDetailOtherFee.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableDestCountryFreight = resAmount.getReceivableDestCountryFreight();
        if (pageReceivableDestCountryFreight.contains(receivableBoxAmount.receivableDestCountryFreight)) {
            pageReceivableDestCountryFreight.get(pageReceivableDestCountryFreight.indexOf(receivableBoxAmount.receivableDestCountryFreight)).addAmount(receivableBoxAmount.receivableDestCountryFreight.getAmount());
        } else {
            pageReceivableDestCountryFreight.add(receivableBoxAmount.receivableDestCountryFreight.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableDestCountryClearance = resAmount.getReceivableDestCountryClearance();
        if (pageReceivableDestCountryClearance.contains(receivableBoxAmount.receivableDestCountryClearance)) {
            pageReceivableDestCountryClearance.get(pageReceivableDestCountryClearance.indexOf(receivableBoxAmount.receivableDestCountryClearance)).addAmount(receivableBoxAmount.receivableDestCountryClearance.getAmount());
        } else {
            pageReceivableDestCountryClearance.add(receivableBoxAmount.receivableDestCountryClearance.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableDestCountryOtherFee = resAmount.getReceivableDestCountryOtherFee();
        if (pageReceivableDestCountryOtherFee.contains(receivableBoxAmount.receivableDestCountryOtherFee)) {
            pageReceivableDestCountryOtherFee.get(pageReceivableDestCountryOtherFee.indexOf(receivableBoxAmount.receivableDestCountryOtherFee)).addAmount(receivableBoxAmount.receivableDestCountryOtherFee.getAmount());
        } else {
            pageReceivableDestCountryOtherFee.add(receivableBoxAmount.receivableDestCountryOtherFee.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableDestCountryOtherFeeSub = resAmount.getReceivableDestCountryOtherFeeSub();
        if (pageReceivableDestCountryOtherFeeSub.contains(receivableBoxAmount.receivableDestCountryOtherFeeSub)) {
            pageReceivableDestCountryOtherFeeSub.get(pageReceivableDestCountryOtherFeeSub.indexOf(receivableBoxAmount.receivableDestCountryOtherFeeSub)).addAmount(receivableBoxAmount.receivableDestCountryOtherFeeSub.getAmount());
        } else {
            pageReceivableDestCountryOtherFeeSub.add(receivableBoxAmount.receivableDestCountryOtherFeeSub.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableStartCountryFreight = resAmount.getReceivableStartCountryFreight();
        if (pageReceivableStartCountryFreight.contains(receivableBoxAmount.receivableStartCountryFreight)) {
            pageReceivableStartCountryFreight.get(pageReceivableStartCountryFreight.indexOf(receivableBoxAmount.receivableStartCountryFreight)).addAmount(receivableBoxAmount.receivableStartCountryFreight.getAmount());
        } else {
            pageReceivableStartCountryFreight.add(receivableBoxAmount.receivableStartCountryFreight.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableStartCountryClearance = resAmount.getReceivableStartCountryClearance();
        if (pageReceivableStartCountryClearance.contains(receivableBoxAmount.receivableStartCountryClearance)) {
            pageReceivableStartCountryClearance.get(pageReceivableStartCountryClearance.indexOf(receivableBoxAmount.receivableStartCountryClearance)).addAmount(receivableBoxAmount.receivableStartCountryClearance.getAmount());
        } else {
            pageReceivableStartCountryClearance.add(receivableBoxAmount.receivableStartCountryClearance.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableStartCountryOtherFee = resAmount.getReceivableStartCountryOtherFee();
        if (pageReceivableStartCountryOtherFee.contains(receivableBoxAmount.receivableStartCountryOtherFee)) {
            pageReceivableStartCountryOtherFee.get(pageReceivableStartCountryOtherFee.indexOf(receivableBoxAmount.receivableStartCountryOtherFee)).addAmount(receivableBoxAmount.receivableStartCountryOtherFee.getAmount());
        } else {
            pageReceivableStartCountryOtherFee.add(receivableBoxAmount.receivableStartCountryOtherFee.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableStartCountryOtherFeeSub = resAmount.getReceivableStartCountryOtherFeeSub();
        if (pageReceivableStartCountryOtherFeeSub.contains(receivableBoxAmount.receivableStartCountryOtherFeeSub)) {
            pageReceivableStartCountryOtherFeeSub.get(pageReceivableStartCountryOtherFeeSub.indexOf(receivableBoxAmount.receivableStartCountryOtherFeeSub)).addAmount(receivableBoxAmount.receivableStartCountryOtherFeeSub.getAmount());
        } else {
            pageReceivableStartCountryOtherFeeSub.add(receivableBoxAmount.receivableStartCountryOtherFeeSub.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableDiscountFreight = resAmount.getReceivableDiscountFreight();
        if (pageReceivableDiscountFreight.contains(receivableBoxAmount.receivableDiscountFreight)) {
            pageReceivableDiscountFreight.get(pageReceivableDiscountFreight.indexOf(receivableBoxAmount.receivableDiscountFreight)).addAmount(receivableBoxAmount.receivableDiscountFreight.getAmount());
        } else {
            pageReceivableDiscountFreight.add(receivableBoxAmount.receivableDiscountFreight.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableDiscountClearance = resAmount.getReceivableDiscountClearance();
        if (pageReceivableDiscountClearance.contains(receivableBoxAmount.receivableDiscountClearance)) {
            pageReceivableDiscountClearance.get(pageReceivableDiscountClearance.indexOf(receivableBoxAmount.receivableDiscountClearance)).addAmount(receivableBoxAmount.receivableDiscountClearance.getAmount());
        } else {
            pageReceivableDiscountClearance.add(receivableBoxAmount.receivableDiscountClearance.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableDiscountOtherFee = resAmount.getReceivableDiscountOtherFee();
        if (pageReceivableDiscountOtherFee.contains(receivableBoxAmount.receivableDiscountOtherFee)) {
            pageReceivableDiscountOtherFee.get(pageReceivableDiscountOtherFee.indexOf(receivableBoxAmount.receivableDiscountOtherFee)).addAmount(receivableBoxAmount.receivableDiscountOtherFee.getAmount());
        } else {
            pageReceivableDiscountOtherFee.add(receivableBoxAmount.receivableDiscountOtherFee.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableUnsettledFreight = resAmount.getReceivableUnsettledFreight();
        if (pageReceivableUnsettledFreight.contains(receivableBoxAmount.receivableUnsettledFreight)) {
            pageReceivableUnsettledFreight.get(pageReceivableUnsettledFreight.indexOf(receivableBoxAmount.receivableUnsettledFreight)).addAmount(receivableBoxAmount.receivableUnsettledFreight.getAmount());
        } else {
            pageReceivableUnsettledFreight.add(receivableBoxAmount.receivableUnsettledFreight.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableUnsettledClearance = resAmount.getReceivableUnsettledClearance();
        if (pageReceivableUnsettledClearance.contains(receivableBoxAmount.receivableUnsettledClearance)) {
            pageReceivableUnsettledClearance.get(pageReceivableUnsettledClearance.indexOf(receivableBoxAmount.receivableUnsettledClearance)).addAmount(receivableBoxAmount.receivableUnsettledClearance.getAmount());
        } else {
            pageReceivableUnsettledClearance.add(receivableBoxAmount.receivableUnsettledClearance.clone());
        }
        List<ReceivableCurrencyAmount> pageReceivableUnsettledOtherFee = resAmount.getReceivableUnsettledOtherFee();
        if (pageReceivableUnsettledOtherFee.contains(receivableBoxAmount.receivableUnsettledOtherFee)) {
            pageReceivableUnsettledOtherFee.get(pageReceivableUnsettledOtherFee.indexOf(receivableBoxAmount.receivableUnsettledOtherFee)).addAmount(receivableBoxAmount.receivableUnsettledOtherFee.getAmount());
        } else {
            pageReceivableUnsettledOtherFee.add(receivableBoxAmount.receivableUnsettledOtherFee.clone());
        }
    }

    /**
     * 整合单个record的字符串结果,用于前端展示
     */
    private void assembleRecord(ReceivableBoxAmount receivableBoxAmount, ReceivableBoxReportPageVO record, Map<Integer, CurrencyRespDTO> allCurrency) {
        StringBuilder receivableCurrencyAmountBuilder = new StringBuilder();
        for (ReceivableCurrencyAmount receivableCurrencyAmount : receivableBoxAmount.receivableTotalSummary) {
            receivableCurrencyAmountBuilder.append(allCurrency.get(receivableCurrencyAmount.getCurrencyId().intValue()).getFuhao()).append(" ").append(receivableCurrencyAmount.getAmount()).append(", ");
        }
        record.setReceivableTotalSummary(receivableCurrencyAmountBuilder.substring(0, receivableCurrencyAmountBuilder.length() - 2));
        record.setReceivableTotalSettle(allCurrency.get(receivableBoxAmount.receivableTotalSettle.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableTotalSettle.getAmount());
        record.setReceivableDetailFreight(allCurrency.get(receivableBoxAmount.receivableDetailFreight.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableDetailFreight.getAmount());
        record.setReceivableDetailClearance(allCurrency.get(receivableBoxAmount.receivableDetailClearance.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableDetailClearance.getAmount());
        record.setReceivableDetailOtherFee(allCurrency.get(receivableBoxAmount.receivableDetailOtherFee.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableDetailOtherFee.getAmount());
        record.setReceivableDestCountryFreight(allCurrency.get(receivableBoxAmount.receivableDestCountryFreight.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableDestCountryFreight.getAmount());
        record.setReceivableDestCountryClearance(allCurrency.get(receivableBoxAmount.receivableDestCountryClearance.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableDestCountryClearance.getAmount());
        record.setReceivableDestCountryOtherFee(allCurrency.get(receivableBoxAmount.receivableDestCountryOtherFee.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableDestCountryOtherFee.getAmount());
        record.setReceivableDestCountryOtherFeeSub(allCurrency.get(receivableBoxAmount.receivableDestCountryOtherFeeSub.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableDestCountryOtherFeeSub.getAmount());
        record.setReceivableStartCountryFreight(allCurrency.get(receivableBoxAmount.receivableStartCountryFreight.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableStartCountryFreight.getAmount());
        record.setReceivableStartCountryClearance(allCurrency.get(receivableBoxAmount.receivableStartCountryClearance.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableStartCountryClearance.getAmount());
        record.setReceivableStartCountryOtherFee(allCurrency.get(receivableBoxAmount.receivableStartCountryOtherFee.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableStartCountryOtherFee.getAmount());
        record.setReceivableStartCountryOtherFeeSub(allCurrency.get(receivableBoxAmount.receivableStartCountryOtherFeeSub.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableStartCountryOtherFeeSub.getAmount());
        record.setReceivableDiscountFreight(allCurrency.get(receivableBoxAmount.receivableDiscountFreight.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableDiscountFreight.getAmount());
        record.setReceivableDiscountClearance(allCurrency.get(receivableBoxAmount.receivableDiscountClearance.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableDiscountClearance.getAmount());
        record.setReceivableDiscountOtherFee(allCurrency.get(receivableBoxAmount.receivableDiscountOtherFee.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableDiscountOtherFee.getAmount());
        record.setReceivableUnsettledFreight(allCurrency.get(receivableBoxAmount.receivableUnsettledFreight.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableUnsettledFreight.getAmount());
        record.setReceivableUnsettledClearance(allCurrency.get(receivableBoxAmount.receivableUnsettledClearance.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableUnsettledClearance.getAmount());
        record.setReceivableUnsettledOtherFee(allCurrency.get(receivableBoxAmount.receivableUnsettledOtherFee.getCurrencyId().intValue()).getFuhao() + " " + receivableBoxAmount.receivableUnsettledOtherFee.getAmount());

        String writeOffPercent = getWriteOffPercent(receivableBoxAmount);
        record.setWriteOffPercent(writeOffPercent);
    }

    private static @NotNull String getWriteOffPercent(ReceivableBoxAmount receivableBoxAmount) {
        if (receivableBoxAmount.getReceivableBaseCurrency() == null || receivableBoxAmount.getReceivableBaseCurrencySettle() == null) {
            return "0.00%";
        }
        BigDecimal receivableBaseCurrency = receivableBoxAmount.getReceivableBaseCurrency().getAmount();
        BigDecimal receivableBaseCurrencySettle = receivableBoxAmount.getReceivableBaseCurrencySettle().getAmount();
        String writeOffPercent;
        if (receivableBaseCurrency.compareTo(BigDecimal.ZERO) == 0 || receivableBaseCurrencySettle.compareTo(BigDecimal.ZERO) == 0) {
            writeOffPercent = "0.00%";
        } else {
            writeOffPercent = receivableBaseCurrencySettle.divide(receivableBaseCurrency, 4, RoundingMode.HALF_UP).multiply(new BigDecimal("100")) + "%";
        }
        return writeOffPercent;
    }

    /**
     * 获取汇率
     */
    private BigDecimal getExchangeRate(List<ExchangeRateRespDTO> allCurrencyRate, Long sourceId, Long targetId) {
        if (sourceId.equals(targetId)) {
            return BigDecimal.ONE;
        }
        for (ExchangeRateRespDTO exchangeRateRespDTO : allCurrencyRate) {
            if (exchangeRateRespDTO.getSourceCurrencyId().equals(sourceId) && exchangeRateRespDTO.getTargetCurrencyId().equals(targetId)) {
                return exchangeRateRespDTO.getCurrencyRate();
            }
        }
        return BigDecimal.ONE;
    }

    /**
     * 应收报表中费用分类
     *
     * @param feeType
     * @return
     */
    private Integer getFeeType(Integer feeType) {
        if (feeType == 1) {
            return 0;
        }
        if (feeType == 2) {
            return 1;
        }
        return 2;
    }

    /**
     * 设置提单状态
     *
     * @param record
     */
    private void setLadingBillStatus(ReceivableBoxReportPageVO record) {
        record.setLadingBillStatus(0);
        if (record.getOrderCount() != 0) {
            if (record.getLadingCount() > 0) {
                if (record.getOrderCount().equals(record.getLadingCount())) {
                    record.setLadingBillStatus(2);
                } else if (record.getOrderCount() < record.getLadingCount()) {
                    // 添加的逻辑，提单有可能补单
                    record.setLadingBillStatus(2);
                } else {
                    record.setLadingBillStatus(1);
                }
            }
        }
    }

    @Data
    @NoArgsConstructor
    private static class ReceivableBoxAmount {
        // 应收款
        private ArrayList<ReceivableCurrencyAmount> receivableTotalSummary = new ArrayList<>();
        private ReceivableCurrencyAmount receivableTotalSettle;
        // 应收款明细
        private ReceivableCurrencyAmount receivableDetailFreight;
        private ReceivableCurrencyAmount receivableDetailClearance;
        private ReceivableCurrencyAmount receivableDetailOtherFee;
        // 折扣
        private ReceivableCurrencyAmount receivableDiscountFreight;
        private ReceivableCurrencyAmount receivableDiscountClearance;
        private ReceivableCurrencyAmount receivableDiscountOtherFee;
        // 未收款
        private ReceivableCurrencyAmount receivableUnsettledFreight;
        private ReceivableCurrencyAmount receivableUnsettledClearance;
        private ReceivableCurrencyAmount receivableUnsettledOtherFee;
        // 始发国实收
        private ReceivableCurrencyAmount receivableStartCountryFreight;
        private ReceivableCurrencyAmount receivableStartCountryClearance;
        private ReceivableCurrencyAmount receivableStartCountryOtherFee;
        private ReceivableCurrencyAmount receivableStartCountryOtherFeeSub;
        // 目国实收
        private ReceivableCurrencyAmount receivableDestCountryFreight;
        private ReceivableCurrencyAmount receivableDestCountryClearance;
        private ReceivableCurrencyAmount receivableDestCountryOtherFee;
        private ReceivableCurrencyAmount receivableDestCountryOtherFeeSub;

        // 基准币种应收
        private ReceivableCurrencyAmount receivableBaseCurrency;
        // 基准币种实收
        private ReceivableCurrencyAmount receivableBaseCurrencySettle;

        public ReceivableBoxAmount(Long freightCurrency, Long clearanceCurrency, Long otherFeeCurrency, Long otherSubCurrency, Long receivableCurrency) {
            this.receivableTotalSummary = new ArrayList<>();
            this.receivableTotalSettle = new ReceivableCurrencyAmount(receivableCurrency, BigDecimal.ZERO);
            this.receivableDetailFreight = new ReceivableCurrencyAmount(freightCurrency, BigDecimal.ZERO);
            this.receivableDetailClearance = new ReceivableCurrencyAmount(clearanceCurrency, BigDecimal.ZERO);
            this.receivableDetailOtherFee = new ReceivableCurrencyAmount(otherFeeCurrency, BigDecimal.ZERO);
            this.receivableDiscountFreight = new ReceivableCurrencyAmount(freightCurrency, BigDecimal.ZERO);
            this.receivableDiscountClearance = new ReceivableCurrencyAmount(clearanceCurrency, BigDecimal.ZERO);
            this.receivableDiscountOtherFee = new ReceivableCurrencyAmount(otherFeeCurrency, BigDecimal.ZERO);
            this.receivableUnsettledFreight = new ReceivableCurrencyAmount(freightCurrency, BigDecimal.ZERO);
            this.receivableUnsettledClearance = new ReceivableCurrencyAmount(clearanceCurrency, BigDecimal.ZERO);
            this.receivableUnsettledOtherFee = new ReceivableCurrencyAmount(otherFeeCurrency, BigDecimal.ZERO);
            this.receivableStartCountryFreight = new ReceivableCurrencyAmount(freightCurrency, BigDecimal.ZERO);
            this.receivableStartCountryClearance = new ReceivableCurrencyAmount(clearanceCurrency, BigDecimal.ZERO);
            this.receivableStartCountryOtherFee = new ReceivableCurrencyAmount(otherFeeCurrency, BigDecimal.ZERO);
            this.receivableStartCountryOtherFeeSub = new ReceivableCurrencyAmount(otherSubCurrency, BigDecimal.ZERO);
            this.receivableDestCountryFreight = new ReceivableCurrencyAmount(freightCurrency, BigDecimal.ZERO);
            this.receivableDestCountryClearance = new ReceivableCurrencyAmount(clearanceCurrency, BigDecimal.ZERO);
            this.receivableDestCountryOtherFee = new ReceivableCurrencyAmount(otherFeeCurrency, BigDecimal.ZERO);
            this.receivableDestCountryOtherFeeSub = new ReceivableCurrencyAmount(otherSubCurrency, BigDecimal.ZERO);

            this.receivableBaseCurrency = new ReceivableCurrencyAmount(receivableCurrency, BigDecimal.ZERO);
            this.receivableBaseCurrencySettle = new ReceivableCurrencyAmount(receivableCurrency, BigDecimal.ZERO);
        }
    }
}