package cn.iocoder.yudao.module.wealth.vo.receipt;

import cn.iocoder.yudao.module.wealth.vo.receiptInvoice.InvoiceInReceiptRespVO;
import cn.iocoder.yudao.module.wealth.vo.receiptInvoice.ReceiptInvoiceBaseVO;
import com.alibaba.excel.annotation.ExcelProperty;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

/**
 * 收款单 Response VO
 *
 * @author 郑屹
 */
@Data
@ApiModel("管理后台 - 收款单 Response VO")
public class ReceiptBackVO {

    @ExcelProperty("主键")
    @ApiModelProperty(value = "主键", required = true)
    private Long id;

    @ExcelProperty("收款单号（例：SKD001）")
    @ApiModelProperty(value = "收款单号（例：SKD001）", required = true)
    private String receiptNo;

    @ExcelProperty("部门Id")
    @ApiModelProperty(value = "部门Id")
    private Long departmentId;

    @ExcelProperty("部门名称")
    @ApiModelProperty(value = "部门名称")
    private String departmentName;

    @ExcelProperty("业务员")
    @ApiModelProperty(value = "业务员")
    private Long salesmanId;

    @ExcelProperty("业务员名称")
    @ApiModelProperty(value = "业务员名称")
    private String salesmanName;

    @ExcelProperty("客户id")
    @ApiModelProperty(value = "客户id")
    private Long customerId;

    @ExcelProperty("客户名称")
    @ApiModelProperty(value = "客户名称")
    private String customerName;

    @ExcelProperty("备注")
    @ApiModelProperty(value = "备注")
    private String remark;

    @ExcelProperty("核销币种")
    @ApiModelProperty(value = "核销币种")
    private Long currencyId;

    @ApiModelProperty(value = "核销比例显示要除以100")
    private BigDecimal writeOffProportion;

    @ApiModelProperty(value = "核销金额")
    private Long writeOffAmount;

    @ExcelProperty("应收总金额")
    @ApiModelProperty(value = "应收总金额")
    private java.math.BigDecimal receivableTotalAmount;

    @ExcelProperty("手续费")
    @ApiModelProperty(value = "手续费")
    private java.math.BigDecimal feeRate;

    @ExcelProperty("收款人id")
    @ApiModelProperty(value = "收款人id")
    private Long payeeId;

    @ExcelProperty("收款人名称")
    @ApiModelProperty(value = "收款人名称")
    private String payeeName;

    @ExcelProperty("核销人id")
    @ApiModelProperty(value = "核销人id")
    private Long writeOffId;

    @ExcelProperty("核销人名称")
    @ApiModelProperty(value = "核销人名称")
    private String writeOffName;

    @ExcelProperty("开票人id")
    @ApiModelProperty(value = "开票人id")
    private Long issuerId;

    @ExcelProperty("开票人名称")
    @ApiModelProperty(value = "开票人名称")
    private String issuerName;

    @ExcelProperty("状态: 0:草稿， 1待收款， 2待核销，3已部分核销，4已核销，5已核销待开票，6已开票 7审核中 8审核失败 9反审核中 10全部核销中 11反核销审核中")
    @ApiModelProperty(value = "状态: 0:草稿， 1待收款， 2待核销，3已部分核销，4已核销，5已核销待开票，6已开票 7审核中 8审核失败 9反审核中 10全部核销中 11反核销审核中")
    private Integer state;

    @ExcelProperty("核销时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "核销时间")
    private Date writeOffAt;

    @ExcelProperty("数据来源（pc端，mobile端）")
    @ApiModelProperty(value = "数据来源（pc端，mobile端）")
    private String source;

    @ExcelProperty("是否需要开票")
    @ApiModelProperty(value = "是否需要开票0-不需要1需要")
    private Integer openInvoice;

    @ExcelProperty("汇率有效期")
    @ApiModelProperty(value = "汇率有效期")
    private Date rateValidateDate;

    @ExcelProperty("当前流程ID")
    @ApiModelProperty(value = "当前流程ID")
    private String bmpId;

    @ApiModelProperty(value = "驳回原因")
    private String comment;

    @ApiModelProperty(value = "财务备注")
    private String financeRemark;

    @ExcelProperty("明细状态")
    @ApiModelProperty(value = "明细状态")
    private String mxStatus;

    @ApiModelProperty(value = "应收金额")
    private String ysShow;

    @ApiModelProperty(value = "实数金额")
    private String ssShow;

    @ApiModelProperty(value = "核销比例")
    private String hxBiLi;

    @ApiModelProperty(value = "明细状态（数据库）")
    private Integer receiptItemStatus;

    @ApiModelProperty(value = "开票状态")
    private Integer invoicingStatus;

    @ApiModelProperty(value = "开票资料状态")
    private Integer invoiceDataStatus;

    @ApiModelProperty(value = "开票id")
    private Long invoiceId;

    @ApiModelProperty(value = "生成路径")
    private Integer generatePath;

    @ApiModelProperty(value = "开票信息")
    private InvoiceInReceiptRespVO invoiceVO;

    @ApiModelProperty(value = "相关订单信息列表")
    private List<ReceiptRelationOrderInfoVO> orderInfoList;

    @ApiModelProperty(value = "收款单日志列表")
    private List<ReceiptLogBackVO> logList;

    @ApiModelProperty(value = "创建时间")
    private Date createTime;
    @ApiModelProperty(value = "更新时间")
    private Date updateTime;
    @ApiModelProperty(value = "创建人")
    private String creator;
    @ApiModelProperty(value = "更新人")
    private String updater;

    @ApiModelProperty(value = "实收日期")
    private Date payedAt;

    @ApiModelProperty(value = "核销误差兑额外费用主币种金额")
    private String writeOffDiffMainCurrency;
}
