package cn.iocoder.yudao.module.wealth.vo.receipt;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

/**
 * 收款单 Base VO，提供给添加、修改、详细的子 VO 使用
 * 如果子 VO 存在差异的字段，请不要添加到这里，影响 Swagger 文档生成
 */
@Data
public class ReceiptBaseVO {

    @ApiModelProperty(value = "收款单号（例：SKD001）")
    private String receiptNo;

    @ApiModelProperty(value = "部门Id")
    private Long departmentId;

    @ApiModelProperty(value = "部门名称")
    private String departmentName;

    @ApiModelProperty(value = "业务员")
    @NotNull(message = "业务员不能为空")
    private Long salesmanId;

    @ApiModelProperty(value = "业务员名称")
    @NotBlank(message = "业务员不能为空")
    private String salesmanName;

    @ApiModelProperty(value = "客户id")
    @NotNull(message = "客户不能为空")
    private Long customerId;

    @ApiModelProperty(value = "客户名称")
    @NotBlank(message = "客户不能为空")
    private String customerName;

    @ApiModelProperty(value = "备注")
    private String remark;

    @ApiModelProperty(value = "核销币种")
    //@NotNull(message = "核销币种不能为空")
    private Long currencyId;

    @ApiModelProperty(value = "核销比例传值要乘以100")
    private BigDecimal writeOffProportion;

    @ApiModelProperty(value = "核销金额")
    private Long writeOffAmount;

    @ApiModelProperty(value = "应收总金额")
    private java.math.BigDecimal receivableTotalAmount;

    @ApiModelProperty(value = "手续费")
    private BigDecimal feeRate;

    //@ApiModelProperty(value = "发票抬头")
    //private String invoice;

    //@ApiModelProperty(value = "纳税人识别号")
    //private String taxpayer;

    //@ApiModelProperty(value = "开户行")
    //private String accountBank;

    //@ApiModelProperty(value = "账号")
    //private String accountName;

    //@ApiModelProperty(value = "项目（目前原型叫开票品名）")
    //private String projectName;

    //@ApiModelProperty(value = "税率")
    //private BigDecimal taxRate;

    //@ApiModelProperty(value = "开票地址、电话")
    //private String addressPhone;

    @ApiModelProperty(value = "实收日期")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date payedAt;

    //@ApiModelProperty(value = "发票号码")
    //private String invoiceNumber;
    //
    //@ApiModelProperty(value = "开票类型(字典表 invoicing_type)")
    //private Integer invoicingTypeId;
    //
    //@ApiModelProperty(value = "开票类型名称")
    //private String invoicingType;
    //
    //@ApiModelProperty(value = "开票备注")
    //private String invoicingRemark;

    @ApiModelProperty(value = "通知人id")
    private Long noticeAdminId;

    @ApiModelProperty(value = "通知人名称")
    private String noticeAdminName;

    //@ApiModelProperty(value = "收款人id")
    //private Long payeeId;
    //
    //@ApiModelProperty(value = "收款人名称")
    //private String payeeName;
    //
    @ApiModelProperty(value = "核销人id")
    private Long writeOffId;

    @ApiModelProperty(value = "核销人名称")
    private String writeOffName;
    //
    //@ApiModelProperty(value = "开票人id")
    //private Long issuerId;
    //
    //@ApiModelProperty(value = "开票人名称")
    //private String issuerName;

    @ApiModelProperty(value = "状态: 0:草稿， 1待收款， 2待核销，3已部分核销，4已核销，5已核销待开票，6已开票")
    private Integer state;

    @ApiModelProperty(value = "核销时间")
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date writeOffAt;

    //@ApiModelProperty(value = "开票时间")
    //@DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    //private Date invoicingAt;

    @ApiModelProperty(value = "数据来源（pc端，mobile端）")
    private String source;

    @ApiModelProperty(value = "是否需要开票0-不需要1需要")
    private Integer openInvoice;

    @ApiModelProperty(value = "汇率有效期")
    private Date rateValidateDate;

    @ApiModelProperty(value = "税点")
    private BigDecimal taxPoint;

    @ApiModelProperty(value = "财务备注")
    private String financeRemark;

    @ApiModelProperty(value = "生成路径")
    private Integer generatePath;
}
