package cn.iocoder.yudao.module.wealth.vo.receiptItem;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

@Data
@ApiModel("分页结果")
public final class ReceiptItemReviewPageResult<T> implements Serializable {

    @ApiModelProperty(value = "数据", required = true)
    private List<T> list;

    @ApiModelProperty(value = "总量", required = true, example = "0")
    private Long total = 0L;

    @ApiModelProperty(value = "每页记录数", required = true, example = "10")
    private Long rows = 10L;

    @ApiModelProperty(value = "当前页数", required = true, example = "1")
    private Long page = 1L;

    @ApiModelProperty(value = "总页数", required = true, example = "0")
    private Long pages = 0L;

    @ApiModelProperty(value = "当前查询实收")
    private List<CurrencyAmount> actualAmount;

    @ApiModelProperty(value = "总实收")
    List<CurrencyAmount> totalActualAmount;

    @ApiModelProperty(value = "结算总合计")
    List<CurrencyAmount> totalWriteOffAmount;

    public ReceiptItemReviewPageResult() {}

    public ReceiptItemReviewPageResult(List<T> list, long total) {
        this.list = list;
        this.total = total;
    }

    public ReceiptItemReviewPageResult(List<T> list, long total, long rows, long page, long pages) {
        this.list = list;
        this.rows = rows;
        this.page = page;
        this.pages = pages;
        this.total = total;
    }

    public ReceiptItemReviewPageResult(long total) {
        this.list = new ArrayList<>();
        this.total = total;
    }

    public static <T> ReceiptItemReviewPageResult<T> empty() {
        return new ReceiptItemReviewPageResult<>();
    }

    public static <T> ReceiptItemReviewPageResult<T> empty(long total) {
        return new ReceiptItemReviewPageResult<>(total);
    }

    /****
     * 分页转换
     * @param pager
     * @return
     */
    public static <T> ReceiptItemReviewPageResult<T> of(IPage<T> pager){
        return new ReceiptItemReviewPageResult<>(pager.getRecords(), pager.getTotal(), pager.getSize(), pager.getCurrent(), pager.getPages());
    }

    /****
     * 分页转换
     * @param pager
     * @return
     */
    public static <T> ReceiptItemReviewPageResult<T> of(PageInfo<T> pager){
        return new ReceiptItemReviewPageResult<>(pager.getList(), pager.getTotal(), pager.getPageSize(), pager.getPageNum(), pager.getPages());
    }
}
