package cn.iocoder.yudao.module.wealth.vo.receivable;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@Data
public class ReceivableBoxReportQueryVO {

    @ApiModelProperty(value = "自编号")
    private String selfNo;

    @ApiModelProperty(value = "始发地id列表")
    private List<Long> startWarehouseIdList;

    @ApiModelProperty(value = "目的国家ID")
    private List<Long> destCountryId;

    @ApiModelProperty(value = "目的城市ID")
    private List<Long> destCity;

    @ApiModelProperty(value = "目的仓库id列表")
    private List<Long> destWarehouseIdList;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始结算时间")
    private Date beginJsDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束结算时间")
    private Date endJsDate;

    @ApiModelProperty(value = "191-未结算，192-结算中，193-已结算")
    private Integer slStatus;

    @ApiModelProperty(value = "出货状态")
    private List<Integer> boxStatus;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始到港时间")
    private Date beginDaogangTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束到港时间")
    private Date endDaogangTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始清关时间")
    private Date beginQingguanTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束清关时间")
    private Date endQingguanTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始卸柜/到仓时间")
    private Date  beginUlWarehouseTime ;


    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束卸柜/到仓时间")
    private Date  endUlWarehouseTime ;

    @ApiModelProperty(value = "提单制作,0-未完成，1-部分完成，2-已完成")
    private Integer ladingBillStatus;

    @ApiModelProperty(value = "结算时间排序(默认倒序)")
    private Boolean jsTimeDesc = true;
    @ApiModelProperty(value = "卸柜/到仓时间排序(默认倒序)")
    private Boolean ulWarehouseTimeDesc = true;
    @ApiModelProperty(value = "清关时间排序(默认倒序)")
    private Boolean qingguanTimeDesc = true;
    @ApiModelProperty(value = "到港时间排序(默认倒序)")
    private Boolean daogangTimeDesc = true;

    private Boolean forceUpdateCache = false;

    private List<String> selfNoList;

    public Boolean isEmpty() {
        return StringUtils.isBlank(selfNo)
                && (CollectionUtil.isEmpty(startWarehouseIdList))
                && (CollectionUtil.isEmpty(destCountryId))
                && (CollectionUtil.isEmpty(destCity))
                && (CollectionUtil.isEmpty(destWarehouseIdList))
                && (beginJsDate == null)
                && (endJsDate == null)
                && (slStatus == null)
                && (boxStatus == null)
                && (beginDaogangTime == null)
                && (endDaogangTime == null)
                && (beginQingguanTime == null)
                && (endQingguanTime == null)
                && (beginUlWarehouseTime == null)
                && (endUlWarehouseTime == null)
                && (ladingBillStatus == null);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        ReceivableBoxReportQueryVO that = (ReceivableBoxReportQueryVO) o;
        return Objects.equals(selfNo, that.selfNo) &&
                // 比较两个list中的元素相同
                CollectionUtil.isEqualList(startWarehouseIdList, that.startWarehouseIdList) &&
                CollectionUtil.isEqualList(destCountryId, that.destCountryId) &&
                CollectionUtil.isEqualList(destCity, that.destCity) &&
                CollectionUtil.isEqualList(destWarehouseIdList, that.destWarehouseIdList) &&
                Objects.equals(beginJsDate, that.beginJsDate) && Objects.equals(endJsDate, that.endJsDate) && Objects.equals(slStatus, that.slStatus) && Objects.equals(boxStatus, that.boxStatus) && Objects.equals(beginDaogangTime, that.beginDaogangTime) && Objects.equals(endDaogangTime, that.endDaogangTime) && Objects.equals(beginQingguanTime, that.beginQingguanTime) && Objects.equals(endQingguanTime, that.endQingguanTime) && Objects.equals(beginUlWarehouseTime, that.beginUlWarehouseTime) && Objects.equals(endUlWarehouseTime, that.endUlWarehouseTime) && Objects.equals(ladingBillStatus, that.ladingBillStatus) && Objects.equals(jsTimeDesc, that.jsTimeDesc) && Objects.equals(ulWarehouseTimeDesc, that.ulWarehouseTimeDesc) && Objects.equals(qingguanTimeDesc, that.qingguanTimeDesc) && Objects.equals(daogangTimeDesc, that.daogangTimeDesc);
    }

    @Override
    public int hashCode() {
        return Objects.hash(selfNo, startWarehouseIdList, destCountryId, destCity, destWarehouseIdList, beginJsDate, endJsDate, slStatus, boxStatus, beginDaogangTime, endDaogangTime, beginQingguanTime, endQingguanTime, beginUlWarehouseTime, endUlWarehouseTime, ladingBillStatus, jsTimeDesc, ulWarehouseTimeDesc, qingguanTimeDesc, daogangTimeDesc);
    }
}
